/*
 * Deferred (extracted from jQuery) v0.1
 *
 * Copyright 2011, Mykhaylo Gavrylyuk
 * Licensed under the MIT license
 *
 * Date: Sun Aug 28 07:03:08 EEST 2011
 */


(function (undefined) {

var promiseMethods = "then done fail isResolved isRejected promise".split(" ");

function getType(obj) {
  return obj == null ? String(obj) : class2type[Object.prototype.toString.call(obj)] || "object";
}

var class2type = {};

// Populate the class2type map
var classNames = "Boolean Number String Function Array Date RegExp Object".split(" ");

for (var i in classNames) {
  var name = classNames[i];
	class2type[ "[object " + name + "]" ] = name.toLowerCase();
}

function isFunction(obj) {
  return getType(obj) === "function";
}

function isArray(obj) {
  return getType(obj) === "array";
}

function isPlainObject(obj) {
  // Must be an Object.
  // Because of IE, we also have to check the presence of the constructor property.
  if (!obj || getType(obj) !== "object") return false;

  // Own properties are enumerated firstly, so to speed up,
  // if last one is own, then all properties are own.
  var key;
  for (key in obj) {}

  return key === undefined || Object.prototype.hasOwnProperty.call(obj, key);
}


function extend() {
  var options, name, src, copy, copyIsArray, clone,
    target = arguments[0] || {},
    i = 1,
    length = arguments.length,
    deep = false;

  // Handle a deep copy situation
  if (typeof target === "boolean") {
    deep = target;
    target = arguments[1] || {};
    // skip the boolean and the target
    i = 2;
  }

  // Handle case when target is a string or something (possible in deep copy)
  if (typeof target !== "object" && !isFunction(target)) target = {};

  // extend itself if only one argument is passed
  if (length === i) {
    target = this;
    --i;
  }

  for (; i < length; i++) {
    // Only deal with non-null/undefined values
    if ((options = arguments[i]) != null) {
      // Extend the base object
      for (name in options) {
        src = target[name];
        copy = options[name];

        // Prevent never-ending loop
        if (target === copy) continue;

        // Recurse if we're merging plain objects or arrays
        if (deep && copy && (isPlainObject(copy) || (copyIsArray = isArray(copy)))) {
          if (copyIsArray) {
            copyIsArray = false;
            clone = src && isArray(src) ? src : [];

          } else {
            clone = src && isPlainObject(src) ? src : {};
          }

          // Never move original objects, clone them
          target[name] = extend(deep, clone, copy);

        // Don't bring in undefined values
        } else if (copy !== undefined) {
          target[name] = copy;
        }
      }
    }
  }

  // Return the modified object
  return target;
}

// Create a simple deferred (one callbacks list)
function SimpleDeferred() {

  // callbacks list
  var callbacks = [];

  // stored [context , args]
  var fired;

  // to avoid firing when already doing so
  var firing;

  // flag to know if the deferred has been cancelled
  var cancelled;

  // the deferred itself
  var deferred  = {

    // done(f1, f2, ...)
    done: function () {
      if (!cancelled) {
        var args = arguments, i, length, elem, type, _fired;
        if (fired) {
          _fired = fired;
          fired = 0;
        }
        for (i = 0, length = args.length; i < length; i++) {
          elem = args[i];
          type = getType(elem);
          if (type === "array") {
            deferred.done.apply(deferred, elem);
          }
          else if (type === "function") {
            callbacks.push(elem);
          }
        }
        if (_fired) {
          deferred.resolveWith(_fired[0], _fired[1]);
        }
      }
      return this;
    },

    // resolve with given context and args
    resolveWith: function (context, args) {
      if (!cancelled && !fired && !firing) {
        firing = 1;
        try {
          while (callbacks[0]) {
            callbacks.shift().apply(context, args);
          }
        }
        finally {
          fired = [context, args];
          firing = 0;
        }
      }
      return this;
    },

    // resolve with this as context and given arguments
    resolve: function() {
      deferred.resolveWith(isFunction(this.promise) ? this.promise() : this, arguments);
      return this;
    },

    // Has this deferred been resolved?
    isResolved: function() {
      return !!(firing || fired);
    },

    // Cancel
    cancel: function() {
      cancelled = 1;
      callbacks = [];
      return this;
    }
  };

  return deferred;
}

// Full fledged deferred (two callbacks list)
this.Deferred = function(func) {
  var deferred = SimpleDeferred(),
    failDeferred = SimpleDeferred(),
    promise;

  // Add errorDeferred methods, then and promise
  extend(deferred, {
    then: function (doneCallbacks, failCallbacks) {
      deferred.done(doneCallbacks).fail(failCallbacks);
      return this;
    },
    fail: failDeferred.done,
    rejectWith: failDeferred.resolveWith,
    reject: failDeferred.resolve,
    isRejected: failDeferred.isResolved,

    // Get a promise for this deferred
    // If obj is provided, the promise aspect is added to the object
    promise: function (obj) {
      if (obj == null) {
        if (promise) return promise;
        promise = obj = {};
      }
      var i = promiseMethods.length;
      while (i--)  obj[promiseMethods[i]] = deferred[promiseMethods[i]];
      return obj;
    }
  });

  // Make sure only one callback list will be used
  deferred.done(failDeferred.cancel).fail(deferred.cancel);

  // Unexpose cancel
  delete deferred.cancel;

  // Call given func if any
  if (func) func.call(deferred, deferred);

  return deferred;
}

// Deferred helper
this.when = function (object) {
  var lastIndex = arguments.length,
    deferred = lastIndex <= 1 && object && isFunction(object.promise) ? object : Deferred(),
    promise = deferred.promise();

  if (lastIndex > 1) {
    var array = Array.prototype.slice.call(arguments, 0),
      count = lastIndex,
      iCallback = function (index) {
        return function (value) {
          array[index] = arguments.length > 1 ? Array.prototype.slice.call(arguments, 0) : value;
          if (!(--count)) deferred.resolveWith(promise, array);
        }
      };

    while ((lastIndex--)) {
      object = array[lastIndex];
      if (object && isFunction(object.promise))
        object.promise().then(iCallback(lastIndex), deferred.reject);
      else
        --count;
    }

    if (!count) deferred.resolveWith(promise, array);
  }
  else if (deferred !== object) {
    deferred.resolve(object);
  }
  return promise;
}

})();
var Zepto = (function() {
    var undefined, key, $, classList, emptyArray = [], slice = emptyArray.slice,
        document = window.document,
        elementDisplay = {}, classCache = {},
        getComputedStyle = document.defaultView.getComputedStyle,
        cssNumber = { 'column-count': 1, 'columns': 1, 'font-weight': 1, 'line-height': 1,'opacity': 1, 'z-index': 1, 'zoom': 1 },
        fragmentRE = /^\s*<(\w+|!)[^>]*>/,

    // Used by `$.zepto.init` to wrap elements, text/comment nodes, document,
    // and document fragment node types.
        elementTypes = [1, 3, 8, 9, 11],

        adjacencyOperators = [ 'after', 'prepend', 'before', 'append' ],
        table = document.createElement('table'),
        tableRow = document.createElement('tr'),
        containers = {
            'tr': document.createElement('tbody'),
            'tbody': table, 'thead': table, 'tfoot': table,
            'td': tableRow, 'th': tableRow,
            '*': document.createElement('div')
        },
        readyRE = /complete|loaded|interactive/,
        classSelectorRE = /^\.([\w-]+)$/,
        idSelectorRE = /^#([\w-]+)$/,
        tagSelectorRE = /^[\w-]+$/,
        toString = ({}).toString,
        zepto = {},
        camelize, uniq,
        tempParent = document.createElement('div')

    zepto.matches = function(element, selector) {
        if (!element || element.nodeType !== 1) return false
        var matchesSelector = element.webkitMatchesSelector || element.mozMatchesSelector ||
            element.oMatchesSelector || element.matchesSelector
        if (matchesSelector) return matchesSelector.call(element, selector)
        // fall back to performing a selector:
        var match, parent = element.parentNode, temp = !parent
        if (temp) (parent = tempParent).appendChild(element)
        match = ~zepto.qsa(parent, selector).indexOf(element)
        temp && tempParent.removeChild(element)
        return match
    }

    function isFunction(value) { return toString.call(value) == "[object Function]" }
    function isObject(value) { return value instanceof Object }
    function isPlainObject(value) {
        var key, ctor
        if (toString.call(value) !== "[object Object]") return false
        ctor = (isFunction(value.constructor) && value.constructor.prototype)
        if (!ctor || !hasOwnProperty.call(ctor, 'isPrototypeOf')) return false
        for (key in value);
        return key === undefined || hasOwnProperty.call(value, key)
    }
    function isArray(value) { return value instanceof Array }
    function likeArray(obj) { return typeof obj.length == 'number' }

    function compact(array) { return array.filter(function(item){ return item !== undefined && item !== null }) }
    function flatten(array) { return array.length > 0 ? [].concat.apply([], array) : array }
    camelize = function(str){ return str.replace(/-+(.)?/g, function(match, chr){ return chr ? chr.toUpperCase() : '' }) }
    function dasherize(str) {
        return str.replace(/::/g, '/')
            .replace(/([A-Z]+)([A-Z][a-z])/g, '$1_$2')
            .replace(/([a-z\d])([A-Z])/g, '$1_$2')
            .replace(/_/g, '-')
            .toLowerCase()
    }
    uniq = function(array){ return array.filter(function(item, idx){ return array.indexOf(item) == idx }) }

    function classRE(name) {
        return name in classCache ?
            classCache[name] : (classCache[name] = new RegExp('(^|\\s)' + name + '(\\s|$)'))
    }

    function maybeAddPx(name, value) {
        return (typeof value == "number" && !cssNumber[dasherize(name)]) ? value + "px" : value
    }

    function defaultDisplay(nodeName) {
        var element, display
        if (!elementDisplay[nodeName]) {
            element = document.createElement(nodeName)
            document.body.appendChild(element)
            display = getComputedStyle(element, '').getPropertyValue("display")
            element.parentNode.removeChild(element)
            display == "none" && (display = "block")
            elementDisplay[nodeName] = display
        }
        return elementDisplay[nodeName]
    }

    // `$.zepto.fragment` takes a html string and an optional tag name
    // to generate DOM nodes nodes from the given html string.
    // The generated DOM nodes are returned as an array.
    // This function can be overriden in plugins for example to make
    // it compatible with browsers that don't support the DOM fully.
    zepto.fragment = function(html, name) {
        if (name === undefined) name = fragmentRE.test(html) && RegExp.$1
        if (!(name in containers)) name = '*'
        var container = containers[name]
        container.innerHTML = '' + html
        return $.each(slice.call(container.childNodes), function(){
            container.removeChild(this)
        })
    }

    // `$.zepto.Z` swaps out the prototype of the given `dom` array
    // of nodes with `$.fn` and thus supplying all the Zepto functions
    // to the array. Note that `__proto__` is not supported on Internet
    // Explorer. This method can be overriden in plugins.
    zepto.Z = function(dom, selector) {
        dom = dom || []
        dom.__proto__ = arguments.callee.prototype
        dom.selector = selector || ''
        return dom
    }

    // `$.zepto.isZ` should return `true` if the given object is a Zepto
    // collection. This method can be overriden in plugins.
    zepto.isZ = function(object) {
        return object instanceof zepto.Z
    }

    // `$.zepto.init` is Zepto's counterpart to jQuery's `$.fn.init` and
    // takes a CSS selector and an optional context (and handles various
    // special cases).
    // This method can be overriden in plugins.
    zepto.init = function(selector, context) {
        // If nothing given, return an empty Zepto collection
        if (!selector) return zepto.Z()
        // If a function is given, call it when the DOM is ready
        else if (isFunction(selector)) return $(document).ready(selector)
        // If a Zepto collection is given, juts return it
        else if (zepto.isZ(selector)) return selector
        else {
            var dom
            // normalize array if an array of nodes is given
            if (isArray(selector)) dom = compact(selector)
            // if a JavaScript object is given, return a copy of it
            // this is a somewhat peculiar option, but supported by
            // jQuery so we'll do it, too
            else if (isPlainObject(selector))
                dom = [$.extend({}, selector)], selector = null
            // wrap stuff like `document` or `window`
            else if (elementTypes.indexOf(selector.nodeType) >= 0 || selector === window)
                dom = [selector], selector = null
            // If it's a html fragment, create nodes from it
            else if (fragmentRE.test(selector))
                dom = zepto.fragment(selector.trim(), RegExp.$1), selector = null
            // If there's a context, create a collection on that context first, and select
            // nodes from there
            else if (context !== undefined) return $(context).find(selector)
            // And last but no least, if it's a CSS selector, use it to select nodes.
            else dom = zepto.qsa(document, selector)
            // create a new Zepto collection from the nodes found
            return zepto.Z(dom, selector)
        }
    }

    // `$` will be the base `Zepto` object. When calling this
    // function just call `$.zepto.init, whichs makes the implementation
    // details of selecting nodes and creating Zepto collections
    // patchable in plugins.
    $ = function(selector, context){
        return zepto.init(selector, context)
    }

    // Copy all but undefined properties from one or more
    // objects to the `target` object.
    $.extend = function(target){
        slice.call(arguments, 1).forEach(function(source) {
            for (key in source)
                if (source[key] !== undefined)
                    target[key] = source[key]
        })
        return target
    }

    // `$.zepto.qsa` is Zepto's CSS selector implementation which
    // uses `document.querySelectorAll` and optimizes for some special cases, like `#id`.
    // This method can be overriden in plugins.
    zepto.qsa = function(element, selector){
        var found
        return (element === document && idSelectorRE.test(selector)) ?
            ( (found = element.getElementById(RegExp.$1)) ? [found] : emptyArray ) :
            (element.nodeType !== 1 && element.nodeType !== 9) ? emptyArray :
                slice.call(
                    classSelectorRE.test(selector) ? element.getElementsByClassName(RegExp.$1) :
                        tagSelectorRE.test(selector) ? element.getElementsByTagName(selector) :
                            element.querySelectorAll(selector)
                )
    }

    function filtered(nodes, selector) {
        return selector === undefined ? $(nodes) : $(nodes).filter(selector)
    }

    function funcArg(context, arg, idx, payload) {
        return isFunction(arg) ? arg.call(context, idx, payload) : arg
    }

    $.isFunction = isFunction
    $.isObject = isObject
    $.isArray = isArray
    $.isPlainObject = isPlainObject

    $.inArray = function(elem, array, i){
        return emptyArray.indexOf.call(array, elem, i)
    }

    $.trim = function(str) { return str.trim() }

    // plugin compatibility
    $.uuid = 0

    $.map = function(elements, callback){
        var value, values = [], i, key
        if (likeArray(elements))
            for (i = 0; i < elements.length; i++) {
                value = callback(elements[i], i)
                if (value != null) values.push(value)
            }
        else
            for (key in elements) {
                value = callback(elements[key], key)
                if (value != null) values.push(value)
            }
        return flatten(values)
    }

    $.each = function(elements, callback){
        var i, key
        if (likeArray(elements)) {
            for (i = 0; i < elements.length; i++)
                if (callback.call(elements[i], i, elements[i]) === false) return elements
        } else {
            for (key in elements)
                if (callback.call(elements[key], key, elements[key]) === false) return elements
        }

        return elements
    }

    // Define methods that will be available on all
    // Zepto collections
    $.fn = {
        // Because a collection acts like an array
        // copy over these useful array functions.
        forEach: emptyArray.forEach,
        reduce: emptyArray.reduce,
        push: emptyArray.push,
        indexOf: emptyArray.indexOf,
        concat: emptyArray.concat,

        // `map` and `slice` in the jQuery API work differently
        // from their array counterparts
        map: function(fn){
            return $.map(this, function(el, i){ return fn.call(el, i, el) })
        },
        slice: function(){
            return $(slice.apply(this, arguments))
        },

        ready: function(callback){
            if (readyRE.test(document.readyState)) callback($)
            else document.addEventListener('DOMContentLoaded', function(){ callback($) }, false)
            return this
        },
        get: function(idx){
            return idx === undefined ? slice.call(this) : this[idx]
        },
        toArray: function(){ return this.get() },
        size: function(){
            return this.length
        },
        remove: function(){
            return this.each(function(){
                if (this.parentNode != null)
                    this.parentNode.removeChild(this)
            })
        },
        each: function(callback){
            this.forEach(function(el, idx){ callback.call(el, idx, el) })
            return this
        },
        filter: function(selector){
            return $([].filter.call(this, function(element){
                return zepto.matches(element, selector)
            }))
        },
        add: function(selector,context){
            return $(uniq(this.concat($(selector,context))))
        },
        is: function(selector){
            return this.length > 0 && zepto.matches(this[0], selector)
        },
        not: function(selector){
            var nodes=[]
            if (isFunction(selector) && selector.call !== undefined)
                this.each(function(idx){
                    if (!selector.call(this,idx)) nodes.push(this)
                })
            else {
                var excludes = typeof selector == 'string' ? this.filter(selector) :
                    (likeArray(selector) && isFunction(selector.item)) ? slice.call(selector) : $(selector)
                this.forEach(function(el){
                    if (excludes.indexOf(el) < 0) nodes.push(el)
                })
            }
            return $(nodes)
        },
        eq: function(idx){
            return idx === -1 ? this.slice(idx) : this.slice(idx, + idx + 1)
        },
        first: function(){
            var el = this[0]
            return el && !isObject(el) ? el : $(el)
        },
        last: function(){
            var el = this[this.length - 1]
            return el && !isObject(el) ? el : $(el)
        },
        find: function(selector){
            var result
            if (this.length == 1) result = zepto.qsa(this[0], selector)
            else result = this.map(function(){ return zepto.qsa(this, selector) })
            return $(result)
        },
        closest: function(selector, context){
            var node = this[0]
            while (node && !zepto.matches(node, selector))
                node = node !== context && node !== document && node.parentNode
            return $(node)
        },
        parents: function(selector){
            var ancestors = [], nodes = this
            while (nodes.length > 0)
                nodes = $.map(nodes, function(node){
                    if ((node = node.parentNode) && node !== document && ancestors.indexOf(node) < 0) {
                        ancestors.push(node)
                        return node
                    }
                })
            return filtered(ancestors, selector)
        },
        parent: function(selector){
            return filtered(uniq(this.pluck('parentNode')), selector)
        },
        children: function(selector){
            return filtered(this.map(function(){ return slice.call(this.children) }), selector)
        },
        siblings: function(selector){
            return filtered(this.map(function(i, el){
                return slice.call(el.parentNode.children).filter(function(child){ return child!==el })
            }), selector)
        },
        empty: function(){
            return this.each(function(){ this.innerHTML = '' })
        },
        // `pluck` is borrowed from Prototype.js
        pluck: function(property){
            return this.map(function(){ return this[property] })
        },
        show: function(){
            return this.each(function(){
                this.style.display == "none" && (this.style.display = null)
                if (getComputedStyle(this, '').getPropertyValue("display") == "none")
                    this.style.display = defaultDisplay(this.nodeName)
            })
        },
        replaceWith: function(newContent){
            return this.before(newContent).remove()
        },
        wrap: function(newContent){
            return this.each(function(){
                $(this).wrapAll($(newContent)[0].cloneNode(false))
            })
        },
        wrapAll: function(newContent){
            if (this[0]) {
                $(this[0]).before(newContent = $(newContent))
                newContent.append(this)
            }
            return this
        },
        unwrap: function(){
            this.parent().each(function(){
                $(this).replaceWith($(this).children())
            })
            return this
        },
        clone: function(){
            return $(this.map(function(){ return this.cloneNode(true) }))
        },
        hide: function(){
            return this.css("display", "none")
        },
        toggle: function(setting){
            return (setting === undefined ? this.css("display") == "none" : setting) ? this.show() : this.hide()
        },
        prev: function(){ return $(this.pluck('previousElementSibling')) },
        next: function(){ return $(this.pluck('nextElementSibling')) },
        html: function(html){
            return html === undefined ?
                (this.length > 0 ? this[0].innerHTML : null) :
                this.each(function(idx){
                    var originHtml = this.innerHTML
                    $(this).empty().append( funcArg(this, html, idx, originHtml) )
                })
        },
        text: function(text){
            return text === undefined ?
                (this.length > 0 ? this[0].textContent : null) :
                this.each(function(){ this.textContent = text })
        },
        attr: function(name, value){
            var result
            return (typeof name == 'string' && value === undefined) ?
                (this.length == 0 || this[0].nodeType !== 1 ? undefined :
                    (name == 'value' && this[0].nodeName == 'INPUT') ? this.val() :
                        (!(result = this[0].getAttribute(name)) && name in this[0]) ? this[0][name] : result
                    ) :
                this.each(function(idx){
                    if (this.nodeType !== 1) return
                    if (isObject(name)) for (key in name) this.setAttribute(key, name[key])
                    else this.setAttribute(name, funcArg(this, value, idx, this.getAttribute(name)))
                })
        },
        removeAttr: function(name){
            return this.each(function(){ if (this.nodeType === 1) this.removeAttribute(name) })
        },
        prop: function(name, value){
            return (value === undefined) ?
                (this[0] ? this[0][name] : undefined) :
                this.each(function(idx){
                    this[name] = funcArg(this, value, idx, this[name])
                })
        },
        data: function(name, value){
            var data = this.attr('data-' + dasherize(name), value)
            return data !== null ? data : undefined
        },
        val: function(value){
            return (value === undefined) ?
                (this.length > 0 ? this[0].value : undefined) :
                this.each(function(idx){
                    this.value = funcArg(this, value, idx, this.value)
                })
        },
        offset: function(){
            if (this.length==0) return null
            var obj = this[0].getBoundingClientRect()
            return {
                left: obj.left + window.pageXOffset,
                top: obj.top + window.pageYOffset,
                width: obj.width,
                height: obj.height
            }
        },
        css: function(property, value){
            if (value === undefined && typeof property == 'string')
                return (
                    this.length == 0
                        ? undefined
                        : this[0].style[camelize(property)] || getComputedStyle(this[0], '').getPropertyValue(property))

            var css = ''
            for (key in property)
                if(typeof property[key] == 'string' && property[key] == '')
                    this.each(function(){ this.style.removeProperty(dasherize(key)) })
                else
                    css += dasherize(key) + ':' + maybeAddPx(key, property[key]) + ';'

            if (typeof property == 'string')
                if (value == '')
                    this.each(function(){ this.style.removeProperty(dasherize(property)) })
                else
                    css = dasherize(property) + ":" + maybeAddPx(property, value)

            return this.each(function(){ this.style.cssText += ';' + css })
        },
        index: function(element){
            return element ? this.indexOf($(element)[0]) : this.parent().children().indexOf(this[0])
        },
        hasClass: function(name){
            if (this.length < 1) return false
            else return classRE(name).test(this[0].className)
        },
        addClass: function(name){
            return this.each(function(idx){
                classList = []
                var cls = this.className, newName = funcArg(this, name, idx, cls)
                newName.split(/\s+/g).forEach(function(klass){
                    if (!$(this).hasClass(klass)) classList.push(klass)
                }, this)
                classList.length && (this.className += (cls ? " " : "") + classList.join(" "))
            })
        },
        removeClass: function(name){
            return this.each(function(idx){
                if (name === undefined)
                    return this.className = ''
                classList = this.className
                funcArg(this, name, idx, classList).split(/\s+/g).forEach(function(klass){
                    classList = classList.replace(classRE(klass), " ")
                })
                this.className = classList.trim()
            })
        },
        toggleClass: function(name, when){
            return this.each(function(idx){
                var newName = funcArg(this, name, idx, this.className)
                    ;(when === undefined ? !$(this).hasClass(newName) : when) ?
                    $(this).addClass(newName) : $(this).removeClass(newName)
            })
        }
    }

        // Generate the `width` and `height` functions
    ;['width', 'height'].forEach(function(dimension){
        $.fn[dimension] = function(value){
            var offset, Dimension = dimension.replace(/./, function(m){ return m[0].toUpperCase() })
            if (value === undefined) return this[0] == window ? window['inner' + Dimension] :
                this[0] == document ? document.documentElement['offset' + Dimension] :
                    (offset = this.offset()) && offset[dimension]
            else return this.each(function(idx){
                var el = $(this)
                el.css(dimension, funcArg(this, value, idx, el[dimension]()))
            })
        }
    })

    function insert(operator, target, node) {
        var parent = (operator % 2) ? target : target.parentNode
        parent ? parent.insertBefore(node,
            !operator ? target.nextSibling :      // after
                operator == 1 ? parent.firstChild :   // prepend
                    operator == 2 ? target :              // before
                        null) :                               // append
            $(node).remove()
    }

    function traverseNode(node, fun) {
        fun(node)
        for (var key in node.childNodes) traverseNode(node.childNodes[key], fun)
    }

    // Generate the `after`, `prepend`, `before`, `append`,
    // `insertAfter`, `insertBefore`, `appendTo`, and `prependTo` methods.
    adjacencyOperators.forEach(function(key, operator) {
        $.fn[key] = function(){
            // arguments can be nodes, arrays of nodes, Zepto objects and HTML strings
            var nodes = $.map(arguments, function(n){ return isObject(n) ? n : zepto.fragment(n) })
            if (nodes.length < 1) return this
            var size = this.length, copyByClone = size > 1, inReverse = operator < 2

            return this.each(function(index, target){
                for (var i = 0; i < nodes.length; i++) {
                    var node = nodes[inReverse ? nodes.length-i-1 : i]
                    traverseNode(node, function(node){
                        if (node.nodeName != null && node.nodeName.toUpperCase() === 'SCRIPT' && (!node.type || node.type === 'text/javascript'))
                            window['eval'].call(window, node.innerHTML)
                    })
                    if (copyByClone && index < size - 1) node = node.cloneNode(true)
                    insert(operator, target, node)
                }
            })
        }

        $.fn[(operator % 2) ? key+'To' : 'insert'+(operator ? 'Before' : 'After')] = function(html){
            $(html)[key](this)
            return this
        }
    })

    zepto.Z.prototype = $.fn

    // Export internal API functions in the `$.zepto` namespace
    zepto.camelize = camelize
    zepto.uniq = uniq
    $.zepto = zepto

    return $
})()

// If `$` is not yet defined, point it to `Zepto`
window.Zepto = Zepto
'$' in window || (window.$ = Zepto);

(function($){
    var $$ = $.zepto.qsa, handlers = {}, _zid = 1, specialEvents={}

    specialEvents.click = specialEvents.mousedown = specialEvents.mouseup = specialEvents.mousemove = 'MouseEvents'

    function zid(element) {
        return element._zid || (element._zid = _zid++)
    }
    function findHandlers(element, event, fn, selector) {
        event = parse(event)
        if (event.ns) var matcher = matcherFor(event.ns)
        return (handlers[zid(element)] || []).filter(function(handler) {
            return handler
                && (!event.e  || handler.e == event.e)
                && (!event.ns || matcher.test(handler.ns))
                && (!fn       || zid(handler.fn) === zid(fn))
                && (!selector || handler.sel == selector)
        })
    }
    function parse(event) {
        var parts = ('' + event).split('.')
        return {e: parts[0], ns: parts.slice(1).sort().join(' ')}
    }
    function matcherFor(ns) {
        return new RegExp('(?:^| )' + ns.replace(' ', ' .* ?') + '(?: |$)')
    }

    function eachEvent(events, fn, iterator){
        if ($.isObject(events)) $.each(events, iterator)
        else events.split(/\s/).forEach(function(type){ iterator(type, fn) })
    }

    function add(element, events, fn, selector, getDelegate, capture){
        capture = !!capture
        var id = zid(element), set = (handlers[id] || (handlers[id] = []))
        eachEvent(events, fn, function(event, fn){
            var delegate = getDelegate && getDelegate(fn, event),
                callback = delegate || fn
            var proxyfn = function (event) {
                var result = callback.apply(element, [event].concat(event.data))
                if (result === false) event.preventDefault()
                return result
            }
            var handler = $.extend(parse(event), {fn: fn, proxy: proxyfn, sel: selector, del: delegate, i: set.length})
            set.push(handler)
            element.addEventListener(handler.e, proxyfn, capture)
        })
    }
    function remove(element, events, fn, selector){
        var id = zid(element)
        eachEvent(events || '', fn, function(event, fn){
            findHandlers(element, event, fn, selector).forEach(function(handler){
                delete handlers[id][handler.i]
                element.removeEventListener(handler.e, handler.proxy, false)
            })
        })
    }

    $.event = { add: add, remove: remove }

    $.proxy = function(fn, context) {
        if ($.isFunction(fn)) {
            var proxyFn = function(){ return fn.apply(context, arguments) }
            proxyFn._zid = zid(fn)
            return proxyFn
        } else if (typeof context == 'string') {
            return $.proxy(fn[context], fn)
        } else {
            throw new TypeError("expected function")
        }
    }

    $.fn.bind = function(event, callback){
        return this.each(function(){
            add(this, event, callback)
        })
    }
    $.fn.unbind = function(event, callback){
        return this.each(function(){
            remove(this, event, callback)
        })
    }
    $.fn.one = function(event, callback){
        return this.each(function(i, element){
            add(this, event, callback, null, function(fn, type){
                return function(){
                    var result = fn.apply(element, arguments)
                    remove(element, type, fn)
                    return result
                }
            })
        })
    }

    var returnTrue = function(){return true},
        returnFalse = function(){return false},
        eventMethods = {
            preventDefault: 'isDefaultPrevented',
            stopImmediatePropagation: 'isImmediatePropagationStopped',
            stopPropagation: 'isPropagationStopped'
        }
    function createProxy(event) {
        var proxy = $.extend({originalEvent: event}, event)
        $.each(eventMethods, function(name, predicate) {
            proxy[name] = function(){
                this[predicate] = returnTrue
                return event[name].apply(event, arguments)
            }
            proxy[predicate] = returnFalse
        })
        return proxy
    }

    // emulates the 'defaultPrevented' property for browsers that have none
    function fix(event) {
        if (!('defaultPrevented' in event)) {
            event.defaultPrevented = false
            var prevent = event.preventDefault
            event.preventDefault = function() {
                this.defaultPrevented = true
                prevent.call(this)
            }
        }
    }

    $.fn.delegate = function(selector, event, callback){
        var capture = false
        if(event == 'blur' || event == 'focus'){
            if($.iswebkit)
                event = event == 'blur' ? 'focusout' : event == 'focus' ? 'focusin' : event
            else
                capture = true
        }

        return this.each(function(i, element){
            add(element, event, callback, selector, function(fn){
                return function(e){
                    var evt, match = $(e.target).closest(selector, element).get(0)
                    if (match) {
                        evt = $.extend(createProxy(e), {currentTarget: match, liveFired: element})
                        return fn.apply(match, [evt].concat([].slice.call(arguments, 1)))
                    }
                }
            }, capture)
        })
    }
    $.fn.undelegate = function(selector, event, callback){
        return this.each(function(){
            remove(this, event, callback, selector)
        })
    }

    $.fn.live = function(event, callback){
        $(document.body).delegate(this.selector, event, callback)
        return this
    }
    $.fn.die = function(event, callback){
        $(document.body).undelegate(this.selector, event, callback)
        return this
    }

    $.fn.on = function(event, selector, callback){
        return selector == undefined || $.isFunction(selector) ?
            this.bind(event, selector) : this.delegate(selector, event, callback)
    }
    $.fn.off = function(event, selector, callback){
        return selector == undefined || $.isFunction(selector) ?
            this.unbind(event, selector) : this.undelegate(selector, event, callback)
    }

    $.fn.trigger = function(event, data){
        if (typeof event == 'string') event = $.Event(event)
        fix(event)
        event.data = data
        return this.each(function(){
            // items in the collection might not be DOM elements
            // (todo: possibly support events on plain old objects)
            if('dispatchEvent' in this) this.dispatchEvent(event)
        })
    }

    // triggers event handlers on current element just as if an event occurred,
    // doesn't trigger an actual event, doesn't bubble
    $.fn.triggerHandler = function(event, data){
        var e, result
        this.each(function(i, element){
            e = createProxy(typeof event == 'string' ? $.Event(event) : event)
            e.data = data
            e.target = element
            $.each(findHandlers(element, event.type || event), function(i, handler){
                result = handler.proxy(e)
                if (e.isImmediatePropagationStopped()) return false
            })
        })
        return result
    }

        // shortcut methods for `.bind(event, fn)` for each event type
    ;('focusin focusout load resize scroll unload click dblclick '+
        'mousedown mouseup mousemove mouseover mouseout '+
        'change select keydown keypress keyup error').split(' ').forEach(function(event) {
            $.fn[event] = function(callback){ return this.bind(event, callback) }
        })

    ;['focus', 'blur'].forEach(function(name) {
        $.fn[name] = function(callback) {
            if (callback) this.bind(name, callback)
            else if (this.length) try { this.get(0)[name]() } catch(e){}
            return this
        }
    })

    $.Event = function(type, props) {
        var event = document.createEvent(specialEvents[type] || 'Events'), bubbles = true
        if (props) for (var name in props) (name == 'bubbles') ? (bubbles = !!props[name]) : (event[name] = props[name])
        event.initEvent(type, bubbles, true, null, null, null, null, null, null, null, null, null, null, null, null)
        return event
    }

})(Zepto);

(function($){
    var jsonpID = 0,
        isObject = $.isObject,
        document = window.document,
        key,
        name,
        rscript = /<script\b[^<]*(?:(?!<\/script>)<[^<]*)*<\/script>/gi,
        scriptTypeRE = /^(?:text|application)\/javascript/i,
        xmlTypeRE = /^(?:text|application)\/xml/i,
        jsonType = 'application/json',
        htmlType = 'text/html',
        blankRE = /^\s*$/

    // trigger a custom event and return false if it was cancelled
    function triggerAndReturn(context, eventName, data) {
        var event = $.Event(eventName)
        $(context).trigger(event, data)
        return !event.defaultPrevented
    }

    // trigger an Ajax "global" event
    function triggerGlobal(settings, context, eventName, data) {
        if (settings.global) return triggerAndReturn(context || document, eventName, data)
    }

    // Number of active Ajax requests
    $.active = 0

    function ajaxStart(settings) {
        if (settings.global && $.active++ === 0) triggerGlobal(settings, null, 'ajaxStart')
    }
    function ajaxStop(settings) {
        if (settings.global && !(--$.active)) triggerGlobal(settings, null, 'ajaxStop')
    }

    // triggers an extra global event "ajaxBeforeSend" that's like "ajaxSend" but cancelable
    function ajaxBeforeSend(xhr, settings) {
        var context = settings.context
        if (settings.beforeSend.call(context, xhr, settings) === false ||
            triggerGlobal(settings, context, 'ajaxBeforeSend', [xhr, settings]) === false)
            return false

        triggerGlobal(settings, context, 'ajaxSend', [xhr, settings])
    }
    function ajaxSuccess(data, xhr, settings) {
        var context = settings.context, status = 'success'
        settings.success.call(context, data, status, xhr)
        triggerGlobal(settings, context, 'ajaxSuccess', [xhr, settings, data])
        ajaxComplete(status, xhr, settings)
    }
    // type: "timeout", "error", "abort", "parsererror"
    function ajaxError(error, type, xhr, settings) {
        var context = settings.context
        settings.error.call(context, xhr, type, error)
        triggerGlobal(settings, context, 'ajaxError', [xhr, settings, error])
        ajaxComplete(type, xhr, settings)
    }
    // status: "success", "notmodified", "error", "timeout", "abort", "parsererror"
    function ajaxComplete(status, xhr, settings) {
        var context = settings.context
        settings.complete.call(context, xhr, status)
        triggerGlobal(settings, context, 'ajaxComplete', [xhr, settings])
        ajaxStop(settings)
    }

    // Empty function, used as default callback
    function empty() {}

    $.ajaxJSONP = function(options){
        var callbackName = 'jsonp' + (++jsonpID),
            script = document.createElement('script'),
            abort = function(){
                $(script).remove()
                if (callbackName in window) window[callbackName] = empty
                ajaxComplete('abort', xhr, options)
            },
            xhr = { abort: abort }, abortTimeout

        if (options.error) script.onerror = function() {
            xhr.abort()
            options.error()
        }

        window[callbackName] = function(data){
            clearTimeout(abortTimeout)
            $(script).remove()
            delete window[callbackName]
            ajaxSuccess(data, xhr, options)
        }

        serializeData(options)
        script.src = options.url.replace(/=\?/, '=' + callbackName)
        $('head').append(script)

        if (options.timeout > 0) abortTimeout = setTimeout(function(){
            xhr.abort()
            ajaxComplete('timeout', xhr, options)
        }, options.timeout)

        return xhr
    }

    $.ajaxSettings = {
        // Default type of request
        type: 'GET',
        // Callback that is executed before request
        beforeSend: empty,
        // Callback that is executed if the request succeeds
        success: empty,
        // Callback that is executed the the server drops error
        error: empty,
        // Callback that is executed on request complete (both: error and success)
        complete: empty,
        // The context for the callbacks
        context: null,
        // Whether to trigger "global" Ajax events
        global: true,
        // Transport
        xhr: function () {
            return new window.XMLHttpRequest()
        },
        // MIME types mapping
        accepts: {
            script: 'text/javascript, application/javascript',
            json:   jsonType,
            xml:    'application/xml, text/xml',
            html:   htmlType,
            text:   'text/plain'
        },
        // Whether the request is to another domain
        crossDomain: false,
        // Default timeout
        timeout: 0
    }

    function mimeToDataType(mime) {
        return mime && ( mime == htmlType ? 'html' :
            mime == jsonType ? 'json' :
                scriptTypeRE.test(mime) ? 'script' :
                    xmlTypeRE.test(mime) && 'xml' ) || 'text'
    }

    function appendQuery(url, query) {
        return (url + '&' + query).replace(/[&?]{1,2}/, '?')
    }

    // serialize payload and append it to the URL for GET requests
    function serializeData(options) {
        if (isObject(options.data)) options.data = $.param(options.data)
        if (options.data && (!options.type || options.type.toUpperCase() == 'GET'))
            options.url = appendQuery(options.url, options.data)
    }

    $.ajax = function(options){
        var settings = $.extend({}, options || {})
        for (key in $.ajaxSettings) if (settings[key] === undefined) settings[key] = $.ajaxSettings[key]

        ajaxStart(settings)

        if (!settings.crossDomain) settings.crossDomain = /^([\w-]+:)?\/\/([^\/]+)/.test(settings.url) &&
            RegExp.$2 != window.location.host

        var dataType = settings.dataType, hasPlaceholder = /=\?/.test(settings.url)
        if (dataType == 'jsonp' || hasPlaceholder) {
            if (!hasPlaceholder) settings.url = appendQuery(settings.url, 'callback=?')
            return $.ajaxJSONP(settings)
        }

        if (!settings.url) settings.url = window.location.toString()
        serializeData(settings)

        var mime = settings.accepts[dataType],
            baseHeaders = { },
            protocol = /^([\w-]+:)\/\//.test(settings.url) ? RegExp.$1 : window.location.protocol,
            xhr = $.ajaxSettings.xhr(), abortTimeout

        if (!settings.crossDomain) baseHeaders['X-Requested-With'] = 'XMLHttpRequest'
        if (mime) {
            baseHeaders['Accept'] = mime
            if (mime.indexOf(',') > -1) mime = mime.split(',', 2)[0]
            xhr.overrideMimeType && xhr.overrideMimeType(mime)
        }
        if (settings.contentType || (settings.data && settings.type.toUpperCase() != 'GET'))
            baseHeaders['Content-Type'] = (settings.contentType || 'application/x-www-form-urlencoded')
        settings.headers = $.extend(baseHeaders, settings.headers || {})

        xhr.onreadystatechange = function(){
            if (xhr.readyState == 4) {
                clearTimeout(abortTimeout)
                var result, error = false
                if ((xhr.status >= 200 && xhr.status < 300) || xhr.status == 304 || (xhr.status == 0 && protocol == 'file:')) {
                    dataType = dataType || mimeToDataType(xhr.getResponseHeader('content-type'))
                    result = xhr.responseText

                    try {
                        if (dataType == 'script')    (1,eval)(result)
                        else if (dataType == 'xml')  result = xhr.responseXML
                        else if (dataType == 'json') result = blankRE.test(result) ? null : JSON.parse(result)
                    } catch (e) { error = e }

                    if (error) ajaxError(error, 'parsererror', xhr, settings)
                    else ajaxSuccess(result, xhr, settings)
                } else {
                    ajaxError(null, 'error', xhr, settings)
                }
            }
        }

        var async = 'async' in settings ? settings.async : true
        xhr.open(settings.type, settings.url, async)

        for (name in settings.headers) xhr.setRequestHeader(name, settings.headers[name])

        if (ajaxBeforeSend(xhr, settings) === false) {
            xhr.abort()
            return false
        }

        if (settings.timeout > 0) abortTimeout = setTimeout(function(){
            xhr.onreadystatechange = empty
            xhr.abort()
            ajaxError(null, 'timeout', xhr, settings)
        }, settings.timeout)

        // avoid sending empty string (#319)
        xhr.send(settings.data ? settings.data : null)
        return xhr
    }

    $.get = function(url, success){ return $.ajax({ url: url, success: success }) }

    $.post = function(url, data, success, dataType){
        if ($.isFunction(data)) dataType = dataType || success, success = data, data = null
        return $.ajax({ type: 'POST', url: url, data: data, success: success, dataType: dataType })
    }

    $.getJSON = function(url, success){
        return $.ajax({ url: url, success: success, dataType: 'json' })
    }

    $.fn.load = function(url, success){
        if (!this.length) return this
        var self = this, parts = url.split(/\s/), selector
        if (parts.length > 1) url = parts[0], selector = parts[1]
        $.get(url, function(response){
            self.html(selector ?
                $(document.createElement('div')).html(response.replace(rscript, "")).find(selector).html()
                : response)
            success && success.call(self)
        })
        return this
    }

    var escape = encodeURIComponent

    function serialize(params, obj, traditional, scope){
        var array = $.isArray(obj)
        $.each(obj, function(key, value) {
            if (scope) key = traditional ? scope : scope + '[' + (array ? '' : key) + ']'
            // handle data in serializeArray() format
            if (!scope && array) params.add(value.name, value.value)
            // recurse into nested objects
            else if (traditional ? $.isArray(value) : isObject(value))
                serialize(params, value, traditional, key)
            else params.add(key, value)
        })
    }

    $.param = function(obj, traditional){
        var params = []
        params.add = function(k, v){ this.push(escape(k) + '=' + escape(v)) }
        serialize(params, obj, traditional)
        return params.join('&').replace('%20', '+')
    }
})(Zepto);
var json_parse = (function () {
    "use strict";

    var state,      // The state of the parser, one of
        stack,      // The stack, for controlling nesting.
        container,  // The current container object or array
        key,        // The current key
        value,      // The current value
        escapes = { // Escapement translation table
            '\\': '\\',
            '"': '"',
            '/': '/',
            't': '\t',
            'n': '\n',
            'r': '\r',
            'f': '\f',
            'b': '\b'
        },
        string = {   // The actions for string tokens
            go: function () {
                state = 'ok';
            },
            firstokey: function () {
                key = value;
                state = 'colon';
            },
            okey: function () {
                key = value;
                state = 'colon';
            },
            ovalue: function () {
                state = 'ocomma';
            },
            firstavalue: function () {
                state = 'acomma';
            },
            avalue: function () {
                state = 'acomma';
            }
        },
        number = {   // The actions for number tokens
            go: function () {
                state = 'ok';
            },
            ovalue: function () {
                state = 'ocomma';
            },
            firstavalue: function () {
                state = 'acomma';
            },
            avalue: function () {
                state = 'acomma';
            }
        },
        action = {
            '{': {
                go: function () {
                    stack.push({state: 'ok'});
                    container = {};
                    state = 'firstokey';
                },
                ovalue: function () {
                    stack.push({container: container, state: 'ocomma', key: key});
                    container = {};
                    state = 'firstokey';
                },
                firstavalue: function () {
                    stack.push({container: container, state: 'acomma'});
                    container = {};
                    state = 'firstokey';
                },
                avalue: function () {
                    stack.push({container: container, state: 'acomma'});
                    container = {};
                    state = 'firstokey';
                }
            },
            '}': {
                firstokey: function () {
                    var pop = stack.pop();
                    value = container;
                    container = pop.container;
                    key = pop.key;
                    state = pop.state;
                },
                ocomma: function () {
                    var pop = stack.pop();
                    container[key] = value;
                    value = container;
                    container = pop.container;
                    key = pop.key;
                    state = pop.state;
                }
            },
            '[': {
                go: function () {
                    stack.push({state: 'ok'});
                    container = [];
                    state = 'firstavalue';
                },
                ovalue: function () {
                    stack.push({container: container, state: 'ocomma', key: key});
                    container = [];
                    state = 'firstavalue';
                },
                firstavalue: function () {
                    stack.push({container: container, state: 'acomma'});
                    container = [];
                    state = 'firstavalue';
                },
                avalue: function () {
                    stack.push({container: container, state: 'acomma'});
                    container = [];
                    state = 'firstavalue';
                }
            },
            ']': {
                firstavalue: function () {
                    var pop = stack.pop();
                    value = container;
                    container = pop.container;
                    key = pop.key;
                    state = pop.state;
                },
                acomma: function () {
                    var pop = stack.pop();
                    container.push(value);
                    value = container;
                    container = pop.container;
                    key = pop.key;
                    state = pop.state;
                }
            },
            ':': {
                colon: function () {
                    if (Object.hasOwnProperty.call(container, key)) {
                        throw new SyntaxError('Duplicate key "' + key + '"');
                    }
                    state = 'ovalue';
                }
            },
            ',': {
                ocomma: function () {
                    container[key] = value;
                    state = 'okey';
                },
                acomma: function () {
                    container.push(value);
                    state = 'avalue';
                }
            },
            'true': {
                go: function () {
                    value = true;
                    state = 'ok';
                },
                ovalue: function () {
                    value = true;
                    state = 'ocomma';
                },
                firstavalue: function () {
                    value = true;
                    state = 'acomma';
                },
                avalue: function () {
                    value = true;
                    state = 'acomma';
                }
            },
            'false': {
                go: function () {
                    value = false;
                    state = 'ok';
                },
                ovalue: function () {
                    value = false;
                    state = 'ocomma';
                },
                firstavalue: function () {
                    value = false;
                    state = 'acomma';
                },
                avalue: function () {
                    value = false;
                    state = 'acomma';
                }
            },
            'null': {
                go: function () {
                    value = null;
                    state = 'ok';
                },
                ovalue: function () {
                    value = null;
                    state = 'ocomma';
                },
                firstavalue: function () {
                    value = null;
                    state = 'acomma';
                },
                avalue: function () {
                    value = null;
                    state = 'acomma';
                }
            }
        };

    function debackslashify(text) {
        return text.replace(/\\(?:u(.{4})|([^u]))/g, function (a, b, c) {
            return b ? String.fromCharCode(parseInt(b, 16)) : escapes[c];
        });
    }

    return function (source, reviver) {
        var r,          // The result of the exec method.
            tx = /^[\x20\t\n\r]*(?:([,:\[\]{}]|true|false|null)|(-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)|"((?:[^\r\n\t\\\"]|\\(?:["\\\/trnfb]|u[0-9a-fA-F]{4}))*)")/;
        state = 'go';
        stack = [];

        try {

            for (;;) {
                r = tx.exec(source);
                if (!r) {
                    break;
                }

                if (r[1]) {

                    action[r[1]][state]();

                } else if (r[2]) {

                    value = +r[2];
                    number[state]();
                } else {

                    value = debackslashify(r[3]);
                    string[state]();
                }

                source = source.slice(r[0].length);
            }

        } catch (e) {
            state = e;
        }

        if (state !== 'ok' || /[^\x20\t\n\r]/.test(source)) {
            throw state instanceof SyntaxError ? state : new SyntaxError('JSON');
        }

        return typeof reviver === 'function' ? (function walk(holder, key) {
            var k, v, value = holder[key];
            if (value && typeof value === 'object') {
                for (k in value) {
                    if (Object.prototype.hasOwnProperty.call(value, k)) {
                        v = walk(value, k);
                        if (v !== undefined) {
                            value[k] = v;
                        } else {
                            delete value[k];
                        }
                    }
                }
            }
            return reviver.call(holder, key, value);
        }({'': value}, '')) : value;
    };
}());
var Base64 = {

    // private property
    _keyStr : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

    // public method for encoding
    encode : function (input) {
        var output = "";
        var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
        var i = 0;

        input = Base64._utf8_encode(input);

        while (i < input.length) {

            chr1 = input.charCodeAt(i++);
            chr2 = input.charCodeAt(i++);
            chr3 = input.charCodeAt(i++);

            enc1 = chr1 >> 2;
            enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
            enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
            enc4 = chr3 & 63;

            if (isNaN(chr2)) {
                enc3 = enc4 = 64;
            } else if (isNaN(chr3)) {
                enc4 = 64;
            }

            output = output +
                this._keyStr.charAt(enc1) + this._keyStr.charAt(enc2) +
                this._keyStr.charAt(enc3) + this._keyStr.charAt(enc4);

        }

        return output;
    },

    // private method for UTF-8 encoding
    _utf8_encode : function (string) {
        string = string.replace(/\r\n/g,"\n");
        var utftext = "";

        for (var n = 0; n < string.length; n++) {

            var c = string.charCodeAt(n);

            if (c < 128) {
                utftext += String.fromCharCode(c);
            }
            else if((c > 127) && (c < 2048)) {
                utftext += String.fromCharCode((c >> 6) | 192);
                utftext += String.fromCharCode((c & 63) | 128);
            }
            else {
                utftext += String.fromCharCode((c >> 12) | 224);
                utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                utftext += String.fromCharCode((c & 63) | 128);
            }

        }

        return utftext;
    }

}
;
function throttle(fn, threshhold, scope) {
    threshhold || (threshhold = 10);
    var last,
        deferTimer;
    return function () {
        var context = scope || this;
        var now = +new Date,
            args = arguments;
        if (last && now < last + threshhold) {
            // hold on to it
            clearTimeout(deferTimer);
            deferTimer = setTimeout(function () {
                last = now;
                fn.apply(context, args);
            }, threshhold);
        } else {
            last = now;
            fn.apply(context, args);
        }
    };
}
;

/**
Generic application namespace
@namespace Rutv
*/


(function() {
  var Rutv;

  Rutv = {
    /**
    Namespace for debug-stuff and development-time classes
    @namespace Rutv.Debug
    */

    Debug: {},
    Locale: {},
    Platform: {
      Generic: {},
      Storage: {},
      LG: {},
      PC: {},
      Samsung: {},
      Server: {}
    },
    View: {
      /**
      Namespace for debug-stuff and development-time classes
      @namespace Rutv.View.Scene
      */

      Scene: {
        Channels: {}
      },
      Templates: {},
      Modal: {
        Error: {}
      },
      Component: {
        App: {},
        Home: {
          Menu: {
            Button: {}
          },
          Account: {},
          Tv: {
            Button: {}
          }
        },
        Channels: {
          Channel: {}
        },
        Schedule: {
          Channel: {}
        },
        Player: {},
        Modal: {},
        Queue: {}
      },
      Adopter: {},
      App: {}
    },
    Model: {
      Queue: {},
      Channel: {
        Schedule: {}
      }
    },
    Collection: {},
    /**
    Contains object, which controls complex processes
    @namespace Rutv.Manager
    */

    Manager: {},
    Lib: {
      AuthManager: {},
      DeferredWrapper: {},
      RequestWrapper: {}
    },
    /**
    Contains server communication logic classes
    @namespace Rutv.Server
    */

    Server: {
      /**
      Contains requests to server classes
      @namespace Rutv.Server.Requests
      */

      Requests: {}
    },
    User: {
      Channels: {}
    },
    /**
    Contains support classes
    @namespace Rutv.Utils
    */

    Utils: {
      Location: {}
    }
  };

  window.Rutv = Rutv;

}).call(this);

/**
@namespace config

@description Contains application-wide settings.

@property {object} console settings for debug html-console
@property {boolean} console.enabled enables console usage
@property {integer} console.scroll_step step in pixels to scroll console content

@property {object} platform application's environment specific setting
@property {string} platform.force if null, then autodetec TV platform, if 'pc' then forces PC's browser env

@property {object} server server settings
@property {string} server.url base URL for requests to server
@property {string} server.accepts default value for Accept request header to server
*/


(function() {
  var config;

  config = {
    language: 'ru',
    console: {
      enabled: true,
      scroll_step: 10
    },
    defaults: {
      loading_logged_status_timeout: 10000
    },
    view: {
      window: {
        size_program_in_schedule: 2,
        hide_player_panels_after: 5000,
        hide_player_input_panel_after: 5000,
        start_find_channel_after_enter_numbers: 1000
      }
    },
    player: {
      interval_check_began_next_program: 60000,
      rewind_interval: 60,
      update_queue_status_interval: 300000,
      samsung: {
        min_rewind_interval: 300
      },
      lg: {
        min_rewind_interval: 300
      }
    },
    server: {
      url: "http://rutvclub.com",
      accepts: "application/vnd.tvp.stvp+json"
    }
  };

  window.config = config;

}).call(this);
(function() {

  Rutv.Locale.ru = {
    loading: 'Загрузка...',
    channel_by_number_selector: function(number, channel) {
      return {
        number: "" + number,
        hint: channel != null ? channel.get_name() : 'канал не найден'
      };
    },
    modal: {
      exit_confirmation: {
        heading: 'Хотите выйти?',
        description_html: '',
        buttons: ['Выйти', 'Отмена']
      },
      error: {
        request_failed: {
          heading: 'Ошибка сервера',
          description_html: '<p>Произошла ошибка при получении данных с сервера. Проверьте соединенение с сетью и попробуйте ещё раз.</p>',
          buttons: ['Повторить', 'Отмена']
        },
        no_streaming_url: {
          heading: 'Ошибка проигрывания',
          description_html: '<p>ПРОВЕРЬТЕ: &nbsp;1. Оплачен ли членский взнос<br>&nbsp;&nbsp;&nbsp;2. Включена ли Опция "Возможность просмотра на Устройствах"</p>',
          buttons: ['OK']
        }
      }
    },
    login: {
      heading: 'Подключите Smart TV к RUTVclub',
      instructions_html: '\
      <ol class="step-list">\
        <li>Перейдите на компьютере на страницу <a href="http://rutvclub.com/account/devices">rutvclub.com/account/devices</a><br>введите свой email и пароль, если нужно</li>\
        <li>Введите код в желтое окошко и нажмите кнопку «Подключить»</li>\
      </ol>',
      hint: 'Если код введен правильно, то телевизор подключится автоматически'
    },
    home: {
      menu: {
        tv: 'Телеэфир',
        queue: 'Посмотреть',
        account: 'Личный кабинет'
      },
      tv: {
        favourites: function(channels) {
          return {
            count: channels.get_count(),
            label: 'в любимых'
          };
        },
        all: function(channels) {
          var count;
          count = channels.get_count();
          return {
            count: count,
            label: Rutv.Locale.ru.pluralize_resource(count, 'channel')
          };
        },
        random_channel: {
          loading: {
            loading_please_wait: 'Подождите, идёт загрузка...'
          }
        }
      },
      account: function(data) {
        return {
          payment_status: data.account.subscription_active === true ? "Членский взнос оплачен до " + data.account.till : 'Членский взнос не оплачен',
          extend_payment_html: 'Дополнительная информация, поддержка и оплата на <span>rutvclub.com</span>',
          user_timezone: "Ваш часовой пояс: " + data.account.user_timezone,
          display_name: data.account.display_name
        };
      }
    },
    channels: {
      all: {
        heading: function(channels) {
          return {
            label: 'Все каналы',
            of: 'из',
            total: channels != null ? channels.get_count() : null,
            current: channels != null ? channels.get_current_index() + 1 : null
          };
        },
        list: {
          loading: {
            loading_please_wait: 'Подождите, идёт загрузка...'
          },
          empty: {
            sorry_html: 'Каналов нет :('
          }
        }
      },
      favourited: {
        heading: function(channels) {
          return {
            label: 'Любимые каналы',
            of: 'из',
            total: channels != null ? channels.get_count() : null,
            current: channels != null ? channels.get_current_index() + 1 : null
          };
        },
        list: {
          loading: {
            loading_please_wait: 'Подождите, идёт загрузка...'
          },
          empty: {
            sorry_html: 'Каналов нет :('
          }
        }
      }
    },
    schedule: {
      channels: {
        loading: 'Загрузка...',
        loaded: {
          heading: 'Телепрограмма',
          unavailable: ' (просмотр недоступен)'
        },
        loading_html: function(progress, total) {
          return {
            loading_please_wait_html: "Подождите, идёт загрузка...<br><small>" + progress + " из " + total + " " + (Rutv.Locale.ru.pluralize_resource(total, 'channel.genetive')) + "</small>"
          };
        },
        load_failed: {
          loading_please_wait: 'При загрузке программы произошла ошибка'
        },
        programms_not_avalible: 'Телепрограмма передач не доступна.'
      }
    },
    queue: {
      heading: function(queue) {
        return {
          label: 'Посмотреть',
          of: 'из',
          total: queue != null ? queue.get_count() : null,
          current: queue != null ? queue.get_current_index() + 1 : null
        };
      },
      list: {
        loading: {
          loading_please_wait: 'Подождите, идёт загрузка...'
        },
        empty: {
          sorry_html: 'Программ нет :('
        }
      }
    },
    hints: {
      exit: 'Выйти',
      back: 'Вернуться',
      play: 'Просмотреть',
      channels: 'Каналы',
      menu: 'Меню',
      channel: 'Канал',
      favourited_channels: 'Любимые каналы',
      all_channels: 'Все каналы',
      channels_list: 'Список каналов',
      show: 'Показать',
      cancel: 'Отмена',
      tv: 'Телеэфир',
      account: 'Личный кабинет',
      schedule: 'Телепрограмма',
      previous_channel: 'Предыдущий',
      next_channel: 'Следующий',
      previous_schedule_channel: 'Вверх',
      next_schedule_channel: 'Вниз',
      previous_programm: 'Влево',
      next_programm: 'Вправо',
      favouriter: 'Любимый (+/-)',
      queue: 'Посмотреть'
    },
    channel: {
      one: 'канал',
      few: 'канала',
      many: 'каналов',
      other: 'каналов',
      genetive: {
        one: 'канала',
        few: 'каналов',
        many: 'каналов',
        other: 'каналов'
      }
    },
    pluralize: function(n, one, few, many, other) {
      var _ref, _ref1, _ref2, _ref3;
      if (other == null) {
        other = few;
      }
      if ((n % 10) === 1 && n % 100 !== 11) {
        return one;
      } else {
        if (((_ref = n % 10) === 2 || _ref === 3 || _ref === 4) && !((_ref1 = n % 100) === 12 || _ref1 === 13 || _ref1 === 14)) {
          return few;
        } else {
          if ((n % 10) === 0 || ((_ref2 = n % 10) === 5 || _ref2 === 6 || _ref2 === 7 || _ref2 === 8 || _ref2 === 9) || ((_ref3 = n % 100) === 11 || _ref3 === 12 || _ref3 === 13 || _ref3 === 14)) {
            return many;
          } else {
            return other;
          }
        }
      }
    },
    pluralize_resource: function(n, resource) {
      var code, items;
      if (resource.indexOf('.') > -1) {
        code = "items = this." + resource;
        eval(code);
      } else {
        items = this[resource];
      }
      return this.pluralize(n, items.one, items.few, items.many, items.other);
    }
  };

}).call(this);
(function() {
  var string_trim_begin_re, string_trim_end_re;

  if (String.prototype.trim == null) {
    string_trim_begin_re = /^\s+/;
    string_trim_end_re = /\s+$/;
    String.prototype.trim = function() {
      return this.replace(string_trim_begin_re, '').replace(string_trim_end_re, '');
    };
  }

}).call(this);
(function() {
  
var ArrayProto = Array.prototype;
var slice = ArrayProto.slice;

var nativeForEach = ArrayProto.forEach
    var each = function(obj, iterator, context) {
        if (obj == null) return;
        if (nativeForEach && obj.forEach === nativeForEach) {
            obj.forEach(iterator, context);
        } else if (obj.length === +obj.length) {
            for (var i = 0, l = obj.length; i < l; i++) {
                if (iterator.call(context, obj[i], i, obj) === breaker) return;
            }
        } else {
            for (var key in obj) {
                if (_.has(obj, key)) {
                    if (iterator.call(context, obj[key], key, obj) === breaker) return;
                }
            }
        }
    };

var defaults = function(obj) {
    each(slice.call(arguments, 1), function(source) {
        for (var prop in source) {
            if (obj[prop] == null) obj[prop] = source[prop];
        }
    });
    return obj;
};

// By default, Underscore uses ERB-style template delimiters, change the
// following template settings to use alternative delimiters.
var templateSettings = {
    evaluate    : /<%([\s\S]+?)%>/g,
    interpolate : /<%=([\s\S]+?)%>/g,
    escape      : /<%-([\s\S]+?)%>/g
};

    // When customizing templateSettings, if you don't want to define an
// interpolation, evaluation or escaping regex, we need one that is
// guaranteed not to match.
var noMatch = /(.)^/;

// Certain characters need to be escaped so that they can be put into a
// string literal.
var escapes = {
"'":      "'",
'\\':     '\\',
'\r':     'r',
'\n':     'n',
'\t':     't',
'\u2028': 'u2028',
'\u2029': 'u2029'
};

var escaper = /\\|'|\r|\n|\t|\u2028|\u2029/g;

template = function(text, data, settings) {
    settings = defaults({}, settings, templateSettings);

    // Combine delimiters into one regular expression via alternation.
    var matcher = new RegExp([
        (settings.escape || noMatch).source,
        (settings.interpolate || noMatch).source,
        (settings.evaluate || noMatch).source
    ].join('|') + '|$', 'g');

    // Compile the template source, escaping string literals appropriately.
    var index = 0;
    var source = "__p+='";
    text.replace(matcher, function(match, escape, interpolate, evaluate, offset) {
        source += text.slice(index, offset)
            .replace(escaper, function(match) { return '\\' + escapes[match]; });
        source +=
            escape ? "'+\n((__t=(" + escape + "))==null?'':_.escape(__t))+\n'" :
                interpolate ? "'+\n((__t=(" + interpolate + "))==null?'':__t)+\n'" :
                    evaluate ? "';\n" + evaluate + "\n__p+='" : '';
        index = offset + match.length;
    });
    source += "';\n";

    // If a variable is not specified, place data values in local scope.
    if (!settings.variable) source = 'with(obj||{}){\n' + source + '}\n';

    source = "var __t,__p='',__j=Array.prototype.join," +
        "print=function(){__p+=__j.call(arguments,'');};\n" +
        source + "return __p;\n";

    try {
        var render = new Function(settings.variable || 'obj', '_', source);
    } catch (e) {
        e.source = source;
        throw e;
    }

    if (data) return render(data, this);
    var template = function(data) {
        return render.call(this, data, this);
    };

    // Provide the compiled function source as a convenience for precompilation.
    template.source = 'function(' + (settings.variable || 'obj') + '){\n' + source + '}';

    return template;
};
;

  Rutv.Utils.template = template;

}).call(this);
(function() {

  Rutv.Utils.extend = function(destination, source) {
    var key, value;
    for (key in source) {
      value = source[key];
      destination[key] = value;
    }
    return destination;
  };

}).call(this);
(function() {
  var event_splitter, slice;

  event_splitter = /\s+/;

  slice = Array.prototype.slice;

  /**
  @mixin
  
  @desc
  This mixin can be mixed in to any object in order to provide it with
  custom events. You may bind with `on` or remove with `off` callback functions
  to an event; trigger`-ing an event fires all callbacks in succession.
  
  @example
      class Someclass
      Rutv.Utils.extend Someclass::, Jetfid.Events
      instance = new Someclass()
      instance.on('expand', -> alert('expanded'))
      instance.trigger('expand')
  */


  Rutv.Utils.Events = {
    /**
    Bind callback function to specified event(s)
    
    @param {string} events space-separated list of events (or "all" to any event) to execute callback on triggering
    @param {function} callback callback to be executed on specified event(s)
    @param {object} [context=window] callback's execution context
    */

    on: function(events, callback, context) {
      var calls, ev, list, tail;
      events = events.split(event_splitter);
      calls = this._callbacks || (this._callbacks = {});
      while (ev = events.shift()) {
        list = calls[ev] || (calls[ev] = {});
        tail = list.tail || (list.tail = list.next = {});
        tail.callback = callback;
        tail.context = context;
        list.tail = tail.next = {};
      }
      return this;
    },
    /**
    Remove one or many event callbacks
    
    @param {string} [events=null] space-separated list of events, if null removes all callbacks for all events
    @param {function} [callback=null] callback to be removed on specified events, if null removes all callback on specified events
    @param {object} [context=null] callback's execution context, if null removes all callbacks with events and callback specified
    */

    off: function(events, callback, context) {
      var calls, ev, node;
      if (!events) {
        delete this._callbacks;
      } else {
        if ((calls = this._callbacks)) {
          events = events.split(event_splitter);
          while (ev = events.shift()) {
            node = calls[ev];
            delete calls[ev];
            if (!callback || !node) {
              continue;
            }
            while ((node = node.next) && node.next) {
              if (node.callback === callback && (!context || node.context === context)) {
                continue;
              }
              this.on(ev, node.callback, node.context);
            }
          }
        }
      }
      return this;
    },
    /**
    Trigger an event, firing all bound callbacks.
    
    @param {string} event event name to execute callbacks for
    @returns {boolean} false if any callback returns false, true otherwise
    */

    trigger: function(event) {
      var args, callback_result, calls, node, result, tail;
      result = true;
      if ((calls = this._callbacks) && (node = calls[event])) {
        args = slice.call(arguments, 1);
        node = {
          next: node.next,
          tail: node.tail
        };
        tail = node.tail;
        while ((node = node.next) !== tail) {
          callback_result = node.callback.apply(node.context || this, args);
          if (callback_result === false) {
            result = false;
            break;
          }
        }
      }
      return result;
    }
  };

}).call(this);
(function() {
  var __slice = [].slice;

  Rutv.Debug.Console = (function() {

    function Console(config, key_trap, root_selector) {
      this.config = config;
      this.key_trap = key_trap;
      if (root_selector == null) {
        root_selector = ':console';
      }
      this.enabled = this.config.enabled;
      if (this.enabled) {
        this.init(root_selector);
      }
      this.show_handler();
    }

    Console.prototype.log = function() {
      var i, value, values, _i, _len, _results;
      values = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      if (this.enabled) {
        _results = [];
        for (i = _i = 0, _len = values.length; _i < _len; i = ++_i) {
          value = values[i];
          _results.push(this.data_processing(value, "<li><pre>", "</pre></li>"));
        }
        return _results;
      }
    };

    Console.prototype.info = function(value) {
      if (this.enabled) {
        return this.data_processing(value, "<li class='console_info'><div class='console-info'><pre>", "</pre></div></li>");
      }
    };

    Console.prototype.warn = function(value) {
      if (this.enabled) {
        return this.data_processing(value, "<li class='console_warn'><div class='console-warn'><pre>", "</pre></div></li>");
      }
    };

    Console.prototype.error = function(value) {
      if (this.enabled) {
        return this.data_processing(value, "<li class='console_error'><div class='console-error'><pre>", "</pre></div></li>");
      }
    };

    Console.prototype.time = function(value) {
      if (this.enabled) {
        return this.timer_data[value] = new Date();
      }
    };

    Console.prototype.timeEnd = function(value) {
      var time;
      if (this.enabled) {
        time = new Date() - this.timer_data[value];
        return this.el.innerHTML += "<li><pre>" + value + ": <span class='blue'>" + time + "ms</span></pre></li>";
      }
    };

    Console.prototype.group = function(value) {
      var group_arr;
      if (this.enabled) {
        value = value || "";
        this.el.innerHTML += "<li class='console__list_group'>" + value + "<ul class='console-group'></ul></li>";
        group_arr = document.getElementsByClassName('console-group');
        return this.el = group_arr[group_arr.length - 1];
      }
    };

    Console.prototype.groupEnd = function(value) {
      if (this.enabled) {
        return this.el = this.el_box.firstChild;
      }
    };

    Console.prototype.toggle = function() {
      if (this.enabled) {
        this.el_box.style.display = (this.visible ? "none" : "block");
        this.visible = !this.visible;
        return this.el.scrollTop = this.el.scrollHeight;
      }
    };

    Console.prototype.scroll_up = function() {
      if (this.enabled) {
        return this.el.scrollTop -= this.config.scroll_step;
      }
    };

    Console.prototype.scroll_down = function() {
      if (this.enabled) {
        return this.el.scrollTop += this.config.scroll_step;
      }
    };

    Console.prototype.data_processing = function(value, open_tag, close_tag) {
      var html;
      html = this.log_html(value);
      this.el.innerHTML += open_tag + html + close_tag;
      return this.el.scrollTop = this.el.scrollHeight;
    };

    Console.prototype.log_html = function(value, indent) {
      if (indent == null) {
        indent = 2;
      }
      if (typeof value === 'object') {
        return this.log_as_object(value);
      } else {
        return this.log_as_nonobject(value);
      }
    };

    Console.prototype.log_as_number = function(value) {
      return "<span class='blue'>" + (value.toString()) + "</span>";
    };

    Console.prototype.log_as_string = function(value) {
      return "<span class='red'>\"" + value + "\"</span>";
    };

    Console.prototype.log_as_boolean = function(value) {
      return "<span class='blue'>" + (value.toString()) + "</span>";
    };

    Console.prototype.log_as_undefined = function(value) {
      return "<span class='grey'>undefined</span>";
    };

    Console.prototype.log_as_nan = function(value) {
      return "<span class='blue'>NaN</span>";
    };

    Console.prototype.log_as_function = function(value) {
      return "[function]";
    };

    Console.prototype.log_as_nonobject = function(value) {
      var html;
      html = '';
      if (this.is_number(value)) {
        html = this.log_as_number(value);
      } else if (this.is_string(value)) {
        html = this.log_as_string(value);
      } else if (this.is_boolean(value)) {
        html = this.log_as_boolean(value);
      } else if (this.is_undefined(value)) {
        html = this.log_as_undefined(value);
      } else if (this.is_NaN(value)) {
        html = this.log_as_nan(value);
      } else if (this.is_function(value)) {
        html = this.log_as_function(value);
      }
      return html;
    };

    Console.prototype.log_as_second_level = function(value) {
      if (typeof value === 'object') {
        return '[object]';
      } else {
        return this.log_as_nonobject(value);
      }
    };

    Console.prototype.log_as_object = function(source, indent_count) {
      var html, i, key, value, _i, _j, _ref;
      if (indent_count == null) {
        indent_count = 2;
      }
      if (this.is_null(source)) {
        return "<span class='grey'>null</span>";
      } else if (this.is_date(source)) {
        return "<span class='blue'>" + source + "</span>";
      } else {
        html = "{\n";
        for (key in source) {
          value = source[key];
          if (!(key === 'innerHTML' || key === 'outerHTML')) {
            for (i = _i = 1; 1 <= indent_count ? _i <= indent_count : _i >= indent_count; i = 1 <= indent_count ? ++_i : --_i) {
              html += " ";
            }
            html += "<span class='purple'>" + key + "</span>: " + (this.log_as_object_keys[key] === true ? this.log_as_object(value, indent_count + 2) : this.log_as_second_level(value)) + "\n";
          }
        }
        if (indent_count > 2) {
          for (i = _j = 1, _ref = indent_count - 2; 1 <= _ref ? _j <= _ref : _j >= _ref; i = 1 <= _ref ? ++_j : --_j) {
            html += " ";
          }
        }
        return html += "}";
      }
    };

    Console.prototype.is_number = function(obj) {
      return this.toString.call(obj) === '[object Number]';
    };

    Console.prototype.is_date = function(obj) {
      return this.toString.call(obj) === '[object Date]';
    };

    Console.prototype.is_string = function(obj) {
      return this.toString.call(obj) === '[object String]';
    };

    Console.prototype.is_boolean = function(obj) {
      return obj === true || obj === false || this.toString.call(obj) === '[object Boolean]';
    };

    Console.prototype.is_undefined = function(obj) {
      return obj === void 0;
    };

    Console.prototype.is_NaN = function(obj) {
      return obj === null;
    };

    Console.prototype.is_function = function(obj) {
      return this.toString.call(obj) === '[object Function]';
    };

    Console.prototype.is_null = function(obj) {
      return obj === null;
    };

    Console.prototype.init = function(root_selector) {
      this.toString = Object.prototype.toString;
      this.el_box = document.getElementById(root_selector);
      this.el = this.el_box.firstChild;
      this.visible = false;
      this.timer_data = {};
      return this.log_as_object_keys = {
        response: true
      };
    };

    Console.prototype.show_handler = function() {
      return this.key_trap.on('key_press', this.keypress_handler, this);
    };

    Console.prototype.keypress_handler = function(key) {
      if (key === Rutv.Key.Info) {
        return this.toggle();
      }
    };

    return Console;

  })();

}).call(this);
(function() {

  Rutv.Lib.AjaxWrapper = (function() {

    function AjaxWrapper() {}

    AjaxWrapper.prototype.send = function(options) {
      var params;
      params = {
        url: options.url,
        type: this.value_or_default(options.method, 'GET'),
        dataType: this.value_or_default(options.dataType, 'json'),
        success: this.value_or_default(options.success, this.empty),
        error: this.value_or_default(options.error, this.empty),
        crossDomain: true
      };
      if (options.data != null) {
        params.data = options.data;
      }
      if (options.headers != null) {
        params.headers = options.headers;
      }
      return $.ajax(params);
    };

    AjaxWrapper.prototype.value_or_default = function(value, default_value) {
      if (value != null) {
        return value;
      } else {
        return default_value;
      }
    };

    AjaxWrapper.prototype.empty = function() {};

    return AjaxWrapper;

  })();

}).call(this);
(function() {

  Rutv.Lib.DeferredWrapper = (function() {

    function DeferredWrapper() {
      this.deferred = new Deferred();
    }

    DeferredWrapper.prototype.resolve = function(val) {
      return this.deferred.resolve(val);
    };

    DeferredWrapper.prototype.reject = function(val) {
      return this.deferred.reject(val);
    };

    DeferredWrapper.prototype.done = function(fun) {
      this.deferred.done(fun);
      return this;
    };

    DeferredWrapper.prototype.fail = function(fun) {
      this.deferred.fail(fun);
      return this;
    };

    DeferredWrapper.prototype.when = function(object) {
      this.deferred.when(object);
      return this;
    };

    DeferredWrapper.prototype.promise = function() {
      return this.deferred.promise();
    };

    DeferredWrapper.prototype.is_resolved = function() {
      return this.deferred.isResolved();
    };

    return DeferredWrapper;

  })();

}).call(this);
(function() {

  Rutv.Lib.Element = (function() {

    function Element(platform, node) {
      this.platform = platform;
      this.node = node != null ? node : document.createElement('div');
      this.attach_mouse_handlers();
    }

    Element.prototype.show = function() {
      var result;
      result = new Rutv.Lib.DeferredWrapper();
      this.node.style.display = 'block';
      result.resolve();
      return result;
    };

    Element.prototype.hide = function() {
      var result;
      result = new Rutv.Lib.DeferredWrapper();
      this.node.style.display = 'none';
      result.resolve();
      return result;
    };

    Element.prototype.append = function(child_element) {
      this.node.appendChild(child_element.node);
      return this;
    };

    Element.prototype.prepend = function(child_element) {
      this.node.insertBefore(child_element.node, this.node.firstChild);
      return this;
    };

    Element.prototype.remove = function(child_element) {
      this.node.removeChild(child_element.node);
      return this;
    };

    Element.prototype.destroy = function() {
      var _ref;
      if ((_ref = this.node.parentNode) != null) {
        _ref.removeChild(this.node);
      }
      return this;
    };

    Element.prototype.find_value_attribute = function(elem, attribute) {
      var attr, result, _i, _len, _ref;
      result = null;
      if (elem.attributes) {
        _ref = elem.attributes;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          attr = _ref[_i];
          if (attr.name === attribute) {
            result = attr.value;
            break;
          }
        }
      }
      return result;
    };

    Element.prototype.find_by_role = function(node, role, item) {
      var elem, elements, _i, _len, _results;
      elements = node.children;
      _results = [];
      for (_i = 0, _len = elements.length; _i < _len; _i++) {
        elem = elements[_i];
        if (this.find_value_attribute(elem, 'role') === role) {
          item.push(elem);
        }
        if (elem.children != null) {
          _results.push(this.find_by_role(elem, role, item));
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    Element.prototype.find = function(role) {
      var item, selector;
      item = [];
      if (this.node.querySelectorAll) {
        selector = "[role~='" + role + "']";
        item = this.node.querySelectorAll(selector);
      } else {
        this.find_by_role(this.node, role, item);
      }
      if (item.length > 1) {
        throw "" + item.length + " elements found by '" + role + "'. should be 0 or 1";
      }
      return new Rutv.Lib.Element(this.platform, item[0]);
    };

    Element.prototype.set_content = function(template, data) {
      if (data == null) {
        data = {};
      }
      this.node.innerHTML = Rutv.Utils.template(template, data);
      return this;
    };

    Element.prototype.add_class = function(new_class) {
      $(this.node).addClass(new_class);
      return this;
    };

    Element.prototype.remove_class = function(class_to_remove) {
      $(this.node).removeClass(class_to_remove);
      return this;
    };

    Element.prototype.set_class = function(class_name) {
      this.node.className = class_name;
      return this;
    };

    Element.prototype.set_classname = function(classname) {
      this.node.className = classname;
      return this;
    };

    Element.prototype.toggle_class = function(classname) {
      $(this.node).toggleClass(classname);
      return this;
    };

    Element.prototype.set_role = function(role) {
      return this.node.setAttribute('role', role);
    };

    Element.prototype.set_width = function(width) {
      return this.node.style.width = "" + width + "px";
    };

    Element.prototype.set_background = function(background) {
      return this.node.style.backgroundImage = "url(" + background + ")";
    };

    Element.prototype.attach_mouse_handlers = function() {
      var _this = this;
      return this.platform.mouse_handler(this.node, function() {
        return _this.trigger('mouse_in');
      }, function() {
        return _this.trigger('mouse_out');
      }, function() {
        return _this.trigger('press');
      }, function() {
        return _this.trigger('mouse_up');
      }, function() {
        return _this.trigger('mouse_move');
      });
    };

    Element.prototype.empty = function() {
      this.clear();
      return this;
    };

    Element.prototype.clear = function() {
      return this.node.innerHTML = "";
    };

    return Element;

  })();

  Rutv.Utils.extend(Rutv.Lib.Element.prototype, Rutv.Utils.Events);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Lib.ImageElement = (function(_super) {

    __extends(ImageElement, _super);

    function ImageElement() {
      return ImageElement.__super__.constructor.apply(this, arguments);
    }

    ImageElement.prototype.set_src = function(src) {
      var set_src_deffered;
      set_src_deffered = new Rutv.Lib.DeferredWrapper();
      this.node.src = src;
      this.node.onerror = function() {
        return set_src_deffered.reject();
      };
      return set_src_deffered.promise();
    };

    return ImageElement;

  })(Rutv.Lib.Element);

}).call(this);
(function() {

  Rutv.View.Templates.HTML = {
    app: '\
     <div class="layout_padding">\
      <div class="window__container" role="window__container"></div>\
      <div class="modal__container" role="modal__container" style="display:none;"></div>\
     </div>',
    channel_by_number_selector: '<div><table>\
      <tr><td class="number"><%= number %></td></tr>\
      <tr><td class="hint"><%= hint %></td></tr>\
    </table></div>',
    login: '\
        <table>\
          <tr>\
            <td align="center" valign="middle" class="code__container">\
              <div class="code-content">\
                <div class="code-info">\
                  <h2><%= heading %></h2>\
                  <%= instructions_html %>\
                  <table class="code-numder__table">\
                    <tr>\
                      <td align="center" valign="middle" class="code-numder__container">\
                        <div class="shadow__container">\
                            <div class="shadow"></div>\
                            <div class="pincode" role="code"></div>\
                        </div>\
                      </td>\
                    </tr>\
                  </table>\
                </div>\
                <div class="footer_message"><%= hint %></div>\
              </div>\
            </td>\
          </tr>\
        </table>',
    home: {
      menu: {
        button: '<div class="label"><%= label %></div><div class="bottom-line"></div>'
      },
      account: '\
          <p><%= display_name %></p>\
          <p><%= payment_status %></p>\
          <p><%= user_timezone %></p>\
          <p><%= extend_payment_html %></p>',
      tv: {
        random_channel: {
          loading: '<p><%= loading_please_wait %></p>',
          base: '\
          <img src="<%= screen_image %>">\
          <div class="channel-button">\
            <div class="title">\
              <div class="name">\
                <%= name %>\
              </div>\
              <div class="description"><%= description %></div>\
            </div>\
            <div class="channel_logo"><img src="<%= logo_image %>"></div>\
          </div>'
        },
        channels: '\
        <span class="count"><%= count %></span>\
        <span class="label"><%= label %></span>'
      }
    },
    channels: {
      header: '\
      <%= label %>\
      <% if (total !== null && total > 0) { %>\
        —\
        <strong><%= current %></strong>\
        <%= of %>\
        <strong><%= total %></strong>\
      <% } %>',
      list: {
        loading: '<p class="loading"><%= loading_please_wait %></p>',
        empty: '<p class="loading"><%= sorry_html %></p>',
        channel: '\
        <table class="screen_image__container" align="center" valign="middle">\
          <tr>\
            <td>\
              <img src="<%= screen_image %>">\
            </td>\
          </tr>\
        </table>\
        <div class="channel-button">\
          <div class="title">\
            <div class="name">\
              <%= name %>\
            </div>\
            <div class="description"><%= description %></div>\
          </div>\
          <div class="channel_logo"><img src="<%= logo_image %>"></div>\
        </div>'
      }
    },
    queue: {
      list: {
        loading: '<p class="loading"><%= loading_please_wait %></p>',
        empty: '<p class="loading"><%= sorry_html %></p>',
        programm: '\
        <table class="screen_image__container" align="center" valign="middle">\
          <tr>\
            <td>\
              <img src="<%= thumb %>">\
            </td>\
          </tr>\
        </table>\
        <div class="channel-button">\
          <div class="title">\
            <div class="name">\
              <%= title %>\
            </div>\
            <div class="description"><%= sub_title %></div>\
          </div>\
        </div>'
      }
    },
    player: {
      lg: '\
        <object\
          role="video-object"\
          type="application/x-netcast-av"\
          autoStart="true"\
          drm_type="wm-drm"\
          preBufferingTime="5"\
          data="">\
          <param name="WMode" value="Transparent">\
        </object>',
      pc: '<video src="" role="video-object"></video>',
      samsung: {
        video_player: '<object id="pluginPlayer" class="pluginPlayer" role="video-object" classid="clsid:SAMSUNG-INFOLINK-PLAYER" style="position: absolute;top: 0;left: 0;width: 1280px;height: 720px"></object>',
        volume_control: '\
        <div class="volume-indicator__text-value" role="volume-indicator__text-value"></div>\
        <div class="volume-indicator__band__container">\
          <div class="volume-indicator__band" role="volume-indicator__band"></div>\
        </div>\
        '
      },
      band: '\
      <div class="player__controls__band__current__time" role="player__controls__band__current__time"></div>\
      <div class="player__controls__band" role="player__controls__band">\
        <div class="player__controls__band__offset" role="player__controls__band__offset"></div>\
        <div class="player__controls__band__mark" role="player__controls__band__mark"></div>\
      </div>\
      <div class="player__controls__band__ending__time" role="player__controls__band__ending__time"></div>'
    },
    schedule: {
      channels: {
        loading: '<p><%= loading_please_wait_html %></p>',
        viewport: '\
        <h1><%= heading %></h1>\
        <div class="schedule__channels__viewport" role="schedule__channels__viewport">\
          <div class="schedule__channels__programms__container">\
            <div class="schedule__channels__programms" role="channel-programms"></div>\
          </div>\
          <div class="schedule__channels__logos" role="channel-logos">\
          <% for (var i = 0, l = logo_urls.length; i < l; i++) { %>\
            <img class="schedule__channel__logo" src="<%= logo_urls[i] %>">\
          <% } %>\
          </div>\
        </div>',
        programm: '<%= title %>',
        programms: {
          not_avalible: '<%= programms_not_avalible %>'
        }
      },
      about_programm: '\
      <div class="schedule__about-programm__thumb__container"><img src="" class="schedule__about-programm__thumb" role="schedule__about-programm__thumb"></div>\
      <h2><%= title %></h2>\
      <% if (description) { %>\
        <div class="schedule__about-programm__description"><%= description %></div>\
      <% } %>\
      <p>\
        <%= beginning_label %> – <%= ending_label %>\
        <%= available ? \'\' : \'<span class="unavailable">\' + unavailable_label + \'</span>\'%>\
      </p>\
    '
    },
    modal: '\
    <table>\
      <tr>\
        <td>\
          <div class="modal__content">\
            <h1><%= heading %></h1>\
            <%= description_html ? description_html : "" %>\
            <div class="modal__buttons" role="buttons"></div>\
          </div>\
        </td>\
      </tr>\
    </table>'
  };

}).call(this);
(function() {

  Rutv.View.Base = (function() {

    function Base(platform) {
      this.platform = platform;
      this.templates = this.platform.templates;
      this.key_trap = this.platform.key_trap;
    }

    Base.prototype.show = function() {
      this.el.show();
      return this;
    };

    Base.prototype.hide = function() {
      this.el.hide();
      return this;
    };

    Base.prototype.render = function() {
      throw "should be implemented in subclass";
    };

    /** @private
    */


    Base.prototype.template = function() {
      var code;
      if (this.template_name.indexOf('.') > -1) {
        code = "this.platform.templates." + this.template_name;
        return eval(code);
      } else {
        return this.platform.templates[this.template_name];
      }
    };

    /** @private
    */


    Base.prototype.template_data = function() {
      return {};
    };

    /** @private
    */


    Base.prototype.el_factory = function() {
      return new Rutv.Lib.Element(this.platform);
    };

    Base.prototype.destroy = function() {
      var _ref;
      if (typeof this.off === "function") {
        this.off();
      }
      if ((_ref = this.el) != null) {
        _ref.destroy();
      }
      return this;
    };

    return Base;

  })();

}).call(this);
(function() {

  Rutv.Platform.Storage.Generic = (function() {

    function Generic() {}

    Generic.prototype.get = function(key) {
      throw "should be implemented in subclass";
    };

    Generic.prototype.set = function(key, value) {
      throw "should be implemented in subclass";
    };

    Generic.prototype.remove = function(key) {
      throw "should be implemented in subclass";
    };

    Generic.prototype.clear = function() {
      throw "should be implemented in subclass";
    };

    return Generic;

  })();

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Platform.Storage.Cookies = (function(_super) {

    __extends(Cookies, _super);

    function Cookies() {
      this.data = false;
    }

    Cookies.prototype.get = function(key) {
      if (this.data === false) {
        this.load();
      }
      return this.data[key];
    };

    Cookies.prototype.set = function(key, value) {
      if (this.data === false) {
        this.load();
      }
      if (value != null) {
        this.data[key] = value;
      } else {
        delete this.data[key];
      }
      return this.save();
    };

    Cookies.prototype.remove = function(key) {
      return this.set(key);
    };

    Cookies.prototype.clear = function() {
      this.data = {};
      return this.save();
    };

    Cookies.prototype.load = function() {
      var cookie_data, i, unescaped_data, _i, _ref, _results;
      this.data = {};
      cookie_data = document.cookie.match(/data=(.+?)(;|$)/);
      if ((cookie_data != null) && cookie_data[1].length > 0) {
        unescaped_data = unescape(cookie_data[1]).split('|');
        _results = [];
        for (i = _i = 0, _ref = unescaped_data.length - 1; _i <= _ref; i = _i += 2) {
          if ((unescaped_data[i] != null) && (unescaped_data[i + 1] != null)) {
            _results.push(this.data[unescaped_data[i]] = unescaped_data[i + 1]);
          } else {
            _results.push(void 0);
          }
        }
        return _results;
      }
    };

    Cookies.prototype.save = function() {
      var cookie_string;
      cookie_string = "data=" + (this.escaped_data()) + (this.expires());
      return document.cookie = cookie_string;
    };

    Cookies.prototype.escaped_data = function() {
      var data, key, value, _ref;
      data = '';
      _ref = this.data;
      for (key in _ref) {
        value = _ref[key];
        data += "" + key + "|" + value + "|";
      }
      return escape(data);
    };

    Cookies.prototype.expires = function() {
      var date, result;
      date = new Date();
      date.setTime(date.getTime() + (30 * 24 * 60 * 60 * 1000));
      result = "; expires=" + (date.toGMTString()) + ";";
      return result;
    };

    return Cookies;

  })(Rutv.Platform.Storage.Generic);

}).call(this);
(function() {

  Rutv.Platform.Generic.Platform = (function() {

    function Platform(config) {
      this.config = config;
      this.templates = this.templates_factory();
      this.key_trap = this.key_trap_factory();
    }

    Platform.prototype.get_device_id = function() {
      throw "get_device_id() should be implemented in subclasses";
    };

    Platform.prototype.get_device_type = function() {
      throw "get_device_type() should be implemented in subclasses";
    };

    Platform.prototype.get_el = function() {
      return new Rutv.Lib.Element(this, document.getElementsByTagName('body')[0].firstChild);
    };

    Platform.prototype.mouse_handler = function(el, in_handler, out_handler, click_handler, up_handler, move_handler) {
      this.attach_mouse_handler(el, 'onmouseover', in_handler);
      this.attach_mouse_handler(el, 'onmouseout', out_handler);
      this.attach_mouse_handler(el, 'onmousedown', click_handler);
      this.attach_mouse_handler(el, 'onmouseup', up_handler);
      return this.attach_mouse_handler(el, 'onmousemove', move_handler);
    };

    Platform.prototype.attach_mouse_handler = function(el, event, handler) {
      if (handler != null) {
        return el[event] = function(event) {
          event.preventDefault();
          event.stopPropagation();
          return handler();
        };
      }
    };

    Platform.prototype.exit = function() {
      throw "exit() should be implemented in subclasses";
    };

    Platform.prototype.key_trap_factory = function() {
      return new Rutv.Platform.KeyTrap();
    };

    Platform.prototype.templates_factory = function() {
      return Rutv.View.Templates.HTML;
    };

    Platform.prototype.storage_factory = function() {
      return new Rutv.Platform.Storage.Cookies();
    };

    Platform.prototype.init_console = function() {
      this.console = new Rutv.Debug.Console(this.config.console, this.key_trap);
      this.console.info('HTML console inited');
      return window.console = document.console = this.console;
    };

    return Platform;

  })();

  Rutv.Utils.extend(Rutv.Platform.Generic.Platform.prototype, Rutv.Utils.Events);

}).call(this);
(function() {

  Rutv.Key = {
    Left: 10,
    Right: 20,
    Up: 30,
    Down: 40,
    Red: 50,
    Green: 60,
    Yellow: 70,
    Blue: 81,
    Ok: 82,
    Info: 83,
    Esc: 84,
    Exit: 85,
    Button1: 91,
    Button2: 92,
    Button3: 93,
    Button4: 94,
    Button5: 95,
    Button6: 96,
    Button7: 97,
    Button8: 98,
    Button9: 99,
    Button0: 90,
    Refresh: 88,
    Back: 71,
    Play: 72,
    Pause: 73,
    Stop: 74,
    Rewind: 75,
    Forward: 76,
    VolumeUp: 77,
    VolumeDown: 78,
    VolumeMute: 79
  };

  Rutv.Platform.Generic.KeyTrap = (function() {

    function KeyTrap() {
      this.attach_handler();
    }

    KeyTrap.prototype.attach_handler = function() {
      var _this = this;
      return document.onkeydown = function(event) {
        return _this.keypress_handler(event);
      };
    };

    KeyTrap.prototype.keypress_handler = function(event) {
      var key;
      key = this.get_key(event);
      if ((key != null) && this.trigger('key_press', key)) {
        return this.trigger('after_key_press', key);
      }
    };

    KeyTrap.prototype.get_key = function(event) {
      throw "get_key() should be implemented in subclasses";
    };

    return KeyTrap;

  })();

  Rutv.Utils.extend(Rutv.Platform.Generic.KeyTrap.prototype, Rutv.Utils.Events);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Platform.Generic.Player = (function(_super) {

    __extends(Player, _super);

    function Player(platform) {
      this.platform = platform;
      Player.__super__.constructor.call(this, this.platform);
      this.current_play_time = 0;
      this.play_time_deffered = new Rutv.Lib.DeferredWrapper();
    }

    Player.prototype.render = function() {
      if (this.template_name != null) {
        return (this.el = this.el_factory()).set_content(this.template(), this.template_data());
      }
    };

    Player.prototype.rewind = function() {};

    Player.prototype.forward = function() {};

    Player.prototype.toggle_ratio = function() {};

    Player.prototype.get_ratio_status = function() {
      return '';
    };

    Player.prototype.move_to_time = function(time) {};

    Player.prototype.on_stream_info_ready = function() {};

    Player.prototype.on_buffering_complete = function() {};

    Player.prototype.get_current_play_time = function() {
      return this.current_play_time;
    };

    Player.prototype.on_current_play_time = function(handler) {
      if (handler != null) {
        handler.call;
      }
      return this.play_time_deffered.promise();
    };

    Player.prototype.capabilities_with_rewind = function() {
      return ['play', 'prev-channel', 'rewind', 'toggle_ratio', 'forward', 'next-channel', 'exit'];
    };

    Player.prototype.capabilities_without_rewind = function() {
      return ['play', 'prev-channel', 'toggle_ratio', 'next-channel', 'exit'];
    };

    Player.prototype.destroy = function() {
      return this;
    };

    return Player;

  })(Rutv.View.Base);

  Rutv.Utils.extend(Rutv.Platform.Generic.Player.prototype, Rutv.Utils.Events);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Platform.LG.KeyTrap = (function(_super) {

    __extends(KeyTrap, _super);

    function KeyTrap() {
      KeyTrap.__super__.constructor.call(this);
    }

    /** private
    */


    KeyTrap.prototype.get_key = function(event) {
      switch (event.keyCode) {
        case 37:
          return Rutv.Key.Left;
        case 38:
          return Rutv.Key.Up;
        case 39:
          return Rutv.Key.Right;
        case 40:
          return Rutv.Key.Down;
        case 403:
          return Rutv.Key.Red;
        case 404:
          return Rutv.Key.Green;
        case 405:
          return Rutv.Key.Yellow;
        case 406:
          return Rutv.Key.Blue;
        case 457:
          return Rutv.Key.Info;
        case 13:
          return Rutv.Key.Ok;
        case 49:
          return Rutv.Key.Button1;
        case 50:
          return Rutv.Key.Button2;
        case 51:
          return Rutv.Key.Button3;
        case 52:
          return Rutv.Key.Button4;
        case 53:
          return Rutv.Key.Button5;
        case 54:
          return Rutv.Key.Button6;
        case 55:
          return Rutv.Key.Button7;
        case 56:
          return Rutv.Key.Button8;
        case 57:
          return Rutv.Key.Button9;
        case 48:
          return Rutv.Key.Button0;
        case 461:
          return Rutv.Key.Back;
        case 415:
          return Rutv.Key.Play;
        case 19:
          return Rutv.Key.Pause;
        case 413:
          return Rutv.Key.Stop;
        case 412:
          return Rutv.Key.Rewind;
        case 417:
          return Rutv.Key.Forward;
        default:
          return null;
      }
    };

    return KeyTrap;

  })(Rutv.Platform.Generic.KeyTrap);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Platform.LG.Player = (function(_super) {

    __extends(Player, _super);

    Player.prototype.template_name = 'player.lg';

    function Player(platform, config) {
      this.platform = platform;
      this.config = config;
      Player.__super__.constructor.call(this, this.platform);
      this.play_state = null;
      this.timer = 0;
    }

    Player.prototype.render = function() {
      Player.__super__.render.call(this);
      this.video = this.el.find('video-object');
      return this;
    };

    Player.prototype.start_timer = function() {
      var timer_interval,
        _this = this;
      this.pause_timer();
      return timer_interval = setInterval(function() {
        return _this.timer += 1;
      }, 1000);
    };

    Player.prototype.pause_timer = function() {
      return clearInterval(this.timer_interval);
    };

    Player.prototype.stop_timer = function() {
      clearInterval(this.timer_interval);
      return this.timer = 0;
    };

    Player.prototype.timer_move_to = function(offset) {
      return this.timer += offset;
    };

    Player.prototype.resume = function() {
      this.start_timer();
      return this.video.node.play(1);
    };

    Player.prototype.pause = function() {
      this.pause_timer();
      return this.video.node.play(0);
    };

    Player.prototype.play = function(url) {
      var _this = this;
      this.stop_timer();
      this.start_timer();
      this.on_play_state_change(function() {
        _this.play_time_deffered.resolve();
        return _this.play_state_handler();
      });
      this.el.remove(this.video);
      this.video.node.data = url;
      return this.el.append(this.video);
    };

    Player.prototype.rewind = function() {
      var position_to_set;
      this.timer_move_to(-this.config.player.rewind_interval);
      position_to_set = this.get_current_play_time() - this.config.player.rewind_interval;
      if (position_to_set >= 0) {
        return this.move_to_time(position_to_set);
      } else {
        return this.move_to_time(0);
      }
    };

    Player.prototype.forward = function() {
      var position_to_set;
      this.timer_move_to(this.config.player.rewind_interval);
      position_to_set = this.get_current_play_time() + this.config.player.rewind_interval;
      if (position_to_set <= this.get_duration()) {
        return this.move_to_time(position_to_set);
      } else {
        return this.move_to_time(this.get_duration());
      }
    };

    Player.prototype.get_duration = function() {
      return Math.round(parseFloat(this.video.node.playTime) / 1000);
    };

    Player.prototype.get_current_play_time = function() {
      return Math.round(parseFloat(this.video.node.playPosition) / 1000);
    };

    Player.prototype.on_stream_info_ready = function(handler) {
      return handler.call();
    };

    Player.prototype.on_play_state_change = function(handler) {
      return this.video.node.onPlayStateChange = handler;
    };

    Player.prototype.move_to_time = function(time) {
      return this.video.node.seek(time * 1000);
    };

    Player.prototype.capabilities_with_rewind = function() {
      return ['play', 'prev-channel', 'rewind', 'forward', 'next-channel', 'exit'];
    };

    Player.prototype.capabilities_without_rewind = function() {
      return ['play', 'prev-channel', 'next-channel', 'exit'];
    };

    Player.prototype.play_state_handler = function() {
      if (this.video.node.playState === 5) {
        return this.trigger('finished');
      }
    };

    Player.prototype.get_timer = function() {
      return this.timer;
    };

    Player.prototype.destroy = function() {
      Player.__super__.destroy.call(this);
      this.stop_timer();
      return this;
    };

    return Player;

  })(Rutv.Platform.Generic.Player);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Platform.LG.Platform = (function(_super) {

    __extends(Platform, _super);

    function Platform(config) {
      this.config = config;
      Platform.__super__.constructor.call(this, this.config);
      this.get_el().add_class('lg');
      this.storage = this.storage_factory();
      this.device = this.create_device_element();
      this.init_console();
    }

    Platform.prototype.get_device_id = function() {
      return this.device.serialNumber;
    };

    Platform.prototype.get_device_type = function() {
      return "LG TV";
    };

    Platform.prototype.key_trap_factory = function() {
      return new Rutv.Platform.LG.KeyTrap();
    };

    Platform.prototype.create_device_element = function() {
      var element;
      element = document.createElement("div");
      element.innerHTML = '<object type="application/x-netcast-info"></object>';
      document.body.appendChild(element);
      return element.firstChild;
    };

    Platform.prototype.player_factory = function() {
      return (this.player = new Rutv.Platform.LG.Player(this, this.config)).render();
    };

    Platform.prototype.exit = function() {
      return window.NetCastBack();
    };

    return Platform;

  })(Rutv.Platform.Generic.Platform);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Platform.Samsung.KeyTrap = (function(_super) {

    __extends(KeyTrap, _super);

    function KeyTrap(widgetAPI, platfrom) {
      this.widgetAPI = widgetAPI;
      this.platfrom = platfrom;
      KeyTrap.__super__.constructor.call(this);
      this.tvKey = new Common.API.TVKeyValue();
    }

    /** private
    */


    KeyTrap.prototype.get_key = function(event) {
      switch (event.keyCode) {
        case this.tvKey.KEY_LEFT:
          return Rutv.Key.Left;
        case this.tvKey.KEY_UP:
          return Rutv.Key.Up;
        case this.tvKey.KEY_RIGHT:
          return Rutv.Key.Right;
        case this.tvKey.KEY_DOWN:
          return Rutv.Key.Down;
        case this.tvKey.KEY_RED:
          return Rutv.Key.Red;
        case this.tvKey.KEY_GREEN:
          return Rutv.Key.Green;
        case this.tvKey.KEY_YELLOW:
          return Rutv.Key.Yellow;
        case this.tvKey.KEY_BLUE:
          return Rutv.Key.Blue;
        case this.tvKey.KEY_INFO:
          return Rutv.Key.Info;
        case this.tvKey.KEY_ENTER:
          return Rutv.Key.Ok;
        case this.tvKey.KEY_1:
          return Rutv.Key.Button1;
        case this.tvKey.KEY_2:
          return Rutv.Key.Button2;
        case this.tvKey.KEY_3:
          return Rutv.Key.Button3;
        case this.tvKey.KEY_4:
          return Rutv.Key.Button4;
        case this.tvKey.KEY_5:
          return Rutv.Key.Button5;
        case this.tvKey.KEY_6:
          return Rutv.Key.Button6;
        case this.tvKey.KEY_7:
          return Rutv.Key.Button7;
        case this.tvKey.KEY_8:
          return Rutv.Key.Button8;
        case this.tvKey.KEY_9:
          return Rutv.Key.Button9;
        case this.tvKey.KEY_0:
          return Rutv.Key.Button0;
        case this.tvKey.KEY_RETURN:
          this.widgetAPI.blockNavigation(event);
          return Rutv.Key.Back;
        case this.tvKey.KEY_PLAY:
          return Rutv.Key.Play;
        case this.tvKey.KEY_PAUSE:
          return Rutv.Key.Pause;
        case this.tvKey.KEY_STOP:
          return Rutv.Key.Stop;
        case this.tvKey.KEY_RW:
          return Rutv.Key.Rewind;
        case this.tvKey.KEY_FF:
          return Rutv.Key.Forward;
        case this.tvKey.KEY_VOL_UP:
          return Rutv.Key.VolumeUp;
        case this.tvKey.KEY_VOL_DOWN:
          return Rutv.Key.VolumeDown;
        case this.tvKey.KEY_MUTE:
          return Rutv.Key.VolumeMute;
        case this.tvKey.KEY_EXIT:
          this.widgetAPI.blockNavigation(event);
          return Rutv.Key.Exit;
        default:
          return null;
      }
    };

    return KeyTrap;

  })(Rutv.Platform.Generic.KeyTrap);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Platform.Samsung.Player = (function(_super) {

    __extends(Player, _super);

    Player.prototype.template_name = 'player.samsung.video_player';

    function Player(platform, config) {
      this.platform = platform;
      this.config = config;
      Player.__super__.constructor.call(this, this.platform);
      this.volume_control = this.platform.volume_control;
      this.ratios = ['4x3', '4x3zoom', '16x9'];
      this.current_ratio_index = 0;
      this.init_rewind_throttles();
    }

    Player.prototype.render = function() {
      Player.__super__.render.call(this);
      this.video = this.el.find('video-object');
      return this;
    };

    Player.prototype.resume = function() {
      return this.video.node.Resume();
    };

    Player.prototype.pause = function() {
      return this.video.node.Pause();
    };

    Player.prototype.play = function(url, video_width, video_height) {
      var _this = this;
      this.video_width = video_width;
      this.video_height = video_height;
      this.on_current_play_time(function(time) {
        _this.current_play_time = time;
        _this.play_time_deffered.resolve();
        return _this.play_state_handler();
      });
      this.video.node.Stop();
      this.set_display_area();
      return this.video.node.Play(url + "|COMPONENT=HLS");
    };

    Player.prototype.init_rewind_throttles = function() {
      var _this = this;
      this.rewind_throttle = throttle(function() {
        return _this.video.node.JumpBackward(_this.config.player.rewind_interval);
      }, this.config.player.samsung.min_rewind_interval);
      return this.forward_throttle = throttle(function() {
        return _this.video.node.JumpForward(_this.config.player.rewind_interval);
      }, this.config.player.samsung.min_rewind_interval);
    };

    Player.prototype.finished_handler = function() {
      if (this.current_play_time >= this.get_duration()) {
        return this.trigger('finished');
      }
    };

    Player.prototype.get_duration = function() {
      return this.video.node.GetDuration();
    };

    Player.prototype.set_display_area = function() {
      var proportion;
      proportion = this.video_width / this.video_height;
      if (proportion < 1.7) {
        this.use_4_to_3_ratio();
        return this.current_ratio_index = 0;
      } else {
        this.use_16_to_9_ratio();
        return this.current_ratio_index = 1;
      }
    };

    Player.prototype.use_4_to_3_ratio = function() {
      return this.video.node.SetDisplayArea(120, 0, 720, 540);
    };

    Player.prototype.use_16_to_9_ratio = function() {
      return this.video.node.SetDisplayArea(0, 0, 960, 540);
    };

    Player.prototype.use_4_to_3_zoom2_ratio = function() {
      this.video.node.SetDisplayArea(0, 0, 960, 540);
      return this.set_crop_area(0, 62, this.width, 416);
    };

    Player.prototype.set_crop_area = function(x, y, width, height) {
      var status;
      return status = this.video.node.SetCropArea(x, y, width, height);
    };

    Player.prototype.toggle_ratio = function() {
      this.toggle_ratio_index();
      this.video.node.SetCropArea(0, 0, 0, 0);
      this.width = this.video.node.GetVideoWidth();
      this.height = this.video.node.GetVideoHeight();
      switch (this.ratios[this.current_ratio_index]) {
        case '4x3':
          return this.use_4_to_3_ratio();
        case '4x3zoom':
          return this.use_4_to_3_zoom2_ratio();
        case '16x9':
          return this.use_16_to_9_ratio();
        default:
          return false;
      }
    };

    Player.prototype.get_ratio_status = function() {
      var ratio;
      ratio = null;
      switch (this.ratios[this.current_ratio_index]) {
        case '4x3':
          ratio = '4x3';
          break;
        case '4x3zoom':
          ratio = '4x3 zoom';
          break;
        case '16x9':
          ratio = '16x9';
          break;
        default:
          false;
      }
      return ratio;
    };

    Player.prototype.move_to_time = function(time) {
      return this.video.node.JumpForward(time);
    };

    Player.prototype.toggle_ratio_index = function() {
      if (this.current_ratio_index + 1 > this.ratios.length - 1) {
        return this.current_ratio_index = 0;
      } else {
        return this.current_ratio_index++;
      }
    };

    Player.prototype.rewind = function() {
      return this.rewind_throttle();
    };

    Player.prototype.forward = function() {
      return this.forward_throttle();
    };

    Player.prototype.on_stream_info_ready = function(handler) {
      var _this = this;
      return this.video.node.OnStreamInfoReady = function() {
        return setTimeout(handler, 150);
      };
    };

    Player.prototype.on_buffering_complete = function(handler) {
      return this.video.node.OnBufferingComplete = handler;
    };

    Player.prototype.on_current_play_time = function(handler) {
      if (handler != null) {
        this.video.node.OnCurrentPlayTime = handler;
      }
      return this.play_time_deffered.promise();
    };

    Player.prototype.get_current_play_time = function() {
      return Math.round(parseFloat(this.current_play_time) / 1000);
    };

    Player.prototype.capabilities_with_rewind = function() {
      return ['play', 'prev-channel', 'rewind', 'toggle_ratio', 'forward', 'next-channel', 'exit'];
    };

    Player.prototype.capabilities_without_rewind = function() {
      return ['play', 'prev-channel', 'toggle_ratio', 'next-channel', 'exit'];
    };

    Player.prototype.destroy = function() {
      this.video.node.Stop();
      return this;
    };

    return Player;

  })(Rutv.Platform.Generic.Player);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Platform.Samsung.VolumeControl = (function(_super) {

    __extends(VolumeControl, _super);

    VolumeControl.prototype.template_name = 'player.samsung.volume_control';

    function VolumeControl(platform, config) {
      this.platform = platform;
      this.config = config;
      this.audio_plugin = this.platform.audio_plugin;
      this.restoring_volume = false;
    }

    VolumeControl.prototype.render = function() {
      (this.el = this.el_factory()).set_classname('volume-indicator');
      if (this.template_name != null) {
        this.el.set_content(this.template(), this.template_data());
      }
      this.band_el = this.el.find('volume-indicator__band');
      this.text_el = this.el.find('volume-indicator__text-value');
      return this;
    };

    VolumeControl.prototype.toggle_mute = function() {
      if (this.audio_plugin.GetUserMute()) {
        this.audio_plugin.SetUserMute(0);
      } else {
        this.audio_plugin.SetUserMute(1);
      }
      return this.update_volume_indicator();
    };

    VolumeControl.prototype.louder = function() {
      this.audio_plugin.SetVolumeWithKey(1);
      return this.update_volume_indicator();
    };

    VolumeControl.prototype.quieter = function() {
      this.audio_plugin.SetVolumeWithKey(0);
      return this.update_volume_indicator();
    };

    VolumeControl.prototype.save_volume = function() {
      return this.saved_volume = this.current_volume();
    };

    VolumeControl.prototype.current_volume = function() {
      return this.audio_plugin.GetVolume();
    };

    VolumeControl.prototype.update_volume_indicator = function() {
      var volume;
      this.show();
      volume = this.audio_plugin.GetSystemMute() ? 0 : this.current_volume();
      this.band_el.node.style.top = "" + (300 - volume * 3) + "px";
      return this.text_el.node.innerHTML = volume;
    };

    return VolumeControl;

  })(Rutv.View.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Platform.Samsung.Platform = (function(_super) {

    __extends(Platform, _super);

    function Platform(config) {
      this.config = config;
      this.widgetAPI = new Common.API.Widget();
      Platform.__super__.constructor.call(this, this.config);
      this.storage = this.storage_factory();
      this.audio_plugin = this.get_audio_plugin_element();
      this.network_plugin = this.get_network_plugin_element();
      this.nnavy_plugin = this.get_nnavy_plugin_element();
      this.get_el().add_class('samsung');
      if (this.get_platform_year() === '2011') {
        this.get_el().add_class('no-css3');
        this.redifine_ajax();
      }
      this.init_console();
      this.get_resolution();
      this.volume_control = new Rutv.Platform.Samsung.VolumeControl(this, this.config);
    }

    Platform.prototype.get_device_id = function() {
      return this.nnavy_plugin.GetDUID(this.network_plugin.GetMAC(1));
    };

    Platform.prototype.get_device_type = function() {
      return "SamsungTV";
    };

    Platform.prototype.get_width = function() {
      return parseFloat(this.width);
    };

    Platform.prototype.get_height = function() {
      return parseFloat(this.height);
    };

    Platform.prototype.get_resolution = function() {
      this.width = 1280;
      return this.height = 720;
    };

    Platform.prototype.get_network_plugin_element = function() {
      return document.getElementById('networkPlugin');
    };

    Platform.prototype.get_nnavy_plugin_element = function() {
      return document.getElementById('NNaviPlugin');
    };

    Platform.prototype.get_audio_plugin_element = function() {
      return document.getElementById('audioPlayer');
    };

    Platform.prototype.get_tvmw_plugin_element = function() {
      return document.getElementById('pluginObjectTVMW');
    };

    Platform.prototype.key_trap_factory = function() {
      return new Rutv.Platform.Samsung.KeyTrap(this.widgetAPI, this);
    };

    Platform.prototype.player_factory = function() {
      return (this.player = new Rutv.Platform.Samsung.Player(this, this.config)).render();
    };

    Platform.prototype.exit = function() {
      return this.widgetAPI.sendReturnEvent();
    };

    Platform.prototype.redifine_ajax = function() {
      var _this = this;
      return $.ajax = function(params) {
        var query_string, url, xmlhttp;
        url = _this.remove_auth_parameters(params.url);
        if ((query_string = _this.to_query_string(params.data)) !== '') {
          url = "" + url + "?" + query_string;
        }
        xmlhttp = new XMLHttpRequest();
        xmlhttp.open(params.type, url, true);
        _this.set_request_headers(xmlhttp, params.headers);
        _this.set_authorization_header(xmlhttp, _this.config.server.url);
        xmlhttp.onreadystatechange = function() {
          if (xmlhttp.readyState === 4) {
            if (_this.request_is_success(xmlhttp)) {
              return params.success(json_parse(xmlhttp.responseText));
            } else {
              return params.error(xmlhttp, 'error');
            }
          }
        };
        return xmlhttp.send(null);
      };
    };

    Platform.prototype.request_is_success = function(xmlhttp) {
      var status;
      return ((status = xmlhttp.status) >= 200 && status < 300) || (status === 304) || (status === 0);
    };

    Platform.prototype.set_request_headers = function(xmlhttp, headers) {
      var key, value, _results;
      _results = [];
      for (key in headers) {
        value = headers[key];
        if (key === 'Accept') {
          key = 'X-Accept';
        }
        _results.push(xmlhttp.setRequestHeader(key, value));
      }
      return _results;
    };

    Platform.prototype.to_query_string = function(data) {
      var key, result, value;
      result = [];
      for (key in data) {
        value = data[key];
        result.push("" + key + "=" + value);
      }
      return result.join('&');
    };

    Platform.prototype.get_platform_year = function() {
      var platform_year, reg_exp_year, result_array;
      platform_year = '2011';
      reg_exp_year = /20\d{2}/;
      result_array = reg_exp_year.exec(this.nnavy_plugin.GetFirmware());
      if (result_array !== null) {
        platform_year = result_array[0];
      }
      return platform_year;
    };

    Platform.prototype.set_authorization_header = function(xmlhttp, url) {
      var login, pass;
      login = /\w{1,}(?=\:\w{1,}\@\w{1,})/.exec(url);
      pass = /\w{1,}(?=\@\w{1,})/.exec(url);
      if ((login != null) && (pass != null)) {
        return xmlhttp.setRequestHeader('Authorization', 'Basic ' + Base64.encode(login[0] + ':' + pass[0]));
      }
    };

    Platform.prototype.remove_auth_parameters = function(url) {
      return url.replace(/\w{1,}\:\w{1,}\@(?=\w{1,})/, "");
    };

    return Platform;

  })(Rutv.Platform.Generic.Platform);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Platform.PC.KeyTrap = (function(_super) {

    __extends(KeyTrap, _super);

    function KeyTrap() {
      KeyTrap.__super__.constructor.call(this);
    }

    KeyTrap.prototype.get_key = function(event) {
      switch (event.keyCode) {
        case 37:
          return Rutv.Key.Left;
        case 38:
          return Rutv.Key.Up;
        case 39:
          return Rutv.Key.Right;
        case 40:
          return Rutv.Key.Down;
        case 13:
          return Rutv.Key.Ok;
        case 49:
          return Rutv.Key.Button1;
        case 50:
          return Rutv.Key.Button2;
        case 51:
          return Rutv.Key.Button3;
        case 52:
          return Rutv.Key.Button4;
        case 53:
          return Rutv.Key.Button5;
        case 54:
          return Rutv.Key.Button6;
        case 55:
          return Rutv.Key.Button7;
        case 56:
          return Rutv.Key.Button8;
        case 57:
          return Rutv.Key.Button9;
        case 48:
          return Rutv.Key.Button0;
        case 27:
          return Rutv.Key.Back;
        case 66:
          return Rutv.Key.Blue;
        case 65:
          return Rutv.Key.Red;
        case 80:
          return Rutv.Key.Play;
        case 73:
          return Rutv.Key.Pause;
        case 69:
          return Rutv.Key.Stop;
        case 82:
          return Rutv.Key.Rewind;
        case 70:
          return Rutv.Key.Forward;
        case 89:
          return Rutv.Key.Yellow;
        case 71:
          return Rutv.Key.Green;
        case 81:
          return Rutv.Key.Exit;
        default:
          return null;
      }
    };

    return KeyTrap;

  })(Rutv.Platform.Generic.KeyTrap);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Platform.PC.Player = (function(_super) {

    __extends(Player, _super);

    Player.prototype.template_name = 'player.pc';

    function Player(platform) {
      this.platform = platform;
      Player.__super__.constructor.call(this, this.platform);
      this.current_play_time = 0;
    }

    Player.prototype.render = function() {
      Player.__super__.render.call(this);
      this.video = this.el.find('video-object');
      this.node = this.video.node;
      return this;
    };

    Player.prototype.resume = function() {
      return this.node.play();
    };

    Player.prototype.pause = function() {
      return this.node.pause();
    };

    Player.prototype.play = function(url, video_width, video_height) {
      this.node.src = url;
      this.set_display_area(video_width, video_height);
      return this.node.play();
    };

    Player.prototype.set_display_area = function(video_width, video_height) {
      var platform_height, player_width, player_width_offset, proportion;
      platform_height = this.platform.get_height();
      player_width = Math.round(platform_height / parseInt(video_height) * parseInt(video_width));
      player_width_offset = Math.round((this.platform.get_width() - player_width) / 2);
      proportion = video_width / video_height;
      console.log(video_width, video_height);
      if (proportion < 1.7) {
        return console.log('4:3');
      } else {
        return console.log('16:9');
      }
    };

    return Player;

  })(Rutv.Platform.Generic.Player);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Platform.PC.Platform = (function(_super) {

    __extends(Platform, _super);

    function Platform(config) {
      Platform.__super__.constructor.call(this, config);
      this.get_el().add_class('pc');
      this.storage = this.storage_factory();
    }

    Platform.prototype.get_device_id = function() {
      return "12345678902";
    };

    Platform.prototype.get_device_type = function() {
      return "Lgtv";
    };

    Platform.prototype.get_width = function() {
      return 1280;
    };

    Platform.prototype.get_height = function() {
      return 720;
    };

    Platform.prototype.player_factory = function() {
      return (this.player = new Rutv.Platform.PC.Player(this)).render();
    };

    Platform.prototype.key_trap_factory = function() {
      return new Rutv.Platform.PC.KeyTrap();
    };

    Platform.prototype.exit = function() {
      return window.location.reload();
    };

    return Platform;

  })(Rutv.Platform.Generic.Platform);

}).call(this);
(function() {

  Rutv.Platform.Detector = (function() {

    function Detector(config) {
      this.config = config;
    }

    Detector.prototype.detect = function() {
      switch (window.platform_name) {
        case 'pc':
          return new Rutv.Platform.PC.Platform(this.config);
        case 'lg':
          return new Rutv.Platform.LG.Platform(this.config);
        case 'samsung':
          return new Rutv.Platform.Samsung.Platform(this.config);
        default:
          return null;
      }
    };

    return Detector;

  })();

}).call(this);
(function() {

  Rutv.Server.Requests.Base = (function() {

    function Base(config, path, method) {
      this.config = config;
      this.path = path;
      this.method = method != null ? method : 'GET';
      this.data = {};
      this.headers = this.headers_factory();
    }

    Base.prototype.perform = function() {
      return this.perform_request();
    };

    Base.prototype.update_data = function(params) {};

    Base.prototype.url_factory = function() {
      return "" + this.config.url + this.path;
    };

    Base.prototype.headers_factory = function() {
      return {
        Accept: this.config.accepts
      };
    };

    Base.prototype.perform_request = function() {
      var _this = this;
      this.trigger('start');
      return (new Rutv.Lib.AjaxWrapper()).send({
        url: this.url_factory(),
        method: this.method,
        headers: this.headers,
        dataType: 'json',
        data: this.data,
        success: function(data) {
          _this.trigger('stop');
          return _this.trigger('success', data);
        },
        error: function(xhr, error_message, error_code) {
          _this.trigger('stop');
          return _this.trigger('failure', xhr, error_message, error_code);
        }
      });
    };

    return Base;

  })();

  Rutv.Utils.extend(Rutv.Server.Requests.Base.prototype, Rutv.Utils.Events);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Server.Requests.CheckCode = (function(_super) {

    __extends(CheckCode, _super);

    function CheckCode(config) {
      this.config = config;
      CheckCode.__super__.constructor.call(this, this.config, '/devices/rendezvous/result');
    }

    CheckCode.prototype.update_data = function(params) {
      this.data['device_id'] = params.device_id;
      this.data['device_type'] = params.device_type;
      this.data['code'] = params.code;
      return this;
    };

    return CheckCode;

  })(Rutv.Server.Requests.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Server.Requests.GetCode = (function(_super) {

    __extends(GetCode, _super);

    function GetCode(config) {
      this.config = config;
      GetCode.__super__.constructor.call(this, this.config, '/devices/rendezvous/request');
    }

    GetCode.prototype.update_data = function(params) {
      this.data['device_id'] = params.device_id;
      this.data['device_type'] = params.device_type;
      return this;
    };

    return GetCode;

  })(Rutv.Server.Requests.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Server.Requests.VerifyToken = (function(_super) {

    __extends(VerifyToken, _super);

    function VerifyToken(config) {
      this.config = config;
      VerifyToken.__super__.constructor.call(this, this.config, '/session/verify');
    }

    VerifyToken.prototype.update_data = function(params) {
      this.headers['X-Device-Token'] = params.token;
      return this;
    };

    return VerifyToken;

  })(Rutv.Server.Requests.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Server.Requests.GetChannels = (function(_super) {

    __extends(GetChannels, _super);

    function GetChannels(config) {
      this.config = config;
      GetChannels.__super__.constructor.call(this, this.config, '/live');
    }

    GetChannels.prototype.update_data = function(params) {
      this.headers['X-Device-Token'] = params.token;
      return this;
    };

    return GetChannels;

  })(Rutv.Server.Requests.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Server.Requests.GetSchedule = (function(_super) {

    __extends(GetSchedule, _super);

    function GetSchedule(config, link) {
      this.config = config;
      GetSchedule.__super__.constructor.call(this, this.config, link);
    }

    GetSchedule.prototype.update_data = function(params) {
      this.headers['X-Device-Token'] = params.token;
      return this;
    };

    return GetSchedule;

  })(Rutv.Server.Requests.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Server.Requests.AddFavouritedChannel = (function(_super) {

    __extends(AddFavouritedChannel, _super);

    function AddFavouritedChannel(config, id) {
      this.config = config;
      AddFavouritedChannel.__super__.constructor.call(this, this.config, "/account/channel_favourites/" + id, 'PUT');
    }

    AddFavouritedChannel.prototype.update_data = function(params) {
      this.headers['X-Device-Token'] = params.token;
      this.data['dummy'] = '';
      return this;
    };

    return AddFavouritedChannel;

  })(Rutv.Server.Requests.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Server.Requests.RemoveFavouritedChannel = (function(_super) {

    __extends(RemoveFavouritedChannel, _super);

    function RemoveFavouritedChannel(config, id) {
      this.config = config;
      RemoveFavouritedChannel.__super__.constructor.call(this, this.config, "/account/channel_favourites/" + id, 'DELETE');
    }

    RemoveFavouritedChannel.prototype.update_data = function(params) {
      this.headers['X-Device-Token'] = params.token;
      this.data['dummy'] = '';
      return this;
    };

    return RemoveFavouritedChannel;

  })(Rutv.Server.Requests.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Server.Requests.GetAccount = (function(_super) {

    __extends(GetAccount, _super);

    function GetAccount(config) {
      this.config = config;
      GetAccount.__super__.constructor.call(this, this.config, '/account');
    }

    GetAccount.prototype.update_data = function(params) {
      this.headers['X-Device-Token'] = params.token;
      return this;
    };

    return GetAccount;

  })(Rutv.Server.Requests.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Server.Requests.GetQueue = (function(_super) {

    __extends(GetQueue, _super);

    function GetQueue(config) {
      this.config = config;
      GetQueue.__super__.constructor.call(this, this.config, '/queue');
    }

    GetQueue.prototype.update_data = function(params) {
      this.headers['X-Device-Token'] = params.token;
      return this;
    };

    return GetQueue;

  })(Rutv.Server.Requests.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Server.Requests.UpdateQueueStatus = (function(_super) {

    __extends(UpdateQueueStatus, _super);

    function UpdateQueueStatus(config, airtime_id, playhead_time) {
      this.config = config;
      UpdateQueueStatus.__super__.constructor.call(this, this.config, "/history?airtime_id=" + airtime_id + "&playhead_time=" + playhead_time, 'POST');
    }

    UpdateQueueStatus.prototype.update_data = function(params) {
      this.headers['X-Device-Token'] = params.token;
      this.data['dummy'] = '';
      return this;
    };

    return UpdateQueueStatus;

  })(Rutv.Server.Requests.Base);

}).call(this);
(function() {
  var __slice = [].slice;

  Rutv.Server.Base = (function() {

    function Base(app, config) {
      this.app = app;
      this.config = config;
      this.config = this.app.config.server;
      this.active_requests_count = 0;
    }

    Base.prototype.get_code = function(device_id, device_type) {
      return this.send((new Rutv.Server.Requests.GetCode(this.config)).update_data({
        device_id: device_id,
        device_type: device_type
      }));
    };

    Base.prototype.check_code = function(code, device_id, device_type) {
      return this.send((new Rutv.Server.Requests.CheckCode(this.config)).update_data({
        device_id: device_id,
        device_type: device_type,
        code: code
      }));
    };

    Base.prototype.verify_token = function(token) {
      return this.perform_token_request(token, Rutv.Server.Requests.VerifyToken);
    };

    Base.prototype.get_channels = function(token) {
      return this.perform_token_request(token, Rutv.Server.Requests.GetChannels);
    };

    Base.prototype.get_schedule = function(token, link) {
      return this.perform_token_request(token, Rutv.Server.Requests.GetSchedule, link);
    };

    Base.prototype.add_favourited_channel = function(token, id) {
      return this.perform_token_request(token, Rutv.Server.Requests.AddFavouritedChannel, id);
    };

    Base.prototype.update_queue_status = function(token, airtime_id, playhead_time) {
      return this.perform_token_request(token, Rutv.Server.Requests.UpdateQueueStatus, airtime_id, playhead_time);
    };

    Base.prototype.remove_favourited_channel = function(token, id) {
      return this.perform_token_request(token, Rutv.Server.Requests.RemoveFavouritedChannel, id);
    };

    Base.prototype.get_account = function(token) {
      return this.perform_token_request(token, Rutv.Server.Requests.GetAccount);
    };

    Base.prototype.get_queue = function(token) {
      return this.perform_token_request(token, Rutv.Server.Requests.GetQueue);
    };

    Base.prototype.perform_token_request = function() {
      var constructor, params, token;
      token = arguments[0], constructor = arguments[1], params = 3 <= arguments.length ? __slice.call(arguments, 2) : [];
      return this.send(((function(func, args, ctor) {
        ctor.prototype = func.prototype;
        var child = new ctor, result = func.apply(child, args), t = typeof result;
        return t == "object" || t == "function" ? result || child : child;
      })(constructor, [this.config].concat(__slice.call(params)), function(){})).update_data({
        token: token
      }));
    };

    Base.prototype.send = function(request) {
      var request_result,
        _this = this;
      request_result = new Rutv.Lib.DeferredWrapper();
      request.on('start', this.request_start_handler, this).on('stop', this.request_stop_handler, this).on('success', function(data) {
        return request_result.resolve(data);
      }).on('failure', function(xhr, error_message, error_code) {
        return _this.request_failure_handler(request, request_result, xhr, error_message, error_code);
      }).perform();
      return request_result.promise();
    };

    Base.prototype.show_request_send = function(request) {
      var key, value, _ref, _ref1, _results;
      console.log("new request");
      console.log("request_path: " + request.path);
      console.log("request_method: " + request.method);
      console.log("request_data");
      _ref = request.data;
      for (key in _ref) {
        value = _ref[key];
        console.log("  " + key + ": " + value);
      }
      console.log("request_headers");
      _ref1 = request.headers;
      _results = [];
      for (key in _ref1) {
        value = _ref1[key];
        _results.push(console.log("  " + key + ": " + value));
      }
      return _results;
    };

    Base.prototype.show_fail_request_response = function(xhr) {
      console.log("request fail");
      console.log("xhr_readyState: " + xhr.readyState);
      console.log("xhr_status: " + xhr.status);
      return console.log("xhr_status_text: " + xhr.statusText);
    };

    Base.prototype.request_failure_handler = function(request, request_result_deferred, xhr, error_message, error_code) {
      return this.app.scene.modals.show((new Rutv.View.Modal.Error.RequestFailed(this.app)).on('retry', function() {
        return request.perform();
      }).on('ok', function() {
        return request_result_deferred.fail(xhr, error_message, error_code);
      }));
    };

    Base.prototype.request_start_handler = function() {
      this.active_requests_count += 1;
      return this.trigger('active_requests_count_changed');
    };

    Base.prototype.request_stop_handler = function() {
      this.active_requests_count -= 1;
      return this.trigger('active_requests_count_changed');
    };

    Base.prototype.get_active_requests_count = function() {
      return this.active_requests_count;
    };

    return Base;

  })();

  Rutv.Utils.extend(Rutv.Server.Base.prototype, Rutv.Utils.Events);

}).call(this);
(function() {

  Rutv.Model.Channel.Base = (function() {

    function Base(data, app) {
      this.app = app;
      this.init_fields(data);
    }

    Base.prototype.get_name = function() {
      return this.channel_name;
    };

    Base.prototype.get_streaming_urls = function() {
      return this.streaming_urls;
    };

    Base.prototype.get_current_streaming_url = function() {
      var _ref;
      return (_ref = this.streaming_urls) != null ? _ref[0] : void 0;
    };

    Base.prototype.get_id = function() {
      return this.id;
    };

    Base.prototype.get_logo_url = function() {
      return this.logo_url;
    };

    Base.prototype.get_thumb_url = function() {
      return this.thumb_url;
    };

    Base.prototype.get_title = function() {
      return this.title;
    };

    Base.prototype.get_link = function() {
      return this.link;
    };

    Base.prototype.get_width = function() {
      return this.width;
    };

    Base.prototype.get_height = function() {
      return this.height;
    };

    Base.prototype.get_button = function() {
      return this.button;
    };

    Base.prototype.get_beginning = function() {
      return this.beginning;
    };

    Base.prototype.get_beginning_label = function() {
      return this.beginning_label;
    };

    Base.prototype.get_ending_label = function() {
      return this.ending_label;
    };

    Base.prototype.get_ending = function() {
      return this.ending;
    };

    Base.prototype.get_duration = function() {
      return this.ending - this.beginning;
    };

    Base.prototype.get_tz_offset = function() {
      return this.tz_offset;
    };

    Base.prototype.template_data = function() {
      return {
        name: this.get_name(),
        description: this.get_title(),
        screen_image: this.get_thumb_url(),
        logo_image: this.get_logo_url(),
        button: this.get_button()
      };
    };

    Base.prototype.get_schedule = function() {
      if (this.schedule_deffered == null) {
        this.schedule_deffered = new Rutv.Lib.DeferredWrapper();
        this.load_schedule();
      }
      return this.schedule_deffered.promise();
    };

    Base.prototype.load_schedule = function() {
      var _this = this;
      return this.app.server.get_schedule(this.app.user.get_token(), this.get_link()).done(function(schedule) {
        _this.schedule = new Rutv.Model.Channel.Schedule.Items(schedule, _this, _this.app);
        return _this.schedule_deffered.resolve(_this.schedule);
      }).fail(function() {
        return _this.schedule_deffered.reject();
      });
    };

    Base.prototype.init_fields = function(data) {
      this.favourited = data.favourited;
      this.id = data.id;
      this.logo_url = data.logo;
      this.thumb_url = data.thumb_url;
      this.link = data.link;
      this.channel_name = data.name;
      this.tz_offset = data.tz_offset;
      if (data.telecast != null) {
        this.title = data.telecast.title;
        this.beginning_label = data.telecast.beginning_str;
        this.ending_label = data.telecast.ending_str;
        this.detect_time(data);
      } else {
        this.title = data.name;
      }
      this.button = data.button;
      this.streaming_urls = data.streaming_urls;
      this.width = data.width;
      return this.height = data.height;
    };

    Base.prototype.detect_time = function(data) {
      this.beginning = data.telecast.beginning + this.tz_offset;
      return this.ending = data.telecast.ending + this.tz_offset;
    };

    Base.prototype.is_favourited = function() {
      return this.app.user.favourited_channels.exists(this);
    };

    Base.prototype.toggle_favourited = function() {
      if (this.is_favourited()) {
        this.app.server.remove_favourited_channel(this.app.user.get_token(), this.get_id());
        this.app.user.favourited_channels.remove(this);
      } else {
        this.app.server.add_favourited_channel(this.app.user.get_token(), this.get_id());
        this.app.user.favourited_channels.add(this);
      }
      return this.trigger('favourited_changed', this);
    };

    return Base;

  })();

  Rutv.Utils.extend(Rutv.Model.Channel.Base.prototype, Rutv.Utils.Events);

}).call(this);
(function() {

  Rutv.Model.Channel.Schedule.Item = (function() {

    function Item(data, channel, app) {
      this.channel = channel;
      this.app = app;
      this.title = data.title;
      this.description = data.description;
      this.thumb_url = data.thumb_url;
      this.streaming_urls = data.streaming_urls;
      this.beginning_label = data.beginning_str;
      this.ending_label = data.ending_str;
      this.available = data.avail;
      this.duration = data.duration;
      this.id = data.id;
      this.width = data.width;
      this.height = data.height;
      this.detect_time(data);
    }

    Item.prototype.detect_time = function(data) {
      var tz_offset;
      tz_offset = this.channel.get_tz_offset();
      this.beginning = data.beginning + tz_offset;
      return this.ending = data.ending + tz_offset;
    };

    Item.prototype.get_tz_offset = function() {
      return this.channel.get_tz_offset();
    };

    Item.prototype.get_channel = function() {
      return this.channel;
    };

    Item.prototype.get_title = function() {
      return this.title;
    };

    Item.prototype.get_width = function() {
      return this.width;
    };

    Item.prototype.get_height = function() {
      return this.height;
    };

    Item.prototype.get_id = function() {
      return this.id;
    };

    Item.prototype.get_duration = function() {
      var duration;
      if ((this.next_programm_beginning != null) && this.ending !== this.next_programm_beginning) {
        duration = this.next_programm_beginning - this.beginning;
      } else {
        duration = this.ending - this.beginning;
      }
      return duration;
    };

    Item.prototype.get_beginning = function() {
      return this.beginning;
    };

    Item.prototype.get_beginning_label = function() {
      return this.beginning_label;
    };

    Item.prototype.get_ending_label = function() {
      return this.ending_label;
    };

    Item.prototype.get_ending = function() {
      return this.ending;
    };

    Item.prototype.get_available = function() {
      return this.available;
    };

    Item.prototype.get_current_streaming_url = function() {
      var _ref;
      return (_ref = this.streaming_urls) != null ? _ref[0] : void 0;
    };

    Item.prototype.get_thumb_url = function() {
      return this.thumb_url;
    };

    Item.prototype.set_next_programm_beginning = function(beginning) {
      return this.next_programm_beginning = beginning;
    };

    Item.prototype.get_next_programm_beginning = function() {
      return this.next_programm_beginning;
    };

    Item.prototype.template_data = function() {
      return {
        title: this.title,
        description: this.description,
        beginning_label: this.beginning_label,
        ending_label: this.ending_label,
        thumb_url: this.thumb_url,
        available: this.available
      };
    };

    return Item;

  })();

}).call(this);
(function() {

  Rutv.Model.Channel.Schedule.Items = (function() {

    function Items(data, channel, app) {
      this.channel = channel;
      this.app = app;
      this.avaliable_items = [];
      this.index_by_item_map = {};
      this.items = this.create_items(data);
      this.current_channel_index = null;
      this.max_index = this.items.length;
    }

    Items.prototype.create_items = function(data) {
      var item, previous_item, result, schedule_item, _i, _len;
      result = [];
      if (data !== null) {
        for (_i = 0, _len = data.length; _i < _len; _i++) {
          item = data[_i];
          if (typeof schedule_item !== "undefined" && schedule_item !== null) {
            previous_item = schedule_item;
          }
          schedule_item = new Rutv.Model.Channel.Schedule.Item(item, this.channel, this.app);
          if (previous_item != null) {
            previous_item.set_next_programm_beginning(schedule_item.beginning);
          }
          if (schedule_item.get_available()) {
            this.avaliable_items.push(schedule_item);
          }
          result.push(schedule_item);
          this.index_by_item_map[schedule_item.get_id()] = result.length;
        }
      }
      return result;
    };

    Items.prototype.get_first = function() {
      return this.items[0];
    };

    Items.prototype.get_last_avaliable = function() {
      return this.avaliable_items[this.avaliable_items.length - 1];
    };

    Items.prototype.each = function(iterator, context) {
      var item, _i, _len, _ref;
      _ref = this.items;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        iterator.call(context, item);
      }
      return this;
    };

    Items.prototype.is_empty = function() {
      return this.items.length === 0;
    };

    Items.prototype.move_next = function() {
      if ((this.current_item_index != null) && (this.current_item_index < this.max_index) && this.item_is_avalible(1)) {
        this.current_item_index += 1;
        return this.trigger('current_changed');
      }
    };

    Items.prototype.move_previous = function() {
      if (this.current_item_index > 0 && this.item_is_avalible(-1)) {
        this.current_item_index -= 1;
        return this.trigger('current_changed');
      }
    };

    Items.prototype.get_previous = function() {
      if ((this.current_item_index != null) && (this.current_item_index > 0)) {
        return this.items[this.current_item_index - 1];
      } else {
        return null;
      }
    };

    Items.prototype.get_next = function() {
      if ((this.current_item_index != null) && (this.current_item_index < this.max_index)) {
        return this.items[this.current_item_index + 1];
      } else {
        return null;
      }
    };

    Items.prototype.get_current = function() {
      return this.items[this.current_item_index];
    };

    Items.prototype.get_current_index = function() {
      return this.current_item_index;
    };

    Items.prototype.set_current = function(selected_item) {
      var index, item, _i, _len, _ref, _results;
      _ref = this.items;
      _results = [];
      for (index = _i = 0, _len = _ref.length; _i < _len; index = ++_i) {
        item = _ref[index];
        if (selected_item.beginning === item.beginning) {
          this.current_item_index = index;
          this.trigger('current_changed');
          break;
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    Items.prototype.item_is_avalible = function(offset) {
      return this.items[this.current_item_index + offset].available === true;
    };

    Items.prototype.index_of = function(item) {
      if (item != null) {
        return this.index_by_item_map[item.get_id()];
      } else {
        return null;
      }
    };

    return Items;

  })();

  Rutv.Utils.extend(Rutv.Model.Channel.Schedule.Items.prototype, Rutv.Utils.Events);

}).call(this);
(function() {

  Rutv.Model.Queue.Item = (function() {

    function Item(data, app) {
      this.app = app;
      this.airtime = data.airtime;
      this.title = data.title;
      this.sub_title = data.sub_title;
      this.description = data.description;
      this.thumb = data.thumb;
      this.duration = data.duration;
      this.id = data.queue.id;
      this.streaming_urls = data.streaming_urls;
      this.width = data.width;
      this.height = data.height;
      if (data.history != null) {
        this.playhead_time = data.history.playhead_time;
      }
    }

    Item.prototype.get_playhead_time = function() {
      return this.playhead_time;
    };

    Item.prototype.set_playhead_time = function(playhead_time) {
      return this.playhead_time = playhead_time;
    };

    Item.prototype.get_width = function() {
      return this.width;
    };

    Item.prototype.get_height = function() {
      return this.height;
    };

    Item.prototype.get_title = function() {
      return this.title;
    };

    Item.prototype.get_sub_title = function() {
      return this.sub_title;
    };

    Item.prototype.get_duration = function() {
      return this.duration;
    };

    Item.prototype.get_id = function() {
      return this.id;
    };

    Item.prototype.get_airtime = function() {
      return this.airtime;
    };

    Item.prototype.get_current_streaming_url = function() {
      var _ref;
      return (_ref = this.streaming_urls) != null ? _ref[0] : void 0;
    };

    Item.prototype.get_thumb = function() {
      return this.thumb;
    };

    Item.prototype.template_data = function() {
      return {
        title: this.title,
        sub_title: this.sub_title,
        description: this.description,
        thumb: this.thumb
      };
    };

    return Item;

  })();

}).call(this);
(function() {

  Rutv.Model.User = (function() {

    function User(app) {
      this.app = app;
      this.storage = this.app.platform.storage;
    }

    User.prototype.get_token = function() {
      return this.storage.get('auth_token');
    };

    User.prototype.set_token = function(token_value) {
      return this.storage.set('auth_token', token_value);
    };

    User.prototype.remove_token = function() {
      return this.storage.remove('auth_token');
    };

    User.prototype.get_channels = function() {
      this.load();
      return this.channels_deferred.promise();
    };

    User.prototype.get_queue = function() {
      var _this = this;
      this.queue_deferred = new Rutv.Lib.DeferredWrapper();
      this.app.server.get_queue(this.get_token()).done(function(data) {
        return _this.create_queue_collections(data);
      });
      return this.queue_deferred.promise();
    };

    User.prototype.get_account = function() {
      var _this = this;
      this.account_deferred = new Rutv.Lib.DeferredWrapper();
      this.app.server.get_account(this.get_token()).done(function(data) {
        return _this.account_deferred.resolve(_this.account_data = data);
      });
      return this.account_deferred.promise();
    };

    User.prototype.get_account_timezone = function() {
      var _ref;
      return (_ref = this.account_data) != null ? _ref.account.user_tz_offset : void 0;
    };

    User.prototype.get_timezone = function() {
      return -(new Date()).getTimezoneOffset() * 60;
    };

    User.prototype.get_timezone_offset = function() {
      var now, now_tz, summer_tz, tz, tz_offset, winter_tz;
      tz = null;
      tz_offset = null;
      now = new Date();
      now_tz = now.getTimezoneOffset();
      winter_tz = new Date(now.getFullYear(), 1, 1).getTimezoneOffset();
      summer_tz = new Date(now.getFullYear(), 7, 1).getTimezoneOffset();
      if (winter_tz === summer_tz) {
        tz = now_tz / 60;
      } else {
        if (now_tz === winter_tz) {
          tz = now_tz / 60;
        } else if (now_tz === summer_tz) {
          tz = (now_tz + 60) / 60;
        }
      }
      tz_offset = this.get_account_timezone() + (now_tz * 60);
      return tz_offset;
    };

    User.prototype.current_time_in_string_format = function() {
      var date_with_offset, hours, minutes, time;
      time = new Date().getTime();
      date_with_offset = new Date(time + this.get_timezone_offset() * 1000);
      hours = date_with_offset.getHours();
      minutes = date_with_offset.getMinutes();
      hours = this.add_zero_before_number(parseFloat(hours));
      minutes = this.add_zero_before_number(parseFloat(minutes));
      return "" + hours + ":" + minutes;
    };

    User.prototype.add_zero_before_number = function(number) {
      if (number < 10) {
        number = '0' + number.toString();
      }
      return number;
    };

    User.prototype.get_favourited_channels = function() {
      this.load();
      return this.favourited_channels_deferred.promise();
    };

    User.prototype.channels_ready = function(channels, favourited_channels) {
      this.channels = channels;
      this.favourited_channels = favourited_channels;
      this.channels_deferred.resolve(this.channels);
      return this.favourited_channels_deferred.resolve(this.favourited_channels);
    };

    User.prototype.load = function() {
      var _this = this;
      if (this.channels_deferred == null) {
        this.channels_deferred = new Rutv.Lib.DeferredWrapper();
        this.favourited_channels_deferred = new Rutv.Lib.DeferredWrapper();
        return this.app.server.get_channels(this.get_token()).done(function(data) {
          return _this.create_channels_collections(data);
        });
      }
    };

    User.prototype.create_channels_collections = function(channels_data) {
      var channel, channel_data, channels, favourited_channels, _i, _len;
      channels = new Rutv.Collection.Channels();
      favourited_channels = new Rutv.Collection.Channels();
      for (_i = 0, _len = channels_data.length; _i < _len; _i++) {
        channel_data = channels_data[_i];
        channel = new Rutv.Model.Channel.Base(channel_data, this.app);
        channels.add(channel);
        if (channel_data.favourited) {
          favourited_channels.add(channel);
        }
      }
      return this.channels_ready(channels, favourited_channels);
    };

    User.prototype.create_queue_collections = function(queue_data) {
      var programm, programm_data, _i, _len, _ref;
      this.queue = new Rutv.Collection.Queue();
      _ref = queue_data.queue_items;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        programm_data = _ref[_i];
        programm = new Rutv.Model.Queue.Item(programm_data, this.app);
        this.queue.add(programm);
      }
      return this.queue_deferred.resolve(this.queue);
    };

    return User;

  })();

  Rutv.Utils.extend(Rutv.Model.User.prototype, Rutv.Utils.Events);

}).call(this);

/**
* @author Alexandr Borisov (ab@cifronomika.ru)
*/


(function() {

  Rutv.Manager.Auth = (function() {
    /**
    * @name Rutv.Manager.Auth
    * @constructor
    * @borrows Rutv.Utils.Events#on as #on
    * @borrows Rutv.Utils.Events#off as #off
    * @borrows Rutv.Utils.Events#trigger as #trigger
    */

    function Auth(config, server, app) {
      this.config = config;
      this.server = server;
      this.app = app;
    }

    Auth.prototype.login = function() {
      var token,
        _this = this;
      token = this.app.user.get_token();
      if (token != null) {
        return this.server.verify_token(token).done(function(data) {
          return _this.trigger(data.authorized === true ? 'login' : 'login_fail');
        }).fail(function() {
          return _this.trigger('login_fail');
        });
      } else {
        return this.trigger('login_fail');
      }
    };

    Auth.prototype.login_by_code = function() {
      var _this = this;
      return this.server.get_code(this.app.platform.get_device_id(), this.app.platform.get_device_type()).done(function(data) {
        if (data.code != null) {
          _this.code = data.code;
          _this.trigger('code', _this.code);
          return _this.start_checking();
        }
      });
    };

    Auth.prototype.start_checking = function() {
      return this.next();
    };

    Auth.prototype.next = function() {
      this.start_timeout();
      return this.check();
    };

    Auth.prototype.start_timeout = function() {
      var _this = this;
      this.timeout_deffered = new Rutv.Lib.DeferredWrapper();
      return setTimeout(function() {
        return _this.timeout_deffered.resolve();
      }, this.config.defaults.loading_logged_status_timeout);
    };

    Auth.prototype.check = function() {
      var _this = this;
      return this.server.check_code(this.code, this.app.platform.get_device_id(), this.app.platform.get_device_type()).done(function(data) {
        if (data.authorized === true) {
          _this.token = data.token;
          _this.app.user.set_token(_this.token);
          return _this.trigger('login');
        } else {
          return _this.timeout_deffered.done(function() {
            return _this.next();
          });
        }
      });
    };

    return Auth;

  })();

  Rutv.Utils.extend(Rutv.Manager.Auth.prototype, Rutv.Utils.Events);

}).call(this);
(function() {

  Rutv.Collection.Base = (function() {

    function Base() {
      this.items = [];
      this.index_by_item_map = {};
      this.current_item_index = null;
      this.max_index = null;
    }

    Base.prototype.each = function(iterator, context) {
      var item, _i, _len, _ref;
      _ref = this.items;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        iterator.call(context, item);
      }
      return this;
    };

    Base.prototype.map = function(iterator, context) {
      var item, result, _i, _len, _ref;
      result = [];
      _ref = this.items;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        result.push(iterator.call(context, item));
      }
      return result;
    };

    Base.prototype.get_first = function() {
      return this.items[0];
    };

    Base.prototype.get_last = function() {
      return this.items[this.items.length - 1];
    };

    Base.prototype.move_next = function() {
      if ((this.current_item_index != null) && (this.current_item_index < this.max_index)) {
        this.current_item_index += 1;
      } else {
        this.current_item_index = 0;
      }
      return this.trigger('current_changed');
    };

    Base.prototype.move_previous = function() {
      if (this.current_item_index > 0) {
        this.current_item_index -= 1;
      } else {
        this.current_item_index = this.max_index;
      }
      return this.trigger('current_changed');
    };

    Base.prototype.get_previous = function() {
      if ((this.current_item_index != null) && (this.current_item_index > 0)) {
        return this.items[this.current_item_index - 1];
      } else {
        return null;
      }
    };

    Base.prototype.get_next = function() {
      if ((this.current_item_index != null) && (this.current_item_index < this.max_index)) {
        return this.items[this.current_item_index + 1];
      } else {
        return null;
      }
    };

    Base.prototype.get_current = function() {
      return this.items[this.current_item_index];
    };

    Base.prototype.get_current_index = function() {
      return this.current_item_index;
    };

    Base.prototype.exists = function(channel) {
      return this.index_of(channel) > -1;
    };

    Base.prototype.get_random = function() {
      var index;
      if (this.max_index != null) {
        index = Math.floor(Math.random() * this.max_index);
        return this.items[index];
      } else {
        return null;
      }
    };

    Base.prototype.set_current = function(item) {
      var index;
      index = this.index_of(item);
      if (index != null) {
        this.current_item_index = index;
      } else {
        this.current_item_index = null;
      }
      return this.trigger('current_changed');
    };

    Base.prototype.is_empty = function() {
      if (this.items.length === 0) {
        return true;
      }
    };

    Base.prototype.get_count = function() {
      return this.items.length;
    };

    Base.prototype.exists = function(item) {
      return this.index_of(item) > -1;
    };

    Base.prototype.index_of = function(item) {
      if (item != null) {
        return this.index_by_item_map[item.get_id()];
      } else {
        return null;
      }
    };

    return Base;

  })();

  Rutv.Utils.extend(Rutv.Collection.Base.prototype, Rutv.Utils.Events);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Collection.Channels = (function(_super) {

    __extends(Channels, _super);

    function Channels() {
      Channels.__super__.constructor.call(this);
      this.items_by_button_map = {};
    }

    Channels.prototype.get_most_earlier_programm = function() {
      var item, programm, result, schedules_loaded_count, _i, _len, _ref,
        _this = this;
      result = new Rutv.Lib.DeferredWrapper();
      schedules_loaded_count = 0;
      this.trigger('schedule_progress', schedules_loaded_count);
      programm = null;
      _ref = this.items;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        item.get_schedule().done(function(schedule) {
          var first_programm;
          schedules_loaded_count += 1;
          _this.trigger('schedule_progress', schedules_loaded_count);
          if ((first_programm = schedule.get_first())) {
            if (!(programm != null) || (first_programm.get_beginning() < programm.get_beginning())) {
              programm = first_programm;
            }
          }
          if (_this.items.length === schedules_loaded_count) {
            return result.resolve(programm);
          }
        });
      }
      return result.promise();
    };

    Channels.prototype.get_latest_programm = function() {
      var item, programm, result, schedules_loaded_count, _i, _len, _ref,
        _this = this;
      result = new Rutv.Lib.DeferredWrapper();
      schedules_loaded_count = 0;
      this.trigger('schedule_progress', schedules_loaded_count);
      programm = null;
      _ref = this.items;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        item.get_schedule().done(function(schedule) {
          var later_programm;
          schedules_loaded_count += 1;
          _this.trigger('schedule_progress', schedules_loaded_count);
          if ((later_programm = schedule.get_first())) {
            if (!(programm != null) || (later_programm.get_beginning() > programm.get_beginning())) {
              programm = later_programm;
            }
          }
          if (_this.items.length === schedules_loaded_count) {
            return result.resolve(programm);
          }
        });
      }
      return result.promise();
    };

    Channels.prototype.add = function(item) {
      var index;
      index = this.items.length;
      this.items.push(item);
      this.items_by_button_map[item.get_button()] = item;
      this.index_by_item_map[item.get_id()] = index;
      this.max_index = index;
      return this.trigger('add');
    };

    Channels.prototype.remove = function(item) {
      var index, _i, _len, _ref;
      index = this.index_of(item);
      if (index > -1) {
        this.items.splice(index, 1);
        if (this.items.length === 0) {
          this.max_index = 0;
        } else {
          this.max_index -= 1;
        }
        delete this.items_by_button_map[item.get_button()];
        this.index_by_item_map = {};
        _ref = this.items;
        for (index = _i = 0, _len = _ref.length; _i < _len; index = ++_i) {
          item = _ref[index];
          this.index_by_item_map[item.get_id()] = index;
        }
        if (this.current_item_index >= index) {
          if (this.items.length === 0) {
            this.current_item_index = null;
          } else {
            this.current_item_index -= 1;
          }
          this.trigger('current_changed');
        }
        return this.trigger('remove');
      }
    };

    Channels.prototype.get_by_button = function(button) {
      return this.items_by_button_map[button];
    };

    return Channels;

  })(Rutv.Collection.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.Collection.Queue = (function(_super) {

    __extends(Queue, _super);

    function Queue() {
      Queue.__super__.constructor.call(this);
    }

    Queue.prototype.add = function(item) {
      var index;
      index = this.items.length;
      this.items.push(item);
      this.index_by_item_map[item.get_id()] = index;
      this.max_index = index;
      return this.trigger('add');
    };

    Queue.prototype.remove = function(item) {
      var index, _i, _len, _ref;
      index = this.index_of(item);
      if (index > -1) {
        this.items.splice(index, 1);
        if (this.items.length === 0) {
          this.max_index = 0;
        } else {
          this.max_index -= 1;
        }
        this.index_by_item_map = {};
        _ref = this.items;
        for (index = _i = 0, _len = _ref.length; _i < _len; index = ++_i) {
          item = _ref[index];
          this.index_by_item_map[item.get_id()] = index;
        }
        if (this.current_item_index >= index) {
          if (this.items.length === 0) {
            this.current_item_index = null;
          } else {
            this.current_item_index -= 1;
          }
          this.trigger('current_changed');
        }
        return this.trigger('remove');
      }
    };

    return Queue;

  })(Rutv.Collection.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Base = (function(_super) {

    __extends(Base, _super);

    function Base(platform) {
      Base.__super__.constructor.call(this, platform);
    }

    Base.prototype.show = function() {
      this.trigger('before_show', this);
      Base.__super__.show.call(this);
      this.trigger('show', this);
      return this;
    };

    Base.prototype.hide = function() {
      this.trigger('before_hide', this);
      Base.__super__.hide.call(this);
      this.trigger('hide', this);
      return this;
    };

    return Base;

  })(Rutv.View.Base);

  Rutv.Utils.extend(Rutv.View.Component.Base.prototype, Rutv.Utils.Events);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Button = (function(_super) {

    __extends(Button, _super);

    function Button(platform, hints) {
      this.hints = hints != null ? hints : [];
      Button.__super__.constructor.call(this, platform);
      this.focused = false;
    }

    Button.prototype.render = function() {
      this.el = this.el_factory();
      if (this.template_name != null) {
        this.el.set_content(this.template(), this.template_data());
      }
      this.attach_mouse_handlers();
      return this;
    };

    Button.prototype.focus = function() {
      if (!this.focused) {
        this.trigger('before_focus', this);
        this.key_trap.on('key_press', this.keypress_handler, this);
        this.el.add_class('focused');
        this.focused = true;
        this.trigger('focus', this);
        this.set_current_actions();
      }
      return this;
    };

    Button.prototype.blur = function() {
      if (this.focused) {
        this.key_trap.off('key_press', this.keypress_handler, this);
        this.el.remove_class('focused');
        this.focused = false;
        this.clear_current_actions();
      }
      return this;
    };

    Button.prototype.set_current_actions = function() {
      var _ref, _ref1, _ref2, _ref3, _ref4;
      return (_ref = this.app) != null ? (_ref1 = _ref.scene) != null ? (_ref2 = _ref1.subscenes) != null ? (_ref3 = _ref2.get_current()) != null ? (_ref4 = _ref3.hints) != null ? _ref4.set_current_actions(this.hints) : void 0 : void 0 : void 0 : void 0 : void 0;
    };

    Button.prototype.clear_current_actions = function() {
      var _ref, _ref1, _ref2, _ref3, _ref4;
      return (_ref = this.app) != null ? (_ref1 = _ref.scene) != null ? (_ref2 = _ref1.subscenes) != null ? (_ref3 = _ref2.get_current()) != null ? (_ref4 = _ref3.hints) != null ? _ref4.clear_current_actions() : void 0 : void 0 : void 0 : void 0 : void 0;
    };

    Button.prototype.keypress_handler = function(key) {
      var event_name;
      event_name = (function() {
        switch (key) {
          case Rutv.Key.Left:
            return 'left';
          case Rutv.Key.Right:
            return 'right';
          case Rutv.Key.Up:
            return 'up';
          case Rutv.Key.Down:
            return 'down';
          case Rutv.Key.Ok:
            return 'press';
          case Rutv.Key.Back:
            return 'back';
          case Rutv.Key.Play:
            return 'play';
          case Rutv.Key.Pause:
            return 'pause';
          case Rutv.Key.Stop:
            return 'stop';
          case Rutv.Key.Rewind:
            return 'rewind';
          case Rutv.Key.Forward:
            return 'forward';
          case Rutv.Key.VolumeUp:
            return 'volume_up';
          case Rutv.Key.VolumeDown:
            return 'volume_down';
          case Rutv.Key.VolumeMute:
            return 'toggle_mute';
          case Rutv.Key.Exit:
            return 'exit';
          default:
            return null;
        }
      })();
      if (event_name != null) {
        return this.trigger(event_name, this);
      }
    };

    Button.prototype.attach_mouse_handlers = function() {
      var _this = this;
      return this.el.on('mouse_in', this.focus, this).on('press', function() {
        return _this.trigger('press', _this);
      });
    };

    Button.prototype.destroy = function() {
      this.key_trap.off('key_press', this.keypress_handler, this);
      return Button.__super__.destroy.call(this);
    };

    return Button;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Hints = (function(_super) {

    __extends(Hints, _super);

    function Hints(app, scene_actions) {
      this.app = app;
      this.scene_actions = scene_actions != null ? scene_actions : [];
      Hints.__super__.constructor.call(this, this.app.platform);
      this.current_actions = [];
      this.buttons = [];
    }

    Hints.prototype.set_current_actions = function(current_actions) {
      this.current_actions = current_actions != null ? current_actions : [];
      return this.render_actions();
    };

    Hints.prototype.clear_current_actions = function() {
      return this.set_current_actions([]);
    };

    Hints.prototype.render = function() {
      (this.el = this.el_factory()).set_classname('hints');
      this.render_actions();
      return this;
    };

    Hints.prototype.render_actions = function() {
      var actions;
      if ((actions = this.merge_actions(this.scene_actions, this.current_actions)).length > 0) {
        this.el.clear();
        this.create_buttons(actions);
        return this.el.show();
      } else {
        return this.el.clear();
      }
    };

    Hints.prototype.merge_actions = function() {
      var action, result, _i, _j, _len, _len1, _ref, _ref1;
      result = [];
      _ref = this.scene_actions;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        action = _ref[_i];
        result.push(action);
      }
      _ref1 = this.current_actions;
      for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
        action = _ref1[_j];
        result.push(action);
      }
      return result;
    };

    Hints.prototype.create_buttons = function(actions) {
      var action, _i, _len, _results;
      _results = [];
      for (_i = 0, _len = actions.length; _i < _len; _i++) {
        action = actions[_i];
        _results.push(this.el.append((new Rutv.View.Component.Hints.Button(this.app.platform, action)).render().el));
      }
      return _results;
    };

    Hints.prototype.get_class_by = function(key) {
      switch (key) {
        case Rutv.Key.Ok:
          return 'ok';
        case Rutv.Key.Back:
          return 'back';
        case Rutv.Key.Up:
          return 'up';
        case Rutv.Key.Down:
          return 'down';
        case Rutv.Key.Left:
          return 'left';
        case Rutv.Key.Right:
          return 'right';
        case Rutv.Key.Blue:
          return 'blue';
        case Rutv.Key.Red:
          return 'red';
        default:
          return '';
      }
    };

    Hints.prototype.destroy_buttons = function() {
      var button, _i, _len, _ref, _results;
      _ref = this.buttons;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        button = _ref[_i];
        _results.push(button.destroy());
      }
      return _results;
    };

    return Hints;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Hints.Button = (function(_super) {

    __extends(Button, _super);

    function Button(platform, action_data) {
      this.platform = platform;
      this.key = action_data[0];
      this.label = action_data[1];
      this.context = action_data[2];
      Button.__super__.constructor.call(this, this.platform);
    }

    Button.prototype.render = function() {
      Button.__super__.render.call(this);
      this.el.set_class(this.get_class_by(this.key));
      this.el.set_content(this.label);
      return this;
    };

    Button.prototype.get_class_by = function(key) {
      switch (key) {
        case Rutv.Key.Ok:
          return 'ok';
        case Rutv.Key.Back:
          return 'back';
        case Rutv.Key.Up:
          return 'up';
        case Rutv.Key.Down:
          return 'down';
        case Rutv.Key.Left:
          return 'left';
        case Rutv.Key.Right:
          return 'right';
        case Rutv.Key.Blue:
          return 'blue';
        case Rutv.Key.Red:
          return 'red';
        default:
          return '';
      }
    };

    Button.prototype.mouse_press_handler = function() {
      return this.context.keypress_handler(this.key);
    };

    Button.prototype.attach_mouse_handlers = function() {
      if (this.context != null) {
        return this.el.on('press', this.mouse_press_handler, this);
      }
    };

    Button.prototype.destroy = function() {
      if (this.context != null) {
        this.el.off('press', this.mouse_press_handler, this);
      }
      return Button.__super__.destroy.call(this);
    };

    return Button;

  })(Rutv.View.Component.Button);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Home.Menu.Button.Base = (function(_super) {

    __extends(Base, _super);

    Base.prototype.template_name = 'home.menu.button';

    function Base(app, label, hints) {
      this.app = app;
      this.label = label;
      if (hints == null) {
        hints = [];
      }
      Base.__super__.constructor.call(this, this.app.platform, hints);
      this.selected = false;
    }

    Base.prototype.render = function() {
      Base.__super__.render.call(this);
      this.el.set_class('home__menu__button');
      return this;
    };

    Base.prototype.template_data = function() {
      return {
        label: this.label
      };
    };

    Base.prototype.select = function() {
      if (!this.selected) {
        this.trigger('before_select', this);
        this.el.add_class('selected');
        this.selected = true;
        this.trigger('select', this);
      }
      return this;
    };

    Base.prototype.unselect = function() {
      if (this.selected) {
        this.el.remove_class('selected');
        this.selected = false;
      }
      return this;
    };

    Base.prototype.use_tv_hints = function() {
      if (this.tv_hints != null) {
        return this.hints = this.tv_hints;
      }
    };

    Base.prototype.use_account_hints = function() {
      if (this.account_hints != null) {
        return this.hints = this.account_hints;
      }
    };

    return Base;

  })(Rutv.View.Component.Button);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Home.Menu.Button.Tv = (function(_super) {

    __extends(Tv, _super);

    function Tv(app, label) {
      this.app = app;
      this.label = label;
      this.account_hints = [[Rutv.Key.Back, this.app.resources.hints.tv, this], [Rutv.Key.Ok, this.app.resources.hints.show, this], [Rutv.Key.Left, this.app.resources.hints.account, this], [Rutv.Key.Right, this.app.resources.hints.queue, this], [Rutv.Key.Down, this.app.resources.hints.account, this]];
      this.tv_hints = [[Rutv.Key.Back, this.app.resources.hints.exit, this], [Rutv.Key.Ok, this.app.resources.hints.show, this], [Rutv.Key.Left, this.app.resources.hints.account, this], [Rutv.Key.Right, this.app.resources.hints.queue, this], [Rutv.Key.Down, this.app.resources.hints.channels, this]];
      Tv.__super__.constructor.call(this, this.app, this.label, this.tv_hints);
    }

    return Tv;

  })(Rutv.View.Component.Home.Menu.Button.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Home.Menu.Button.Queue = (function(_super) {

    __extends(Queue, _super);

    function Queue(app, label) {
      this.app = app;
      this.label = label;
      this.account_hints = [[Rutv.Key.Back, this.app.resources.hints.tv, this], [Rutv.Key.Ok, this.app.resources.hints.show, this], [Rutv.Key.Left, this.app.resources.hints.tv, this], [Rutv.Key.Right, this.app.resources.hints.account, this], [Rutv.Key.Down, this.app.resources.hints.account, this]];
      this.tv_hints = [[Rutv.Key.Back, this.app.resources.hints.exit, this], [Rutv.Key.Ok, this.app.resources.hints.show, this], [Rutv.Key.Left, this.app.resources.hints.tv, this], [Rutv.Key.Right, this.app.resources.hints.account, this], [Rutv.Key.Down, this.app.resources.hints.channels, this]];
      Queue.__super__.constructor.call(this, this.app, this.label, this.tv_hints);
    }

    return Queue;

  })(Rutv.View.Component.Home.Menu.Button.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Home.Menu.Button.Account = (function(_super) {

    __extends(Account, _super);

    function Account(app, label) {
      this.app = app;
      this.label = label;
      this.account_hints = [[Rutv.Key.Back, this.app.resources.hints.tv, this], [Rutv.Key.Ok, this.app.resources.hints.show, this], [Rutv.Key.Left, this.app.resources.hints.queue, this], [Rutv.Key.Right, this.app.resources.hints.tv, this], [Rutv.Key.Down, this.app.resources.hints.account, this]];
      this.tv_hints = [[Rutv.Key.Back, this.app.resources.hints.exit, this], [Rutv.Key.Ok, this.app.resources.hints.show, this], [Rutv.Key.Left, this.app.resources.hints.queue, this], [Rutv.Key.Right, this.app.resources.hints.tv, this], [Rutv.Key.Down, this.app.resources.hints.channels, this]];
      Account.__super__.constructor.call(this, this.app, this.label, this.tv_hints);
    }

    return Account;

  })(Rutv.View.Component.Home.Menu.Button.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Home.Menu.Base = (function(_super) {

    __extends(Base, _super);

    function Base(app) {
      this.app = app;
      Base.__super__.constructor.call(this, this.app.platform);
      this.selected_button = null;
      this.previous_selected_button = null;
      this.focused_button = null;
    }

    Base.prototype.render = function() {
      (this.el = this.el_factory()).set_class('home__menu').append(this.create_tv_button().render().el).append(this.create_queue_button().render().el).append(this.create_account_button().render().el);
      return this;
    };

    Base.prototype.focus = function() {
      var _ref;
      if (this.focused_button == null) {
        this.focused_button = this.selected_button;
      }
      if ((_ref = this.focused_button) != null) {
        _ref.focus();
      }
      return this;
    };

    Base.prototype.blur = function() {
      this.blur_focused();
      this.focused_button = null;
      return this;
    };

    Base.prototype.blur_focused = function() {
      var _ref;
      return (_ref = this.focused_button) != null ? _ref.blur() : void 0;
    };

    Base.prototype.unselect_selected = function() {
      var _ref;
      return (_ref = this.selected_button) != null ? _ref.unselect() : void 0;
    };

    Base.prototype.create_tv_button = function() {
      var _this = this;
      return this.attach_common_buttons_event_handlers(this.tv_button = new Rutv.View.Component.Home.Menu.Button.Tv(this.app, this.app.resources.home.menu.tv)).on('left', function() {
        return _this.account_button.focus();
      }).on('right', function() {
        return _this.queue_button.focus();
      }).on('back', function() {
        return _this.back_event_handler();
      }).on('focus', function() {
        return _this.tv_button_focus_handler();
      });
    };

    Base.prototype.create_queue_button = function() {
      var _this = this;
      return this.attach_common_buttons_event_handlers(this.queue_button = new Rutv.View.Component.Home.Menu.Button.Queue(this.app, this.app.resources.home.menu.queue)).on('left', function() {
        return _this.tv_button.focus();
      }).on('right', function() {
        return _this.account_button.focus();
      }).on('back', function() {
        return _this.back_event_handler();
      }).on('press', function() {
        return _this.trigger('queue');
      }).on('focus', function() {
        return _this.queue_button_focus_handler();
      });
    };

    Base.prototype.create_account_button = function() {
      var _this = this;
      return this.attach_common_buttons_event_handlers(this.account_button = new Rutv.View.Component.Home.Menu.Button.Account(this.app, this.app.resources.home.menu.account)).on('left', function() {
        return _this.queue_button.focus();
      }).on('right', function() {
        return _this.tv_button.focus();
      }).on('back', function() {
        return _this.back_event_handler();
      }).on('focus', function() {
        return _this.account_button_focus_handler();
      });
    };

    Base.prototype.attach_common_buttons_event_handlers = function(button) {
      var _this = this;
      return button.on('down', function() {
        return _this.trigger('down');
      }).on('press', function(button) {
        return button.select();
      }).on('before_focus', this.blur_focused, this).on('focus', this.button_focus_handler, this).on('before_select', this.unselect_selected, this).on('select', this.button_select_handler, this).on('exit', function() {
        return _this.trigger('exit');
      });
    };

    Base.prototype.button_focus_handler = function(button) {
      this.focused_button = button;
      return this.trigger('focus');
    };

    Base.prototype.button_select_handler = function(button) {
      this.previous_selected_button = this.selected_button;
      (this.selected_button = button).select();
      switch (this.selected_button) {
        case this.tv_button:
          return this.trigger('tv');
        case this.account_button:
          return this.trigger('account');
      }
    };

    Base.prototype.back_event_handler = function() {
      if (this.selected_button === this.tv_button) {
        return this.trigger('exit');
      } else {
        return this.tv_button.select();
      }
    };

    Base.prototype.account_button_focus_handler = function() {
      var selected_button;
      selected_button = this.detect_selected_button();
      if (selected_button === this.tv_button) {
        return this.account_button.use_tv_hints();
      } else {
        return this.account_button.use_account_hints();
      }
    };

    Base.prototype.queue_button_focus_handler = function() {
      var selected_button;
      selected_button = this.detect_selected_button();
      if (selected_button === this.account_button) {
        return this.queue_button.use_account_hints();
      } else {
        return this.queue_button.use_tv_hints();
      }
    };

    Base.prototype.tv_button_focus_handler = function() {
      var selected_button;
      selected_button = this.detect_selected_button();
      if (selected_button === this.account_button) {
        return this.tv_button.use_account_hints();
      } else {
        return this.tv_button.use_tv_hints();
      }
    };

    Base.prototype.detect_selected_button = function() {
      var selected_button;
      if (this.selected_button === this.queue_button) {
        selected_button = this.previous_selected_button;
      } else {
        selected_button = this.selected_button;
      }
      return selected_button;
    };

    return Base;

  })(Rutv.View.Component.Base);

  Rutv.Utils.extend(Rutv.View.Component.Home.Menu.Base.prototype, Rutv.Utils.Events);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Home.Account.Base = (function(_super) {

    __extends(Base, _super);

    Base.prototype.template_name = 'home.account';

    function Base(app, hints) {
      this.app = app;
      if (hints == null) {
        hints = [];
      }
      Base.__super__.constructor.call(this, this.app.platform, [[Rutv.Key.Back, this.app.resources.hints.tv, this], [Rutv.Key.Up, app.resources.hints.menu, this]]);
      this.focused = false;
    }

    Base.prototype.render = function() {
      this.el = this.el_factory().set_class('home__account');
      this.attach_mouse_handlers();
      this.load_account();
      return this;
    };

    Base.prototype.focus = function() {
      if (!this.focused) {
        this.key_trap.on('key_press', this.keypress_handler, this);
        this.el.add_class('focused');
        this.focused = true;
        this.trigger('focus', this);
        this.set_current_actions();
      }
      return this;
    };

    Base.prototype.blur = function() {
      if (this.focused) {
        this.key_trap.off('key_press', this.keypress_handler, this);
        this.el.remove_class('focused');
        this.focused = false;
        this.clear_current_actions();
      }
      return this;
    };

    Base.prototype.attach_mouse_handlers = function() {
      return this.el.on('mouse_in', this.focus, this).on('mouse_out', this.blur, this);
    };

    Base.prototype.load_account = function() {
      var _this = this;
      return this.app.user.get_account(this.app.user.get_token()).done(function(data) {
        return _this.el.set_content(_this.template(), _this.app.resources.home.account(data));
      });
    };

    return Base;

  })(Rutv.View.Component.Button);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Home.Tv.Base = (function(_super) {

    __extends(Base, _super);

    function Base(app) {
      this.app = app;
      Base.__super__.constructor.call(this, this.app.platform);
      this.focused = false;
    }

    Base.prototype.render = function() {
      (this.el = this.el_factory()).set_class('home__tv').append(this.create_random_channel_button().render().show().el).append(this.create_all_channels_button().render().show().el).append(this.create_favourited_channels_button().render().show().el);
      return this;
    };

    Base.prototype.create_random_channel_button = function() {
      var _this = this;
      (this.random_channel_button = new Rutv.View.Component.Home.Tv.Button.RandomChannel(this.app)).on('right', function() {
        return _this.focus_button(_this.prev_button);
      }).on('up', function() {
        _this.trigger('up');
        return _this.random_channel_button.blur();
      }).on('before_focus', function() {
        _this.random_channel_button_update_hints();
        return _this.blur_focused(_this.random_channel_button);
      }).on('press play', function() {
        return _this.app.user.get_channels().done(function(channels) {
          return _this.app.scene.subscenes.push(new Rutv.View.Scene.Player(_this.app, _this.random_channel_button.get_channel(), channels)).play();
        });
      }).on('back', function() {
        return _this.trigger('back');
      }).on('exit', function() {
        return _this.trigger('exit');
      });
      return this.focused_button = this.random_channel_button;
    };

    Base.prototype.create_all_channels_button = function() {
      var _this = this;
      (this.all_channels_button = new Rutv.View.Component.Home.Tv.Button.AllChannels(this.app)).on('left', function() {
        _this.random_channel_button.focus();
        return _this.prev_button = _this.all_channels_button;
      }).on('down', function() {
        return _this.focus_button(_this.favourited_channels_button);
      }).on('up', function() {
        _this.trigger('up');
        return _this.all_channels_button.blur();
      }).on('before_focus', this.blur_focused, this).on('press', function() {
        return _this.app.scene.subscenes.push(new Rutv.View.Scene.Channels.All(_this.app));
      }).on('back', function() {
        return _this.trigger('back');
      }).on('exit', function() {
        return _this.trigger('exit');
      });
      return this.prev_button = this.all_channels_button;
    };

    Base.prototype.create_favourited_channels_button = function() {
      var _this = this;
      return (this.favourited_channels_button = new Rutv.View.Component.Home.Tv.Button.FavouritedChannels(this.app)).on('left', function() {
        _this.random_channel_button.focus();
        return _this.prev_button = _this.favourited_channels_button;
      }).on('up', function() {
        return _this.focus_button(_this.all_channels_button);
      }).on('before_focus', this.blur_focused, this).on('press', function() {
        _this.app.scene.subscenes.push(new Rutv.View.Scene.Channels.Favourited(_this.app));
        return _this.favourited_channels_button.blur();
      }).on('back', function() {
        return _this.trigger('back');
      }).on('exit', function() {
        return _this.trigger('exit');
      });
    };

    Base.prototype.random_channel_button_update_hints = function() {
      var hint;
      switch (this.focused_button) {
        case this.all_channels_button:
          hint = this.app.resources.hints.all_channels;
          break;
        case this.favourited_channels_button:
          hint = this.app.resources.hints.favourited_channels;
          break;
        default:
          hint = this.app.resources.hints.all_channels;
      }
      return this.random_channel_button.update_right_key_hint(hint);
    };

    Base.prototype.focus = function() {
      if (!this.focused) {
        this.focused_button.focus();
        this.focused = true;
        this.trigger('focus');
      }
      return this;
    };

    Base.prototype.blur = function() {
      var _ref;
      if ((_ref = this.focused_button) != null) {
        _ref.blur();
      }
      this.focused = false;
      return this;
    };

    Base.prototype.focus_button = function(button) {
      this.blur_focused(button);
      return this.focused_button.focus();
    };

    Base.prototype.blur_focused = function(button) {
      var _ref;
      this.trigger('focus');
      if ((_ref = this.focused_button) != null) {
        _ref.blur();
      }
      return this.focused_button = button;
    };

    return Base;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Home.Tv.Button.Base = (function(_super) {

    __extends(Base, _super);

    function Base(app, hints) {
      this.app = app;
      if (hints == null) {
        hints = [];
      }
      Base.__super__.constructor.call(this, this.app.platform, hints);
      this.channels_deferred = this.channels_deffered_factory();
    }

    Base.prototype.render = function() {
      var _this = this;
      Base.__super__.render.call(this);
      this.channels_deferred.done(function(channels) {
        return _this.channels_ready_handler(channels);
      });
      return this;
    };

    Base.prototype.channels_deffered_factory = function() {
      throw "should be implemented in subclass";
    };

    Base.prototype.channels_ready_handler = function() {
      throw "should be implemented in subclass";
    };

    return Base;

  })(Rutv.View.Component.Button);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Home.Tv.Button.RandomChannel = (function(_super) {

    __extends(RandomChannel, _super);

    RandomChannel.prototype.template_name = 'home.tv.random_channel.loading';

    function RandomChannel(app) {
      RandomChannel.__super__.constructor.call(this, app, [[Rutv.Key.Back, app.resources.hints.exit, this], [Rutv.Key.Ok, app.resources.hints.play, this], [Rutv.Key.Right, app.resources.hints.channels, this], [Rutv.Key.Up, app.resources.hints.menu, this]]);
      this.on('press', this.press_muter, this);
    }

    RandomChannel.prototype.render = function() {
      RandomChannel.__super__.render.call(this);
      this.el.set_classname('channel-screen__container loading');
      return this;
    };

    RandomChannel.prototype.template_data = function() {
      return this.app.resources.home.tv.random_channel.loading;
    };

    RandomChannel.prototype.channels_ready_handler = function(channels) {
      var _this = this;
      this.channels = channels;
      if (this.channels.is_empty()) {
        (this.channels_deferred = this.app.user.get_channels()).done(function(channels) {
          _this.channels = channels;
          return _this.select_channel();
        });
      } else {
        this.select_channel();
      }
      return this;
    };

    RandomChannel.prototype.select_channel = function() {
      this.channel = this.channels.get_random();
      this.el.set_content(this.app.platform.templates.home.tv.random_channel.base, this.channel.template_data()).remove_class('loading');
      return this.off('press', this.press_muter, this);
    };

    RandomChannel.prototype.channels_deffered_factory = function() {
      return this.app.user.get_favourited_channels();
    };

    RandomChannel.prototype.get_channels = function() {
      return this.channels;
    };

    RandomChannel.prototype.get_channel = function() {
      return this.channel;
    };

    RandomChannel.prototype.update_right_key_hint = function(right_key_hint_label) {
      return this.hints = [[Rutv.Key.Back, app.resources.hints.exit, this], [Rutv.Key.Ok, app.resources.hints.play, this], [Rutv.Key.Right, right_key_hint_label, this], [Rutv.Key.Up, app.resources.hints.menu, this]];
    };

    /**
    press event never occurs before all channels loaded
    */


    RandomChannel.prototype.press_muter = function() {
      return false;
    };

    return RandomChannel;

  })(Rutv.View.Component.Home.Tv.Button.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Home.Tv.Button.AllChannels = (function(_super) {

    __extends(AllChannels, _super);

    function AllChannels(app, additional_class_name) {
      this.additional_class_name = additional_class_name != null ? additional_class_name : 'home__channels_all';
      AllChannels.__super__.constructor.call(this, app, [[Rutv.Key.Back, app.resources.hints.exit, this], [Rutv.Key.Ok, app.resources.hints.channels_list, this], [Rutv.Key.Left, app.resources.hints.channel, this], [Rutv.Key.Up, app.resources.hints.menu, this], [Rutv.Key.Down, app.resources.hints.favourited_channels, this]]);
    }

    AllChannels.prototype.render = function() {
      AllChannels.__super__.render.call(this);
      this.el.set_classname("home__channels " + this.additional_class_name);
      return this;
    };

    AllChannels.prototype.channels_ready_handler = function(channels) {
      this.channels = channels;
      this.template_name = 'home.tv.channels';
      return this.el.set_content(this.template(), this.app.resources.home.tv.all(this.channels));
    };

    AllChannels.prototype.channels_deffered_factory = function() {
      return this.app.user.get_channels();
    };

    return AllChannels;

  })(Rutv.View.Component.Home.Tv.Button.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Home.Tv.Button.FavouritedChannels = (function(_super) {

    __extends(FavouritedChannels, _super);

    function FavouritedChannels(app, additional_class_name) {
      this.additional_class_name = additional_class_name != null ? additional_class_name : 'home__channels_favourite';
      FavouritedChannels.__super__.constructor.call(this, app, [[Rutv.Key.Back, app.resources.hints.exit, this], [Rutv.Key.Ok, app.resources.hints.channels_list, this], [Rutv.Key.Left, app.resources.hints.channel, this], [Rutv.Key.Up, app.resources.hints.all_channels, this]]);
    }

    FavouritedChannels.prototype.render = function() {
      FavouritedChannels.__super__.render.call(this);
      this.el.set_classname("home__channels " + this.additional_class_name);
      return this;
    };

    FavouritedChannels.prototype.channels_ready_handler = function(channels) {
      this.channels = channels;
      this.template_name = 'home.tv.channels';
      this.update_content();
      return this.app.user.favourited_channels.on('remove add', this.update_content, this);
    };

    FavouritedChannels.prototype.channels_deffered_factory = function() {
      return this.app.user.get_favourited_channels();
    };

    FavouritedChannels.prototype.update_content = function() {
      return this.el.set_content(this.template(), this.app.resources.home.tv.favourites(this.channels));
    };

    return FavouritedChannels;

  })(Rutv.View.Component.Home.Tv.Button.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Channels.Header = (function(_super) {

    __extends(Header, _super);

    Header.prototype.template_name = 'channels.header';

    function Header(app, channels_deferred, resources, additional_class_name) {
      this.app = app;
      this.channels_deferred = channels_deferred;
      this.resources = resources;
      this.additional_class_name = additional_class_name != null ? additional_class_name : '';
      Header.__super__.constructor.call(this, this.app.platform);
    }

    Header.prototype.render = function() {
      var _this = this;
      (this.el = this.el_factory()).set_content(this.template(), this.resources(this.channels)).set_classname("header " + this.additional_class_name);
      this.channels_deferred.done(function(channels) {
        _this.channels = channels;
        return _this.channels.on('current_changed remove add', _this.current_changed_handler, _this);
      });
      return this;
    };

    Header.prototype.current_changed_handler = function() {
      return this.el.set_content(this.template(), this.resources(this.channels));
    };

    Header.prototype.destroy = function() {
      this.channels.off('current_changed', this.current_changed_handler, this);
      return Header.__super__.destroy.call(this);
    };

    Header.prototype.blur = function() {
      this.channels.off('current_changed', this.current_changed_handler, this);
      return this;
    };

    Header.prototype.focus = function() {
      var _ref;
      this.current_changed_handler();
      if ((_ref = this.channels) != null) {
        _ref.on('current_changed', this.current_changed_handler, this);
      }
      return this;
    };

    return Header;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Channels.List = (function(_super) {

    __extends(List, _super);

    List.prototype.template_name = 'channels.list.loading';

    function List(app, channels_deferred, resources) {
      this.app = app;
      this.channels_deferred = channels_deferred;
      this.resources = resources;
      List.__super__.constructor.call(this, this.app.platform);
    }

    List.prototype.render = function() {
      var _this = this;
      (this.el = this.el_factory()).set_classname('list').set_content(this.template(), this.resources.loading);
      this.channels_deferred.done(function(channels) {
        _this.channels = channels;
        _this.channels.on('current_changed remove', _this.current_changed_handler, _this).set_current(_this.channels.get_first());
        return _this.app.platform.key_trap.on('key_press', _this.keypress_handler, _this);
      });
      return this;
    };

    List.prototype.current_changed_handler = function() {
      this.destroy_buttons();
      if (this.channels != null) {
        if (this.channels.get_count() > 0) {
          this.el.empty().append(this.get_left_click_button().el).append(this.get_right_click_button().el);
          this.create_buttons();
        } else {
          this.el.set_content(this.app.platform.templates.channels.list.empty, this.resources.empty);
        }
      }
      if (this.channels.get_previous() != null) {
        this.get_left_click_button().show();
      } else {
        this.get_left_click_button().hide();
      }
      if (this.channels.get_next() != null) {
        return this.get_right_click_button().show();
      } else {
        return this.get_right_click_button().hide();
      }
    };

    List.prototype.keypress_handler = function(key) {
      var _ref;
      switch (key) {
        case Rutv.Key.Blue:
          this.trigger('schedule', this);
          return false;
        case Rutv.Key.Back:
          this.trigger('back', this);
          return false;
        case Rutv.Key.Red:
          if ((_ref = this.focused_button) != null) {
            _ref.favouriter.trigger('press');
          }
          return false;
      }
    };

    List.prototype.get_left_click_button = function() {
      var _this = this;
      if (this.left_click_button == null) {
        this.left_click_button = this.click_button_factory('click-button_left', function() {
          return _this.channels.move_previous();
        });
      }
      return this.left_click_button;
    };

    List.prototype.get_right_click_button = function() {
      var _this = this;
      if (this.right_click_button == null) {
        this.right_click_button = this.click_button_factory('click-button_right', function() {
          return _this.channels.move_next();
        });
      }
      return this.right_click_button;
    };

    List.prototype.click_button_factory = function(class_name, press_callback) {
      return (new Rutv.View.Component.Channels.ClickButton(this.app, class_name)).on('press', press_callback).render().hide();
    };

    List.prototype.focus = function() {
      if (this.channels != null) {
        this.current_changed_handler();
        this.channels.on('current_changed remove', this.current_changed_handler, this);
        this.app.platform.key_trap.on('key_press', this.keypress_handler, this);
        if (this.current_channel_button != null) {
          (this.focused_button = this.current_channel_button).focus();
        }
      }
      return this;
    };

    List.prototype.blur = function() {
      var _ref;
      this.channels.off('current_changed remove', this.current_changed_handler, this);
      this.app.platform.key_trap.off('key_press', this.keypress_handler, this);
      if ((_ref = this.focused_button) != null) {
        _ref.blur();
      }
      return this;
    };

    List.prototype.create_buttons = function() {
      this.previous_channel_button = this.button_factory(this.channels.get_previous(), 'channel_previous');
      this.current_channel_button = this.button_factory(this.channels.get_current(), 'channel_current');
      this.next_channel_button = this.button_factory(this.channels.get_next(), 'channel_next');
      if (this.current_channel_button != null) {
        return (this.focused_button = this.current_channel_button).focus();
      }
    };

    List.prototype.button_factory = function(channel, additional_class_name) {
      var button, hints,
        _this = this;
      hints = this.hints_factory(channel);
      if (channel) {
        (button = new Rutv.View.Component.Channels.Channel.Base(this.app, channel, additional_class_name, hints)).on('back', function() {
          _this.trigger('back');
          return false;
        }).on('exit', function() {
          return _this.trigger('exit');
        }).on('focus', function(button) {
          var _ref;
          if (_this.focused_button !== button) {
            if ((_ref = _this.focused_button) != null) {
              _ref.blur();
            }
            return (_this.focused_button = button).focus();
          }
        }).on('press play', function() {
          return _this.app.scene.subscenes.push(new Rutv.View.Scene.Player(_this.app, channel, _this.channels)).play();
        }).on('left', function() {
          return _this.channels.move_previous();
        }).on('right', function() {
          return _this.channels.move_next();
        });
        this.el.append(button.render().el);
        return button;
      } else {
        return null;
      }
    };

    List.prototype.hints_factory = function(channel) {
      var hints;
      hints = [[Rutv.Key.Ok, this.app.resources.hints.play]];
      if (channel !== this.channels.get_first()) {
        hints.push([Rutv.Key.Left, this.app.resources.hints.previous_channel]);
      }
      if (channel !== this.channels.get_last()) {
        hints.push([Rutv.Key.Right, this.app.resources.hints.next_channel]);
      }
      hints.push([Rutv.Key.Red, this.app.resources.hints.favouriter, this]);
      hints.push([Rutv.Key.Blue, this.app.resources.hints.schedule, this]);
      return hints;
    };

    List.prototype.destroy_buttons = function() {
      var _ref, _ref1, _ref2;
      if ((_ref = this.previous_channel_button) != null) {
        _ref.destroy();
      }
      if ((_ref1 = this.current_channel_button) != null) {
        _ref1.destroy();
      }
      if ((_ref2 = this.next_channel_button) != null) {
        _ref2.destroy();
      }
      this.previous_channel_button = null;
      this.current_channel_button = null;
      return this.next_channel_button = null;
    };

    List.prototype.destroy = function() {
      this.channels.off('current_changed remove', this.current_changed_handler, this);
      this.app.platform.key_trap.off('key_press', this.keypress_handler, this);
      this.destroy_buttons();
      return this;
    };

    return List;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Channels.Channel.Favouriter = (function(_super) {

    __extends(Favouriter, _super);

    function Favouriter(app, channel) {
      this.app = app;
      this.channel = channel;
      Favouriter.__super__.constructor.call(this, this.app.platform, [[Rutv.Key.Ok, app.resources.hints.play, this], [Rutv.Key.Left, app.resources.hints.previous_channel, this], [Rutv.Key.Right, app.resources.hints.next_channel, this], [Rutv.Key.Blue, app.resources.hints.schedule, this], [Rutv.Key.Red, this.app.resources.hints.favourited, this]]);
    }

    Favouriter.prototype.render = function() {
      Favouriter.__super__.render.call(this);
      this.update_state();
      this.on('press', this.press_handler, this);
      this.channel.on('favourited_changed', this.update_state, this);
      return this;
    };

    Favouriter.prototype.press_handler = function() {
      this.channel.toggle_favourited();
      return false;
    };

    Favouriter.prototype.update_state = function() {
      var additional_class_name;
      additional_class_name = this.channel.is_favourited() ? 'favouriter_on' : '';
      return this.el.set_classname("favouriter " + additional_class_name);
    };

    Favouriter.prototype.destroy = function() {
      this.channel.off('favourited_changed', this.update_state, this);
      return Favouriter.__super__.destroy.call(this);
    };

    return Favouriter;

  })(Rutv.View.Component.Button);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Channels.Channel.Base = (function(_super) {

    __extends(Base, _super);

    Base.prototype.template_name = 'channels.list.channel';

    function Base(app, channel, additional_class_name, hints) {
      this.app = app;
      this.channel = channel;
      this.additional_class_name = additional_class_name != null ? additional_class_name : '';
      if (hints == null) {
        hints = [];
      }
      this.set_hints_context(hints);
      Base.__super__.constructor.call(this, this.app.platform, hints);
    }

    Base.prototype.render = function() {
      Base.__super__.render.call(this);
      this.el.set_classname("channel " + this.additional_class_name).append(this.create_favouriter().render().el);
      return this;
    };

    Base.prototype.create_favouriter = function() {
      return this.favouriter = new Rutv.View.Component.Channels.Channel.Favouriter(this.app, this.channel);
    };

    Base.prototype.template_data = function() {
      return this.channel.template_data();
    };

    Base.prototype.set_hints_context = function(hints) {
      var hint, _i, _len, _results;
      _results = [];
      for (_i = 0, _len = hints.length; _i < _len; _i++) {
        hint = hints[_i];
        if (hint[2] == null) {
          _results.push(hint.push(this));
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    Base.prototype.destroy = function() {
      this.favouriter.destroy();
      return Base.__super__.destroy.call(this);
    };

    Base.prototype.focus = function() {
      Base.__super__.focus.call(this);
      this.set_current_actions();
      return this;
    };

    Base.prototype.blur = function() {
      Base.__super__.blur.call(this);
      this.clear_current_actions();
      return this;
    };

    return Base;

  })(Rutv.View.Component.Button);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Channels.ClickButton = (function(_super) {

    __extends(ClickButton, _super);

    function ClickButton(app, additional_class) {
      this.app = app;
      this.additional_class = additional_class != null ? additional_class : '';
      ClickButton.__super__.constructor.call(this, this.app.platform);
    }

    ClickButton.prototype.render = function() {
      ClickButton.__super__.render.call(this);
      this.el.add_class("click-button " + this.additional_class);
      return this;
    };

    ClickButton.prototype.focus = function() {
      this.el.add_class('focused');
      this.set_current_actions();
      return this;
    };

    ClickButton.prototype.blur = function() {
      this.el.remove_class('focused');
      this.clear_current_actions();
      return this;
    };

    ClickButton.prototype.attach_mouse_handlers = function() {
      ClickButton.__super__.attach_mouse_handlers.call(this);
      return this.el.on('mouse_out', this.blur, this);
    };

    return ClickButton;

  })(Rutv.View.Component.Button);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Queue.Header = (function(_super) {

    __extends(Header, _super);

    Header.prototype.template_name = 'channels.header';

    function Header(app, queue_deferred, resources, additional_class_name) {
      this.app = app;
      this.queue_deferred = queue_deferred;
      this.resources = resources;
      this.additional_class_name = additional_class_name != null ? additional_class_name : '';
      Header.__super__.constructor.call(this, this.app.platform);
      this.queue_is_load = false;
    }

    Header.prototype.render = function() {
      var _this = this;
      (this.el = this.el_factory()).set_content(this.template(), this.resources(this.queue)).set_classname("header " + this.additional_class_name);
      this.queue_deferred.done(function(queue) {
        _this.queue = queue;
        _this.queue.on('current_changed remove add', _this.current_changed_handler, _this);
        return _this.queue_is_load = true;
      });
      return this;
    };

    Header.prototype.current_changed_handler = function() {
      return this.el.set_content(this.template(), this.resources(this.queue));
    };

    Header.prototype.destroy = function() {
      if (this.queue_is_load) {
        this.queue.off('current_changed', this.current_changed_handler, this);
      }
      return Header.__super__.destroy.call(this);
    };

    Header.prototype.blur = function() {
      if (this.queue_is_load) {
        this.queue.off('current_changed', this.current_changed_handler, this);
      }
      return this;
    };

    Header.prototype.focus = function() {
      var _ref;
      this.current_changed_handler();
      if ((_ref = this.queue) != null) {
        _ref.on('current_changed', this.current_changed_handler, this);
      }
      return this;
    };

    return Header;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Queue.List = (function(_super) {

    __extends(List, _super);

    List.prototype.template_name = 'queue.list.loading';

    function List(app, queue_deferred, resources) {
      this.app = app;
      this.queue_deferred = queue_deferred;
      this.resources = resources;
      List.__super__.constructor.call(this, this.app.platform);
    }

    List.prototype.render = function() {
      var _this = this;
      (this.el = this.el_factory()).set_classname('list').set_content(this.template(), this.resources.loading);
      this.queue_deferred.done(function(queue) {
        _this.queue = queue;
        _this.queue.on('current_changed remove', _this.current_changed_handler, _this).set_current(_this.queue.get_first());
        return _this.app.platform.key_trap.on('key_press', _this.keypress_handler, _this);
      });
      return this;
    };

    List.prototype.current_changed_handler = function() {
      this.destroy_buttons();
      if (this.queue != null) {
        if (this.queue.get_count() > 0) {
          this.el.empty().append(this.get_left_click_button().el).append(this.get_right_click_button().el);
          this.create_buttons();
        } else {
          this.el.set_content(this.app.platform.templates.queue.list.empty, this.resources.empty);
        }
      }
      if (this.queue.get_previous() != null) {
        this.get_left_click_button().show();
      } else {
        this.get_left_click_button().hide();
      }
      if (this.queue.get_next() != null) {
        return this.get_right_click_button().show();
      } else {
        return this.get_right_click_button().hide();
      }
    };

    List.prototype.keypress_handler = function(key) {
      switch (key) {
        case Rutv.Key.Back:
          this.trigger('back', this);
          return false;
      }
    };

    List.prototype.get_left_click_button = function() {
      var _this = this;
      if (this.left_click_button == null) {
        this.left_click_button = this.click_button_factory('click-button_left', function() {
          return _this.queue.move_previous();
        });
      }
      return this.left_click_button;
    };

    List.prototype.get_right_click_button = function() {
      var _this = this;
      if (this.right_click_button == null) {
        this.right_click_button = this.click_button_factory('click-button_right', function() {
          return _this.queue.move_next();
        });
      }
      return this.right_click_button;
    };

    List.prototype.click_button_factory = function(class_name, press_callback) {
      return (new Rutv.View.Component.Queue.ClickButton(this.app, class_name)).on('press', press_callback).render().hide();
    };

    List.prototype.focus = function() {
      if (this.queue != null) {
        this.current_changed_handler();
        this.queue.on('current_changed remove', this.current_changed_handler, this);
        this.app.platform.key_trap.on('key_press', this.keypress_handler, this);
        if (this.current_programm_button != null) {
          (this.focused_button = this.current_programm_button).focus();
        }
      }
      return this;
    };

    List.prototype.blur = function() {
      var _ref, _ref1;
      if ((_ref = this.queue) != null) {
        _ref.off('current_changed remove', this.current_changed_handler, this);
      }
      this.app.platform.key_trap.off('key_press', this.keypress_handler, this);
      if ((_ref1 = this.focused_button) != null) {
        _ref1.blur();
      }
      return this;
    };

    List.prototype.create_buttons = function() {
      this.previous_programm_button = this.button_factory(this.queue.get_previous(), 'channel_previous');
      this.current_programm_button = this.button_factory(this.queue.get_current(), 'channel_current');
      this.next_programm_button = this.button_factory(this.queue.get_next(), 'channel_next');
      if (this.current_programm_button != null) {
        return (this.focused_button = this.current_programm_button).focus();
      }
    };

    List.prototype.button_factory = function(programm, additional_class_name) {
      var button, hints,
        _this = this;
      hints = this.hints_factory(programm);
      if (programm) {
        (button = new Rutv.View.Component.Queue.Programm(this.app, programm, additional_class_name, hints)).on('back', function() {
          _this.trigger('back');
          return false;
        }).on('exit', function() {
          return _this.trigger('exit');
        }).on('focus', function(button) {
          var _ref;
          if (_this.focused_button !== button) {
            if ((_ref = _this.focused_button) != null) {
              _ref.blur();
            }
            return (_this.focused_button = button).focus();
          }
        }).on('press play', function() {
          return _this.app.scene.subscenes.push(new Rutv.View.Scene.Player(_this.app, programm, _this.queue)).play();
        }).on('left', function() {
          return _this.queue.move_previous();
        }).on('right', function() {
          return _this.queue.move_next();
        });
        this.el.append(button.render().el);
        return button;
      } else {
        return null;
      }
    };

    List.prototype.hints_factory = function(programm) {
      var hints;
      hints = [[Rutv.Key.Ok, this.app.resources.hints.play]];
      if (programm !== this.queue.get_first()) {
        hints.push([Rutv.Key.Left, this.app.resources.hints.previous_channel]);
      }
      if (programm !== this.queue.get_last()) {
        hints.push([Rutv.Key.Right, this.app.resources.hints.next_channel]);
      }
      return hints;
    };

    List.prototype.destroy_buttons = function() {
      var _ref, _ref1, _ref2;
      if ((_ref = this.previous_programm_button) != null) {
        _ref.destroy();
      }
      if ((_ref1 = this.current_programm_button) != null) {
        _ref1.destroy();
      }
      if ((_ref2 = this.next_programm_button) != null) {
        _ref2.destroy();
      }
      this.previous_programm_button = null;
      this.current_programm_button = null;
      return this.next_programm_button = null;
    };

    List.prototype.destroy = function() {
      var _ref;
      if ((_ref = this.queue) != null) {
        _ref.off('current_changed remove', this.current_changed_handler, this);
      }
      this.app.platform.key_trap.off('key_press', this.keypress_handler, this);
      this.destroy_buttons();
      return this;
    };

    return List;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Queue.Programm = (function(_super) {

    __extends(Programm, _super);

    Programm.prototype.template_name = 'queue.list.programm';

    function Programm(app, programm, additional_class_name, hints) {
      this.app = app;
      this.programm = programm;
      this.additional_class_name = additional_class_name != null ? additional_class_name : '';
      if (hints == null) {
        hints = [];
      }
      this.set_hints_context(hints);
      Programm.__super__.constructor.call(this, this.app.platform, hints);
    }

    Programm.prototype.render = function() {
      Programm.__super__.render.call(this);
      this.el.set_classname("channel " + this.additional_class_name);
      return this;
    };

    Programm.prototype.template_data = function() {
      return this.programm.template_data();
    };

    Programm.prototype.set_hints_context = function(hints) {
      var hint, _i, _len, _results;
      _results = [];
      for (_i = 0, _len = hints.length; _i < _len; _i++) {
        hint = hints[_i];
        if (hint[2] == null) {
          _results.push(hint.push(this));
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    Programm.prototype.focus = function() {
      Programm.__super__.focus.call(this);
      this.set_current_actions();
      return this;
    };

    Programm.prototype.blur = function() {
      Programm.__super__.blur.call(this);
      this.clear_current_actions();
      return this;
    };

    return Programm;

  })(Rutv.View.Component.Button);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Queue.ClickButton = (function(_super) {

    __extends(ClickButton, _super);

    function ClickButton(app, additional_class) {
      this.app = app;
      this.additional_class = additional_class != null ? additional_class : '';
      ClickButton.__super__.constructor.call(this, this.app.platform);
    }

    ClickButton.prototype.render = function() {
      ClickButton.__super__.render.call(this);
      this.el.add_class("click-button " + this.additional_class);
      return this;
    };

    ClickButton.prototype.focus = function() {
      this.el.add_class('focused');
      this.set_current_actions();
      return this;
    };

    ClickButton.prototype.blur = function() {
      this.el.remove_class('focused');
      this.clear_current_actions();
      return this;
    };

    ClickButton.prototype.attach_mouse_handlers = function() {
      ClickButton.__super__.attach_mouse_handlers.call(this);
      return this.el.on('mouse_out', this.blur, this);
    };

    return ClickButton;

  })(Rutv.View.Component.Button);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Schedule.Channels = (function(_super) {

    __extends(Channels, _super);

    Channels.prototype.template_name = 'schedule.channels.loading';

    function Channels(app, channels) {
      this.app = app;
      this.channels = channels;
      Channels.__super__.constructor.call(this, this.app.platform);
      this.channel_programms = [];
      this.rendering_channel = false;
    }

    Channels.prototype.render = function() {
      var _this = this;
      (this.el = this.el_factory()).set_classname('schedule__channels').set_content(this.app.platform.templates.schedule.channels.viewport, this.template_data()).find('schedule__channels__viewport').append(this.create_programm_info());
      this.programms_el = this.el.find('channel-programms');
      this.logos_el = this.el.find('channel-logos');
      setTimeout(function() {
        (_this.most_earlier_programm_deffered = _this.channels.get_most_earlier_programm()).fail(function() {
          return _this.render_fail();
        });
        (_this.viewport = new Rutv.View.Component.Schedule.Channel.Viewport(_this.app.config, _this)).on('new_time_borders', function() {
          return _this.update_visible_channels();
        });
        return _this.render_channels();
      }, 500);
      return this;
    };

    Channels.prototype.create_programm_info = function() {
      return (this.programms_info = new Rutv.View.Component.Schedule.ProgrammsInfo(this.app, this.channels)).render().el;
    };

    Channels.prototype.render_channels = function() {
      var i, programms, _i, _ref, _results;
      _results = [];
      for (i = _i = 0, _ref = this.viewport.visible_channels_count; 0 <= _ref ? _i < _ref : _i > _ref; i = 0 <= _ref ? ++_i : --_i) {
        if (this.channels.items[i] == null) {
          break;
        }
        this.channel_programms.push(programms = this.channel_programms_factory(this.channels.items[this.channel_programms.length], this.channel_programms.length));
        this.attach_navigation_behaviour(i);
        _results.push(this.render_channel(this.channels.items[i], programms, this.channel_programms.length - 1));
      }
      return _results;
    };

    Channels.prototype.render_channel = function(channel, programms, programms_index) {
      var _this = this;
      return channel != null ? channel.get_schedule().done(function(programms_data) {
        programms.set_programms_data(programms_data);
        if (programms_data.items.length > 0) {
          _this.viewport.search_most_earlier_programm(programms_data.items);
        }
        programms.render_programms();
        _this.programms_info.clear_channel(programms_index);
        if (programms_index === 0) {
          _this.channel_programms[programms_index].focus();
        }
        return _this.viewport.reset();
      }) : void 0;
    };

    Channels.prototype.attach_navigation_behaviour = function(index) {
      var channel,
        _this = this;
      channel = this.channel_programms[index];
      if (index !== 0) {
        channel.on('up', (function(self, index) {
          return function() {
            var upper_channel;
            _this.render_previous_channel(index);
            if ((upper_channel = _this.find_upper_focusable_channel(index)) != null) {
              upper_channel.focus(_this.channel_programms[index].focused_programm);
              return self.trigger('channel_up', _this);
            }
          };
        })(this, index));
      }
      if (index !== this.channels.items.length - 1) {
        return channel.on('down', (function(self, index) {
          return function() {
            var lower_channel;
            _this.render_next_channel(index);
            if ((lower_channel = _this.find_lower_focusable_channel(index)) != null) {
              lower_channel.focus(_this.channel_programms[index].focused_programm);
              return self.trigger('channel_down', _this);
            }
          };
        })(this, index));
      }
    };

    Channels.prototype.channel_programms_factory = function(channel, channel_index) {
      var programms,
        _this = this;
      this.programms_el.append((programms = new Rutv.View.Component.Schedule.Channel.Programms(this.app, this, channel, this.viewport, channel_index)).on('focus', this.focus_handler, this).on('focus_programm', this.focus_programm_handler, this).on('press_programm', function(programm_component) {
        return _this.trigger('press_programm', _this.focused_channel, programm_component, channel_index);
      }).render().el);
      return programms;
    };

    Channels.prototype.render_fail = function() {
      return this.el.set_content(this.template(), this.app.resources.schedule.channels.load_failed);
    };

    Channels.prototype.template_data = function() {
      var result;
      result = this.app.resources.schedule.channels.loaded;
      result.logo_urls = this.channels.map(function(channel) {
        return channel.get_logo_url();
      });
      return result;
    };

    Channels.prototype.focus_programm_handler = function(programm) {
      return this.trigger('focus_programm', programm);
    };

    Channels.prototype.focus_handler = function(channel_to_focus) {
      this.blur_focused_channel(channel_to_focus);
      return (this.focused_channel = channel_to_focus).force_focused();
    };

    Channels.prototype.blur_focused_channel = function(channel_to_focus) {
      var _ref;
      if (channel_to_focus == null) {
        channel_to_focus = null;
      }
      if (this.focused_channel !== channel_to_focus) {
        return (_ref = this.focused_channel) != null ? _ref.blur() : void 0;
      }
    };

    Channels.prototype.blur = function() {
      var _ref;
      if ((_ref = this.focused_channel) != null) {
        _ref.blur();
      }
      return this;
    };

    Channels.prototype.focus = function() {
      var _ref;
      if ((_ref = this.focused_channel) != null) {
        _ref.focus();
      }
      return this;
    };

    Channels.prototype.find_lower_focusable_channel = function(current_channel_index) {
      return this.find_focusable_channel(current_channel_index, 1);
    };

    Channels.prototype.find_upper_focusable_channel = function(current_channel_index) {
      return this.find_focusable_channel(current_channel_index, -1);
    };

    Channels.prototype.find_focusable_channel = function(current_channel_index, step) {
      var result;
      result = null;
      current_channel_index += step;
      while (this.channel_programms[current_channel_index] != null) {
        if (this.channel_programms[current_channel_index].is_focusable()) {
          result = this.channel_programms[current_channel_index];
          break;
        }
        current_channel_index += step;
      }
      return result;
    };

    Channels.prototype.render_next_channel = function(current_channel_index) {
      var programms;
      if (this.viewport.last_visible_channel(1)) {
        if (this.channel_programms[current_channel_index + 1] != null) {
          this.programms_el.append(this.channel_programms[current_channel_index + 1].el);
          return this.remove_invisible_channel(current_channel_index - 4);
        } else if (this.possible_render_next_channel(current_channel_index)) {
          this.channel_programms.push(programms = this.channel_programms_factory(this.channels.items[this.channel_programms.length], this.channel_programms.length));
          this.attach_navigation_behaviour(current_channel_index + 1);
          this.render_channel(this.channels.items[current_channel_index + 1], programms, current_channel_index + 1);
          return this.remove_invisible_channel(current_channel_index - 4);
        }
      }
    };

    Channels.prototype.render_previous_channel = function(current_channel_index) {
      if (this.viewport.first_visible_channel(-1)) {
        this.remove_invisible_channel(current_channel_index + 4);
        return this.programms_el.prepend(this.channel_programms[current_channel_index - 1].el);
      }
    };

    Channels.prototype.remove_invisible_channel = function(index) {
      var _ref;
      if (this.channel_programms[index].el.node.parentNode === this.programms_el.node) {
        return (_ref = this.channel_programms[index].el) != null ? _ref.destroy() : void 0;
      }
    };

    Channels.prototype.possible_render_next_channel = function(current_channel_index) {
      if (this.channel_programms[current_channel_index + 1] == null) {
        return this.channels.items[current_channel_index + 1] != null;
      }
    };

    Channels.prototype.update_visible_channels = function() {
      var channel, i, _i, _ref, _ref1, _results;
      _results = [];
      for (i = _i = _ref = this.viewport.top_channel_index - 1, _ref1 = this.viewport.top_channel_index + 5; _ref <= _ref1 ? _i <= _ref1 : _i >= _ref1; i = _ref <= _ref1 ? ++_i : --_i) {
        if (this.channel_programms[i] != null) {
          channel = this.channel_programms[i];
          if (channel.el.node.parentNode === this.programms_el.node && (channel.programms_data != null)) {
            _results.push(channel.update_programms());
          } else {
            _results.push(void 0);
          }
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    Channels.prototype.destroy = function() {
      Channels.__super__.destroy.call(this);
      this.blur();
      return this;
    };

    return Channels;

  })(Rutv.View.Component.Base);

  Rutv.Utils.extend(Rutv.View.Component.Schedule.Channels.prototype, Rutv.Utils.Events);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Schedule.Channel.Programms = (function(_super) {

    __extends(Programms, _super);

    Programms.prototype.template_name = 'schedule.channels.programms.not_avalible';

    function Programms(app, channels, channel, viewport, channel_index) {
      this.app = app;
      this.channels = channels;
      this.channel = channel;
      this.viewport = viewport;
      this.channel_index = channel_index;
      Programms.__super__.constructor.call(this, this.app.platform);
      this.programms = [];
      this.focused = false;
      this.current_left = 0;
      this.most_earlier_programm_spacer_size = 0;
    }

    Programms.prototype.render = function() {
      (this.el = this.el_factory()).set_classname('schedule__channel__programms');
      return this;
    };

    Programms.prototype.render_programms = function() {
      var index, _i, _ref,
        _this = this;
      this.viewport.on('new_most_earlier_programm', function() {
        return _this.update_most_earlier_programm_spacer_size();
      });
      if (this.programms_data.is_empty()) {
        return this.render_fail();
      } else {
        this.render_before_programm_spacers();
        for (index = _i = 0, _ref = this.programms_data.items.length; 0 <= _ref ? _i < _ref : _i > _ref; index = 0 <= _ref ? ++_i : --_i) {
          if (this.programms_data.items[index] != null) {
            this.programm_renderer(this.programms_data.items[index], index);
            this.attach_behaviour(index, this.programms[index]);
          }
        }
        this.update_programms();
        this.update_most_earlier_programm_spacer_size();
        if (this.channels.focused_channel == null) {
          return this.focused_channel = this.search_playing_now_programm();
        }
      }
    };

    Programms.prototype.set_programms_data = function(programms_data) {
      return this.programms_data = programms_data;
    };

    Programms.prototype.update_programms = function() {
      var index, programm, programm_is_first, _i, _len, _ref, _results;
      if (!this.programms_data.is_empty()) {
        this.el.empty().append(this.most_earlier_programm_spacer).append(this.invisible_programms_spacer);
        programm_is_first = true;
        _ref = this.programms;
        _results = [];
        for (index = _i = 0, _len = _ref.length; _i < _len; index = ++_i) {
          programm = _ref[index];
          if (this.programm_in_viewport(programm.programm)) {
            if (programm_is_first) {
              this.update_invisible_programm_spacer_size(this.programms_data.items[index]);
              programm_is_first = false;
            }
            _results.push(this.el.append(programm.el));
          } else {
            _results.push(void 0);
          }
        }
        return _results;
      }
    };

    Programms.prototype.render_before_programm_spacers = function() {
      return this.el.append((this.most_earlier_programm_spacer = this.el_factory()).set_classname('schedule__channel__programm-spacer')).append((this.invisible_programms_spacer = this.el_factory()).set_classname('schedule__channel__programm-spacer'));
    };

    Programms.prototype.update_most_earlier_programm_spacer_size = function() {
      var addeding_size, first_programm, size;
      if ((first_programm = this.programms_data.get_first()) != null) {
        size = (first_programm.beginning - this.viewport.most_earlier_programm.beginning) / this.app.config.view.window.size_program_in_schedule;
        if (size !== this.most_earlier_programm_spacer_size) {
          addeding_size = size - this.most_earlier_programm_spacer_size;
          this.most_earlier_programm_spacer.set_width(size);
          this.update_programms_left(addeding_size);
          return this.most_earlier_programm_spacer_size = size;
        }
      }
    };

    Programms.prototype.update_invisible_programm_spacer_size = function(first_visible_programm) {
      var first_programm, size;
      if ((first_programm = this.programms_data.get_first()) != null) {
        size = (first_visible_programm.beginning - first_programm.beginning) / this.app.config.view.window.size_program_in_schedule;
        return this.invisible_programms_spacer.set_width(size);
      }
    };

    Programms.prototype.update_programms_left = function(addeding_size) {
      var programm, _i, _len, _ref, _results;
      this.current_left += addeding_size;
      if (this.channels.focused_channel === this) {
        this.viewport.left -= addeding_size;
        this.viewport.change_programm_position(this.viewport.left);
      }
      _ref = this.programms;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        programm = _ref[_i];
        _results.push(programm.add_left(addeding_size));
      }
      return _results;
    };

    Programms.prototype.destroy = function() {
      var programm, _i, _len, _ref;
      _ref = this.programms;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        programm = _ref[_i];
        programm.destroy();
      }
      return Programms.__super__.destroy.call(this);
    };

    Programms.prototype.programm_renderer = function(programm, index) {
      var current_programm, hints,
        _this = this;
      hints = this.hints_factory(index);
      (current_programm = new Rutv.View.Component.Schedule.Channel.Programm(this.app, programm, this.current_left, this.channel_index, hints)).on('up', function() {
        return _this.trigger('up');
      }).on('down', function() {
        return _this.trigger('down');
      }).on('before_focus', this.blur_focused_programm, this).on('focus', this.focus_handler, this).on('press play', function(programm_component) {
        return _this.trigger('press_programm', programm_component);
      });
      this.current_left += current_programm.get_width();
      this.programms.push(current_programm);
      return current_programm.render();
    };

    Programms.prototype.focus_handler = function(focused_programm) {
      this.focused_programm = focused_programm;
      this.trigger('focus', this);
      return this.trigger('focus_programm', this.focused_programm);
    };

    Programms.prototype.blur_focused_programm = function(programm_to_focus) {
      var _ref;
      if (this.focused_programm !== programm_to_focus) {
        if ((_ref = this.focused_programm) != null) {
          _ref.blur();
        }
      }
      return this.trigger('before_focus', this);
    };

    Programms.prototype.force_focused = function() {
      return this.focused = true;
    };

    Programms.prototype.focus = function(previous_programm) {
      var _ref;
      if (!this.focused) {
        this.trigger('before_focus', this);
        if (previous_programm != null) {
          if (this.programms.length > 0) {
            if (!this.programms_data.is_empty()) {
              this.search_nearest_programm(previous_programm).focus();
            }
          }
        } else {
          if (this.focused_programm != null) {
            if ((_ref = this.focused_programm) != null) {
              _ref.focus();
            }
          } else {
            if (!this.programms_data.is_empty()) {
              this.search_playing_now_programm().focus();
            }
          }
        }
        this.force_focused();
        this.trigger('focus', this);
      }
      return this;
    };

    Programms.prototype.blur = function() {
      if (this.focused) {
        this.trigger('before_blur', this);
        this.blur_focused_programm();
        this.focused = false;
        this.trigger('blur', this);
      }
      return this;
    };

    Programms.prototype.is_focusable = function() {
      return this.programms.length > 0;
    };

    Programms.prototype.attach_behaviour = function(index, programm) {
      if (index !== 0) {
        programm.on('left', (function(self, index) {
          return function() {
            var _ref;
            return (_ref = self.programms[self.current_programm_index = index - 1]) != null ? _ref.focus() : void 0;
          };
        })(this, index));
      }
      return programm.on('right', (function(self, index) {
        return function() {
          var _ref;
          return (_ref = self.programms[self.current_programm_index = index + 1]) != null ? _ref.focus() : void 0;
        };
      })(this, index));
    };

    Programms.prototype.render_fail = function() {
      return this.viewport.programms_info.add_channel_info(this.channel_index, this.app.resources.schedule.channels.programms_not_avalible, 'programms_not_avalible');
    };

    Programms.prototype.search_nearest_programm = function(previous_programm) {
      return this.search_programm_on_time((previous_programm.programm.get_beginning() + previous_programm.programm.get_ending()) / 2);
    };

    Programms.prototype.search_playing_now_programm = function() {
      return this.search_programm_on_time(this.calc_current_time());
    };

    Programms.prototype.search_programm_on_time = function(time, programms) {
      var found_programm, programm, _i, _len;
      if (programms == null) {
        programms = this.programms;
      }
      found_programm = programms[0];
      for (_i = 0, _len = programms.length; _i < _len; _i++) {
        programm = programms[_i];
        if ((programm.programm.get_beginning() <= time && time < programm.programm.get_ending())) {
          found_programm = programm;
          break;
        }
      }
      return found_programm;
    };

    Programms.prototype.calc_current_time = function() {
      var now_time;
      return now_time = parseInt((new Date().getTime() / 1000).toFixed());
    };

    Programms.prototype.programm_in_viewport = function(programm) {
      var _ref, _ref1, _ref2;
      return (this.viewport.left_time_border <= (_ref = programm.get_beginning()) && _ref <= this.viewport.right_time_border) || (this.viewport.left_time_border <= (_ref1 = programm.get_ending()) && _ref1 <= this.viewport.right_time_border) || (programm.get_beginning() <= (_ref2 = this.viewport.left_time_border) && _ref2 <= programm.get_ending());
    };

    Programms.prototype.hints_factory = function(index) {
      var hints;
      hints = [];
      if (this.programms_data.items[index].get_available()) {
        hints.push([Rutv.Key.Ok, app.resources.hints.play]);
      }
      if (this.channel_index > 0) {
        hints.push([Rutv.Key.Up, app.resources.hints.previous_schedule_channel]);
      }
      hints.push([Rutv.Key.Down, app.resources.hints.next_schedule_channel]);
      if (index > 0) {
        hints.push([Rutv.Key.Left, app.resources.hints.previous_programm]);
      }
      hints.push([Rutv.Key.Right, app.resources.hints.next_programm]);
      return hints;
    };

    return Programms;

  })(Rutv.View.Component.Base);

  Rutv.Utils.extend(Rutv.View.Component.Schedule.Channel.Programms.prototype, Rutv.Utils.Events);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Schedule.Channel.Programm = (function(_super) {

    __extends(Programm, _super);

    Programm.prototype.template_name = 'schedule.channels.programm';

    function Programm(app, programm, left, channel_index, hints) {
      this.app = app;
      this.programm = programm;
      this.left = left != null ? left : 0;
      this.channel_index = channel_index;
      if (hints == null) {
        hints = [];
      }
      this.set_hints_context(hints);
      Programm.__super__.constructor.call(this, this.app.platform, hints);
    }

    Programm.prototype.get_programm = function() {
      return this.programm;
    };

    Programm.prototype.get_left = function() {
      return this.left;
    };

    Programm.prototype.add_left = function(left) {
      return this.left += left;
    };

    Programm.prototype.get_width = function() {
      return this.programm.get_duration() / this.app.config.view.window.size_program_in_schedule;
    };

    Programm.prototype.render = function() {
      Programm.__super__.render.call(this);
      this.el.set_width(this.programm.get_duration() / this.app.config.view.window.size_program_in_schedule - 2);
      this.el.set_classname('schedule__channel__programm');
      if (!this.programm.get_available()) {
        this.el.add_class('unavailable');
      }
      return this;
    };

    Programm.prototype.template_data = function() {
      return {
        title: "" + (this.programm.get_beginning_label()) + " " + (this.programm.get_title())
      };
    };

    Programm.prototype.set_hints_context = function(hints) {
      var hint, _i, _len, _results;
      _results = [];
      for (_i = 0, _len = hints.length; _i < _len; _i++) {
        hint = hints[_i];
        if (hint[2] == null) {
          _results.push(hint.push(this));
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    return Programm;

  })(Rutv.View.Component.Button);

}).call(this);
(function() {

  Rutv.View.Component.Schedule.Channel.Viewport = (function() {

    function Viewport(config, channels_list, visible_channels_count, channel_height, max_width) {
      this.config = config;
      this.channels_list = channels_list;
      this.visible_channels_count = visible_channels_count != null ? visible_channels_count : 5;
      this.channel_height = channel_height != null ? channel_height : 60;
      this.max_width = max_width != null ? max_width : 1200;
      this.logos_el = this.channels_list.logos_el;
      this.programms_info = this.channels_list.programms_info;
      this.left = 0;
      this.top_channel_index = this.current_channel_index = 0;
      this.max_channel_index = this.channels_list.channels.get_count();
      this.right_time_border = 0;
      this.left_time_border = 0;
      this.size_in_seconds = 1200 * this.config.view.window.size_program_in_schedule;
      this.channels_list.on('channel_up', this.channel_up_handler, this).on('channel_down', this.channel_down_handler, this).on('focus_programm', this.focus_programm_handler, this);
      this.programms_el = this.channels_list.programms_el;
    }

    Viewport.prototype.channel_up_handler = function() {
      if (!(this.current_channel_index < 0)) {
        if (this.first_visible_channel()) {
          return this.change_top_channel_index(this.top_channel_index - 1);
        }
      }
    };

    Viewport.prototype.channel_down_handler = function() {
      if (this.current_channel_index !== this.max_channel_index) {
        if (this.last_visible_channel()) {
          return this.change_top_channel_index(this.top_channel_index + 1);
        }
      }
    };

    Viewport.prototype.focus_programm_handler = function(programm) {
      var left, not_shown_width, shown_width;
      this.current_channel_index = programm.channel_index;
      left = Math.abs(this.left);
      if (left + this.max_width < programm.get_left() + programm.get_width()) {
        if (programm.get_left() < left + this.max_width) {
          shown_width = left + this.max_width - programm.get_left();
          if (programm.get_width() > this.max_width) {
            this.left = -1 * programm.get_left();
          } else {
            this.left -= programm.get_width() - shown_width;
          }
        } else {
          this.left = -1 * programm.get_left();
        }
        this.programms_el.node.style.left = "" + this.left + "px";
      } else {
        if (programm.get_left() < left) {
          if (programm.get_left() + programm.get_width() > left) {
            if (programm.get_width() > this.max_width) {
              this.left = -programm.get_left();
            } else {
              not_shown_width = programm.get_width() - (programm.get_left() + programm.get_width() - left);
              this.left += not_shown_width;
            }
          } else {
            this.left = -1 * (programm.get_left());
            if (this.left > 0) {
              this.left = 0;
            }
          }
          this.change_programm_position(this.left);
        }
      }
      return this.change_time_borders(programm);
    };

    Viewport.prototype.change_time_borders = function(programm) {
      var left;
      left = (programm.get_left() - Math.abs(this.left)) * config.view.window.size_program_in_schedule;
      this.left_time_border = programm.programm.beginning - left;
      this.right_time_border = programm.programm.beginning + this.size_in_seconds - left;
      return this.trigger('new_time_borders');
    };

    Viewport.prototype.change_top_channel_index = function(top_channel_index) {
      var top_value;
      this.top_channel_index = top_channel_index;
      top_value = "" + (-1 * this.top_channel_index * this.channel_height) + "px";
      this.logos_el.node.style.top = top_value;
      return this.programms_info.el.node.style.top = top_value;
    };

    Viewport.prototype.change_programm_position = function(left) {
      return this.programms_el.node.style.left = "" + left + "px";
    };

    Viewport.prototype.reset = function() {
      var focused_channel, focused_programm;
      if (((focused_channel = this.channels_list.focused_channel) != null) && this.channels_list.channel_programms.length > 0) {
        focused_channel.focus();
        if ((focused_programm = focused_channel.focused_programm) != null) {
          this.change_programm_position(this.left = -1 * focused_programm.get_left());
          this.change_time_borders(focused_programm);
        }
      }
      return this;
    };

    Viewport.prototype.first_visible_channel = function(offset_next_channel) {
      if (offset_next_channel == null) {
        offset_next_channel = 0;
      }
      return this.top_channel_index - 1 === this.current_channel_index + offset_next_channel;
    };

    Viewport.prototype.last_visible_channel = function(offset_next_channel) {
      if (offset_next_channel == null) {
        offset_next_channel = 0;
      }
      return this.top_channel_index + this.visible_channels_count === this.current_channel_index + offset_next_channel;
    };

    Viewport.prototype.search_most_earlier_programm = function(programms_data) {
      if (this.most_earlier_programm == null) {
        this.most_earlier_programm = programms_data[0];
      }
      if (this.most_earlier_programm.beginning > programms_data[0].beginning) {
        this.most_earlier_programm = programms_data[0];
        return this.trigger('new_most_earlier_programm');
      }
    };

    return Viewport;

  })();

  Rutv.Utils.extend(Rutv.View.Component.Schedule.Channel.Viewport.prototype, Rutv.Utils.Events);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Schedule.AboutProgramm = (function(_super) {

    __extends(AboutProgramm, _super);

    AboutProgramm.prototype.template_name = 'schedule.about_programm';

    function AboutProgramm(app) {
      this.app = app;
      AboutProgramm.__super__.constructor.call(this, this.app.platform);
    }

    AboutProgramm.prototype.render = function() {
      (this.el = this.el_factory()).set_classname('schedule__about-programm').hide();
      return this;
    };

    AboutProgramm.prototype.set_programm = function(programm) {
      var image,
        _this = this;
      this.programm = programm;
      this.el.set_content(this.template(), this.template_data()).show();
      return (image = new Rutv.Lib.ImageElement(this.app.platform, this.el.find('schedule__about-programm__thumb').node)).set_src(this.programm.get_thumb_url()).fail(function() {
        return typeof image !== "undefined" && image !== null ? image.destroy() : void 0;
      });
    };

    AboutProgramm.prototype.template_data = function() {
      var result;
      result = this.programm.template_data();
      result.unavailable_label = this.app.resources.schedule.channels.loaded.unavailable;
      return result;
    };

    return AboutProgramm;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Schedule.ProgrammsInfo = (function(_super) {

    __extends(ProgrammsInfo, _super);

    function ProgrammsInfo(app, channels_data) {
      this.app = app;
      this.channels_data = channels_data;
      ProgrammsInfo.__super__.constructor.call(this, this.app.platform);
      this.channels = [];
    }

    ProgrammsInfo.prototype.render = function() {
      (this.el = this.el_factory()).set_classname('schedule__programms__info');
      this.add_channels();
      return this;
    };

    ProgrammsInfo.prototype.add_channels = function() {
      var i, _i, _ref, _results;
      _results = [];
      for (i = _i = 0, _ref = this.channels_data.items.length; 0 <= _ref ? _i < _ref : _i > _ref; i = 0 <= _ref ? ++_i : --_i) {
        _results.push(this.add_channel().add_channel_info(i, this.app.resources.schedule.channels.loading));
      }
      return _results;
    };

    ProgrammsInfo.prototype.add_channel = function() {
      var channel;
      this.el.append(channel = new Rutv.Lib.Element(this.app.platform).set_class('channel_info'));
      this.channels.push(channel);
      return this;
    };

    ProgrammsInfo.prototype.add_channel_info = function(number, info, class_name) {
      var channel;
      if ((channel = this.channels[number]) != null) {
        this.channels[number].set_content(info);
        if (class_name != null) {
          this.channels[number].set_class(class_name);
        }
      }
      return this;
    };

    ProgrammsInfo.prototype.clear_channel = function(number) {
      var _ref;
      if ((_ref = this.channels[number]) != null) {
        _ref.set_content('');
      }
      return this;
    };

    return ProgrammsInfo;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Player.Band = (function(_super) {

    __extends(Band, _super);

    Band.prototype.template_name = 'player.band';

    function Band(app, video_player) {
      var _this = this;
      this.app = app;
      this.video_player = video_player;
      Band.__super__.constructor.call(this, this.app.platform);
      this.band_step = 0;
      this.band_size = 454;
      (this.plugin = this.video_player.player).on('finished', function() {
        _this.trigger('finished');
        return _this.stop_move();
      });
    }

    Band.prototype.render = function() {
      (this.el = this.el_factory()).set_content(this.template()).set_classname('player__controls__band__container').hide();
      this.mark_el = this.el.find('player__controls__band__mark');
      this.offset_el = this.el.find('player__controls__band__offset');
      this.current_time_el = this.el.find('player__controls__band__current__time');
      this.ending_time_el = this.el.find('player__controls__band__ending__time');
      return this;
    };

    Band.prototype.show = function() {
      Band.__super__.show.call(this);
      this.clear();
      this.calc_band_step();
      this.start_mark_move();
      this.update_time_labels();
      return this;
    };

    Band.prototype.move_mark = function() {
      var _ref;
      if ((0 < (_ref = this.plugin.get_current_play_time()) && _ref <= this.duration)) {
        this.move_mark_to(this.plugin.get_current_play_time());
        return this.update_time_labels();
      }
    };

    Band.prototype.move_mark_to = function(time) {
      var mark_position;
      mark_position = time * this.band_step;
      this.mark_el.node.style.left = "" + mark_position + "px";
      return this.offset_el.node.style.width = "" + mark_position + "px";
    };

    Band.prototype.start_mark_move = function() {
      var _this = this;
      return this.move_interval = setInterval(function() {
        return _this.move_mark();
      }, 1000);
    };

    Band.prototype.stop_move = function() {
      return clearInterval(this.move_interval);
    };

    Band.prototype.update_time_labels = function() {
      this.current_time_el.set_content(this.time_to_string(this.plugin.get_current_play_time()));
      return this.ending_time_el.set_content(this.time_to_string(this.duration));
    };

    Band.prototype.set_duration = function(duration) {
      if (isNaN(duration)) {
        return this.duration = 0;
      } else {
        return this.duration = Math.round(parseFloat(duration));
      }
    };

    Band.prototype.calc_band_step = function() {
      return this.band_step = this.band_size / this.duration;
    };

    Band.prototype.create_mark = function() {
      return this.mark = new Rutv.View.Component.Button(this.app.platform);
    };

    Band.prototype.clear = function() {
      this.stop_move();
      return this.move_mark_to(this.video_player.detect_start_play_time());
    };

    Band.prototype.time_to_string = function(data_time) {
      var hours, minutes, seconds, time;
      time = parseInt(data_time);
      hours = Math.floor(time / 3600);
      minutes = Math.floor((time - hours * 3600) / 60);
      seconds = Math.floor(time - minutes * 60 - hours * 3600);
      hours = this.add_zero_before_number(hours);
      minutes = this.add_zero_before_number(minutes);
      seconds = this.add_zero_before_number(seconds);
      return "" + hours + ":" + minutes + ":" + seconds;
    };

    Band.prototype.add_zero_before_number = function(number) {
      if (number < 10) {
        number = '0' + number.toString();
      }
      return number;
    };

    Band.prototype.destroy = function() {
      return this.clear();
    };

    return Band;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Player.Controls = (function(_super) {

    __extends(Controls, _super);

    function Controls(app, channels) {
      this.app = app;
      this.channels = channels;
      Controls.__super__.constructor.call(this, this.app.platform);
      this.playing = true;
      this.buttons_list = [];
      this.buttons = [];
      this.focused_button_name = 'play';
      this.rewind_enable = true;
    }

    Controls.prototype.render = function() {
      (this.el = this.el_factory()).set_classname('player-panel');
      this.use_without_rewind_buttons();
      return this;
    };

    Controls.prototype.create_butons = function() {
      var button, item, _i, _len, _ref;
      this.buttons_destroy();
      _ref = this.buttons_list;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        this.buttons.push(button = this.button_factory(item, this.el));
        this.attach_press_handler(button, this.detect_press_event_handler(item));
      }
      return this.init_navigate_behaviours();
    };

    Controls.prototype.attach_press_handler = function(button, handler) {
      return button.on('press', handler, this);
    };

    Controls.prototype.detach_press_handler = function(button, handler) {
      return button.off('press', handler, this);
    };

    Controls.prototype.init_navigate_behaviours = function() {
      var button, index, _i, _len, _ref, _ref1;
      this.focused_button = null;
      _ref = this.buttons;
      for (index = _i = 0, _len = _ref.length; _i < _len; index = ++_i) {
        button = _ref[index];
        if (button.name === this.focused_button_name) {
          button.focus();
        }
        this.init_navigate_behaviour(button, index);
      }
      if (this.focused_button == null) {
        return (_ref1 = this.buttons[0]) != null ? _ref1.focus() : void 0;
      }
    };

    Controls.prototype.init_navigate_behaviour = function(button, index) {
      var _this = this;
      if (this.buttons[index - 1] != null) {
        button.on('left', function() {
          return _this.focus_button(_this.buttons[index - 1]);
        });
      }
      if (this.buttons[index + 1] != null) {
        return button.on('right', function() {
          return _this.focus_button(_this.buttons[index + 1]);
        });
      }
    };

    Controls.prototype.detect_press_event_handler = function(name) {
      var handler;
      handler = null;
      switch (name) {
        case 'play':
          handler = this.play_pause_toggle;
          break;
        case 'prev-channel':
          handler = this.prev_channel;
          break;
        case 'rewind':
          handler = this.rewind;
          break;
        case 'toggle_ratio':
          handler = this.toggle_ratio;
          break;
        case 'forward':
          handler = this.forward;
          break;
        case 'next-channel':
          handler = this.next_channel;
          break;
        case 'exit':
          handler = this.back;
      }
      return handler;
    };

    Controls.prototype.use_with_rewind_buttons = function() {
      if (!this.rewind_enable) {
        this.buttons_list = this.platform.player.capabilities_with_rewind();
        this.create_butons();
        this.el.add_class('with_band');
        return this.rewind_enable = true;
      }
    };

    Controls.prototype.use_without_rewind_buttons = function() {
      if (this.rewind_enable) {
        this.buttons_list = this.platform.player.capabilities_without_rewind();
        this.create_butons();
        this.el.remove_class('with_band');
        return this.rewind_enable = false;
      }
    };

    Controls.prototype.next_channel = function() {
      var _ref;
      if (this.showing) {
        this.trigger('move_next');
        if ((_ref = this.buttons[0]) != null) {
          _ref.el.remove_class('playing');
        }
        return this.playing = true;
      }
    };

    Controls.prototype.prev_channel = function() {
      var _ref;
      if (this.showing) {
        this.trigger('move_previous');
        if ((_ref = this.buttons[0]) != null) {
          _ref.el.remove_class('playing');
        }
        return this.playing = true;
      }
    };

    Controls.prototype.rewind = function() {
      if (this.showing) {
        return this.trigger('rewind');
      }
    };

    Controls.prototype.forward = function() {
      if (this.showing) {
        return this.trigger('forward');
      }
    };

    Controls.prototype.toggle_ratio = function() {
      if (this.showing) {
        return this.trigger('toggle_ratio');
      }
    };

    Controls.prototype.play = function() {
      var _ref;
      if ((_ref = this.buttons[0]) != null) {
        _ref.el.remove_class('playing');
      }
      this.playing = !this.playing;
      return this.trigger('play');
    };

    Controls.prototype.pause = function() {
      var _ref;
      if ((_ref = this.buttons[0]) != null) {
        _ref.el.add_class('playing');
      }
      this.playing = !this.playing;
      return this.trigger('pause');
    };

    Controls.prototype.play_pause_toggle = function() {
      if (this.showing) {
        if (this.playing) {
          return this.pause();
        } else {
          return this.play();
        }
      }
    };

    Controls.prototype.button_factory = function(classname, parent_el) {
      var button,
        _this = this;
      (button = new Rutv.View.Component.Button(this.app.platform)).on('back', function() {
        return _this.trigger('back');
      }).on('exit', function() {
        return _this.trigger('exit');
      }).on('before_focus', this.blur_focused, this).on('focus', this.button_focus_handler, this).on('play', function() {
        return _this.play();
      }).on('pause', function() {
        return _this.pause();
      }).on('rewind', function() {
        return _this.trigger('rewind');
      }).on('forward', function() {
        return _this.trigger('forward');
      }).on('stop', function() {
        return _this.back();
      }).on('volume_up', function() {
        return _this.trigger('volume_up');
      }).on('volume_down', function() {
        return _this.trigger('volume_down');
      }).on('toggle_mute', function() {
        return _this.trigger('toggle_mute');
      }).render().el.set_classname(classname);
      if (parent_el != null) {
        parent_el.append(button.el);
      }
      button.name = classname;
      return button;
    };

    Controls.prototype.focus_button = function(button) {
      if (this.showing) {
        return button.focus();
      }
    };

    Controls.prototype.blur_focused = function() {
      var _ref;
      return (_ref = this.focused_button) != null ? _ref.blur() : void 0;
    };

    Controls.prototype.button_focus_handler = function(button) {
      this.focused_button = button;
      return this.focused_button_name = button.name;
    };

    Controls.prototype.blur = function() {
      var _ref;
      if ((_ref = this.focused_button) != null) {
        _ref.blur();
      }
      return this;
    };

    Controls.prototype.focus = function() {
      var _ref;
      if ((_ref = this.focused_button) != null) {
        _ref.focus();
      }
      return this;
    };

    Controls.prototype.back = function() {
      if (this.showing) {
        return this.trigger('back');
      }
    };

    Controls.prototype.show = function() {
      this.showing = true;
      return this;
    };

    Controls.prototype.hide = function() {
      this.showing = false;
      return this;
    };

    Controls.prototype.buttons_destroy = function() {
      var button, _i, _len, _ref, _ref1;
      if ((_ref = this.focused_button) != null) {
        _ref.blur();
      }
      _ref1 = this.buttons;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        button = _ref1[_i];
        button.destroy();
        this.detach_press_handler(button, this.detect_press_event_handler(button.name));
      }
      return this.buttons = [];
    };

    Controls.prototype.destroy = function() {
      this.buttons_destroy();
      return Controls.__super__.destroy.call(this);
    };

    return Controls;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Player.Video = (function(_super) {

    __extends(Video, _super);

    function Video(app, collection, collection_type, info_display) {
      this.app = app;
      this.collection = collection;
      this.collection_type = collection_type != null ? collection_type : null;
      this.info_display = info_display;
      Video.__super__.constructor.call(this, this.app.platform);
      this.schedule_is_load = false;
      this.current_play_head_time = 0;
      this.start_play_time = 0;
      this.current_programm_index = null;
      this.init_current_title_content_for_hls_throttle();
    }

    Video.prototype.render = function() {
      var _this = this;
      (this.player = this.app.platform.player_factory()).video.on('mouse_move', function() {
        return _this.trigger('mouse_move');
      });
      (this.el = this.el_factory()).append(this.player.el).set_classname('player-screen');
      this.collection.on('current_changed', this.current_changed_handler, this);
      return this;
    };

    Video.prototype.resume = function() {
      this.player.resume();
      if (this.collection_type === 'queue') {
        this.start_update_queue_status_interval();
      }
      return this.trigger('resume');
    };

    Video.prototype.pause = function() {
      this.player.pause();
      if (this.collection_type === 'queue') {
        clearInterval(this.queue_status_interval);
      }
      return this.trigger('pause');
    };

    Video.prototype.play = function(playable) {
      var streaming_url;
      this.start_play_time = Math.round((new Date()).getTime() / 1000);
      this.current_programm_index = this.collection.get_current_index();
      this.current_title_content_for_hls();
      if (streaming_url = playable.get_current_streaming_url()) {
        if (this.collection_type === 'queue') {
          this.move_to_begin_time();
        }
        this.player.play(streaming_url, playable.get_width(), playable.get_height());
        if (this.collection_type === 'queue') {
          return this.player.pause();
        }
      } else {
        return this.app.scene.modals.show(new Rutv.View.Modal.Error.NoStreamingUrl(this.app));
      }
    };

    Video.prototype.move_to_begin_time = function() {
      var _this = this;
      return this.player.on_current_play_time().done(function() {
        _this.player.move_to_time(_this.detect_start_play_time());
        return _this.player.resume();
      });
    };

    Video.prototype.detect_start_play_time = function() {
      var current_time, _ref;
      if (((_ref = this.collection.get_current()) != null ? _ref.playhead_time : void 0) != null) {
        current_time = this.collection.get_current().get_playhead_time();
      } else {
        current_time = 0;
      }
      return current_time;
    };

    Video.prototype.current_changed_handler = function() {
      var current;
      current = this.collection.get_current();
      this.play(current);
      if (this.current_is_live_streaming()) {
        this.trigger('current_is_live_streaming');
      } else {
        this.trigger('current_changed', current.get_duration());
      }
      if (this.collection_type === 'queue') {
        return this.start_update_queue_status_interval();
      }
    };

    Video.prototype.load_schedule = function() {
      var _this = this;
      this.current_channel = this.collection.get_current();
      return this.current_channel.get_schedule().done(function() {
        _this.schedule_is_load = true;
        _this.set_current();
        _this.set_collection(_this.current_channel.schedule);
        return _this.trigger('schedule_load');
      });
    };

    Video.prototype.set_collection = function(collection) {
      this.collection.off('current_changed', this.current_changed_handler, this);
      this.collection = collection;
      return this.collection.on('current_changed', this.current_changed_handler, this);
    };

    Video.prototype.move_next = function() {
      if (this.collection_type === 'channels') {
        if (this.schedule_is_load) {
          this.collection.move_next();
          return this.collection_type = 'programms';
        }
      } else {
        return this.collection.move_next();
      }
    };

    Video.prototype.move_previous = function() {
      if (this.collection_type === 'channels') {
        if (this.schedule_is_load) {
          this.collection.move_previous();
          return this.collection_type = 'programms';
        }
      } else {
        return this.collection.move_previous();
      }
    };

    Video.prototype.rewind = function() {
      if (!this.current_is_live_streaming()) {
        this.player.rewind();
        if (this.collection_type === 'queue') {
          return this.update_queue_status();
        }
      }
    };

    Video.prototype.forward = function() {
      if (!this.current_is_live_streaming()) {
        this.player.forward();
        if (this.collection_type === 'queue') {
          return this.update_queue_status();
        }
      }
    };

    Video.prototype.toggle_ratio = function() {
      this.player.toggle_ratio();
      return this.info_display.set_content(this.player.get_ratio_status());
    };

    Video.prototype.set_current = function() {
      this.collection.get_current().schedule.set_current(this.collection.get_current());
      return this.current_programm_index = this.collection.get_current_index();
    };

    Video.prototype.current_is_live_streaming = function() {
      return (this.collection.get_current().channel_name != null) || this.current_is_last_programm();
    };

    Video.prototype.current_is_last_programm = function() {
      return this.collection_type === 'programms' && this.collection.get_current() === this.collection.get_last_avaliable();
    };

    Video.prototype.start_update_queue_status_interval = function() {
      var _this = this;
      clearInterval(this.update_queue_status_interval);
      return this.update_queue_status_interval = setInterval(function() {
        return _this.update_queue_status();
      }, this.app.config.player.update_queue_status_interval);
    };

    Video.prototype.update_queue_status = function() {
      if (this.player.get_current_play_time() !== this.current_play_head_time) {
        this.collection.get_current().set_playhead_time(this.player.get_current_play_time());
        this.app.server.update_queue_status(this.app.user.get_token(), this.collection.get_current().get_airtime(), this.player.get_current_play_time());
        return this.current_play_head_time = this.player.get_current_play_time();
      }
    };

    Video.prototype.current_title_content_for_hls = function() {
      return this.current_title_content_for_hls_throttle();
    };

    Video.prototype.init_current_title_content_for_hls_throttle = function() {
      var _this = this;
      return this.current_title_content_for_hls_throttle = throttle(function() {
        var current_programm;
        if (_this.collection_type === 'channels' && _this.schedule_is_load || _this.current_is_last_programm()) {
          current_programm = _this.detect_current_programm_for_hls(_this.start_play_time + _this.detect_current_play_time());
          return _this.trigger('current_programm_content_changed', _this.detect_title_content_for_programm(current_programm));
        } else if (_this.collection_type === 'channels' && !_this.schedule_is_load) {
          return _this.trigger('current_programm_content_changed', _this.detect_title_content_for_programm(_this.collection.get_current()));
        }
      }, 100);
    };

    Video.prototype.detect_title_content_for_programm = function(current_programm) {
      var content, current_time;
      current_time = this.detect_current_time_for_hls();
      if (current_programm != null) {
        content = "" + current_time + " " + (current_programm.get_title());
      } else {
        content = current_time;
      }
      return content;
    };

    Video.prototype.detect_current_programm_for_hls = function(current_time) {
      var current_programm, index, programm, _i, _ref, _ref1;
      current_programm = null;
      for (index = _i = _ref = this.current_programm_index, _ref1 = this.collection.items.length - 1; _ref <= _ref1 ? _i <= _ref1 : _i >= _ref1; index = _ref <= _ref1 ? ++_i : --_i) {
        programm = this.collection.items[index];
        if ((programm.beginning <= current_time && current_time < programm.ending)) {
          current_programm = programm;
          break;
        }
      }
      return current_programm;
    };

    Video.prototype.detect_current_time_for_hls = function() {
      var current_time, day_time;
      if (this.collection_type === "channels") {
        current_time = this.app.user.current_time_in_string_format();
      } else {
        day_time = (this.collection.get_current().get_beginning() + this.detect_current_play_time() + this.app.user.get_timezone()) % 86400;
        current_time = this.time_in_string_format(day_time);
      }
      return current_time;
    };

    Video.prototype.detect_current_play_time = function() {
      var current_play_time;
      if (window.platform_name === 'lg') {
        current_play_time = this.player.get_timer();
      } else {
        current_play_time = this.player.get_current_play_time();
      }
      return current_play_time;
    };

    Video.prototype.time_in_string_format = function(time) {
      var hours, minutes;
      hours = Math.floor(parseFloat(time / 3600));
      minutes = Math.floor(parseFloat(time - hours * 3600) / 60);
      hours = this.add_zero_before_number(hours);
      minutes = this.add_zero_before_number(minutes);
      return "" + hours + ":" + minutes;
    };

    Video.prototype.add_zero_before_number = function(number) {
      if (number < 10) {
        number = '0' + number.toString();
      }
      return number;
    };

    Video.prototype.destroy = function() {
      if (this.collection_type === 'queue') {
        clearInterval(this.update_queue_status_interval);
        this.update_queue_status();
      }
      this.player.destroy();
      this.collection.off('current_changed', this.current_changed_handler, this);
      return Video.__super__.destroy.call(this);
    };

    return Video;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Player.Title = (function(_super) {

    __extends(Title, _super);

    function Title(app, collection, collection_type) {
      this.app = app;
      this.collection = collection;
      this.collection_type = collection_type;
      Title.__super__.constructor.call(this, this.app.platform);
    }

    Title.prototype.render = function() {
      (this.el = this.el_factory()).set_classname('player-title').append((this.el_content = this.el_factory()).set_classname('player-title__content'));
      this.collection.on('current_changed', this.current_changed_handler, this);
      return this;
    };

    Title.prototype.current_changed_handler = function() {
      return this.set_content(this.detect_content());
    };

    Title.prototype.set_collection = function(collection) {
      this.collection.off('current_changed', this.current_changed_handler, this);
      this.collection = collection;
      this.collection_type = 'programms';
      return this.collection.on('current_changed', this.current_changed_handler, this);
    };

    Title.prototype.detect_content = function() {
      var current_video, time;
      current_video = this.collection.get_current();
      if (this.collection.get_current().channel_name != null) {
        time = "" + (this.app.user.current_time_in_string_format()) + " ";
      } else {
        if (current_video.get_beginning_label != null) {
          time = "" + (current_video.get_beginning_label()) + " ";
        } else {
          time = '';
        }
      }
      return "" + time + (current_video.get_title());
    };

    Title.prototype.set_content = function(content) {
      return this.el_content.set_content(content);
    };

    Title.prototype.destroy = function() {
      this.collection.off('current_changed', this.current_changed_handler, this);
      return this;
    };

    return Title;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Player.Panels = (function(_super) {

    __extends(Panels, _super);

    function Panels(app, collection, collection_type) {
      this.app = app;
      this.collection = collection;
      this.collection_type = collection_type;
      Panels.__super__.constructor.call(this, this.app.platform);
    }

    Panels.prototype.render = function() {
      var _this = this;
      (this.el = this.el_factory()).set_classname('panels').append((this.title = new Rutv.View.Component.Player.Title(this.app, this.collection, this.collection_type)).render().el).append((this.controls = new Rutv.View.Component.Player.Controls(this.app, this.collection)).on('play', function() {
        return _this.trigger('play');
      }).on('pause', function() {
        return _this.trigger('pause');
      }).on('back', function() {
        return _this.trigger('back');
      }).on('exit', function() {
        return _this.trigger('exit');
      }).on('move_previous', function() {
        return _this.trigger('move_previous');
      }).on('move_next', function() {
        return _this.trigger('move_next');
      }).on('toggle_ratio', function() {
        return _this.trigger('toggle_ratio');
      }).on('rewind', function() {
        return _this.trigger('rewind');
      }).on('forward', function() {
        return _this.trigger('forward');
      }).render().el);
      return this.append_volume_control();
    };

    Panels.prototype.append_volume_control = function() {
      var _this = this;
      if (this.app.platform.volume_control != null) {
        this.el.append((this.volume_control = this.app.platform.volume_control).render().hide().el);
        this.controls.on('volume_up', function() {
          return _this.volume_control.quieter();
        }).on('volume_down', function() {
          return _this.volume_control.louder();
        }).on('toggle_mute', function() {
          return _this.volume_control.toggle_mute();
        });
      }
      return this;
    };

    Panels.prototype.show = function() {
      var _this = this;
      Panels.__super__.show.call(this);
      this.controls.show();
      clearTimeout(this.show_pannels_timeout);
      this.show_pannels_timeout = setTimeout(function() {
        return _this.hide();
      }, this.app.config.view.window.hide_player_panels_after);
      return this;
    };

    Panels.prototype.hide = function() {
      Panels.__super__.hide.call(this);
      this.showing = false;
      if (this.volume_control != null) {
        this.volume_control.hide();
      }
      this.controls.hide();
      return this;
    };

    Panels.prototype.destroy = function() {
      this.title.destroy();
      this.controls.destroy();
      Panels.__super__.destroy.call(this);
      return this;
    };

    Panels.prototype.blur = function() {
      this.controls.blur();
      return this;
    };

    Panels.prototype.focus = function() {
      this.controls.focus();
      return this;
    };

    return Panels;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Player.InfoDisplay = (function(_super) {

    __extends(InfoDisplay, _super);

    function InfoDisplay(app) {
      this.app = app;
      InfoDisplay.__super__.constructor.call(this, this.app.platform);
    }

    InfoDisplay.prototype.render = function() {
      this.el = this.el_factory();
      this.el.set_classname('player__info__display');
      return this;
    };

    InfoDisplay.prototype.set_content = function(content) {
      this.el.set_content(content);
      if (content !== '') {
        return this.show();
      }
    };

    InfoDisplay.prototype.show = function() {
      var _this = this;
      InfoDisplay.__super__.show.call(this);
      if (this.show_timeout != null) {
        clearTimeout(this.show_timeout);
      }
      return this.show_timeout = setTimeout(function() {
        return _this.hide();
      }, this.app.config.view.window.hide_player_panels_after);
    };

    return InfoDisplay;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Scene.Base = (function(_super) {

    __extends(Base, _super);

    function Base(app, resources) {
      this.app = app;
      this.resources = resources != null ? resources : {};
      Base.__super__.constructor.call(this, this.app.platform);
    }

    Base.prototype.render = function() {
      this.el = this.el_factory();
      if (this.template_name != null) {
        this.el.set_content(this.template(), this.template_data());
      }
      return this;
    };

    Base.prototype.attach_refresh_handler = function() {
      return this.key_trap.on('key_press', this.refresh_handler, this);
    };

    Base.prototype.refresh_handler = function(key) {
      if (key === Rutv.Key.Refresh) {
        return window.location.reload(true);
      }
    };

    Base.prototype.template_data = function() {
      return this.resources;
    };

    /**
    Calls at the first stage of push current scene to top
    */


    Base.prototype.show = function() {
      return Base.__super__.show.call(this);
    };

    /**
    Calls at the second stage of push another scene at top of current
    */


    Base.prototype.hide = function() {
      return Base.__super__.hide.call(this);
    };

    /**
    Calls at the second stage of push current scene to top
    */


    Base.prototype.focus = function() {
      return this;
    };

    /**
    Calls at the first stage of push another scene at top of current
    */


    Base.prototype.blur = function() {
      return this;
    };

    /**
    Calls for scene pop -- when its completely unneeded
    */


    Base.prototype.destroy = function() {
      this.el.destroy();
      return this;
    };

    Base.prototype.exit = function() {
      var _this = this;
      if (window.platform_name === 'samsung' && this.app.platform.get_platform_year() === '2011') {
        return this.app.platform.exit();
      } else {
        return this.app.scene.modals.show((new Rutv.View.Modal.ExitConfirmation(this.app)).on('confirm', function() {
          return _this.app.terminate();
        }));
      }
    };

    return Base;

  })(Rutv.View.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.App.Loading = (function(_super) {

    __extends(Loading, _super);

    function Loading(app) {
      this.app = app;
      Loading.__super__.constructor.call(this, this.app.platform);
    }

    Loading.prototype.render = function() {
      (this.el = this.el_factory()).set_classname('loading-indicator').set_content(this.app.resources.loading);
      this.app.server.on('active_requests_count_changed', this.active_requests_count_changed_handler, this);
      this.active_requests_count_changed_handler();
      return this;
    };

    Loading.prototype.active_requests_count_changed_handler = function() {
      if (this.app.server.get_active_requests_count() === 0) {
        return this.hide();
      } else {
        return this.show();
      }
    };

    return Loading;

  })(Rutv.View.Component.Base);

}).call(this);
(function() {

  Rutv.View.Component.App.Modals = (function() {

    function Modals(app) {
      this.app = app;
      this.modal_container_el = this.app.scene.el.find('modal__container');
      this.modals = [];
      this.showing = false;
    }

    Modals.prototype.show = function(modal) {
      this.modals.push(modal);
      if (!this.showing) {
        return this.show_next();
      }
    };

    Modals.prototype.show_next = function() {
      var modal, _ref;
      this.showing = true;
      modal = this.modals.shift();
      if ((_ref = this.get_parent_scene()) != null) {
        _ref.blur();
      }
      this.modal_container_el.append(modal.on('back', this.modal_back_handler, this).render().el).show();
      return modal.focus();
    };

    Modals.prototype.modal_back_handler = function(modal) {
      var _ref,
        _this = this;
      modal.hide().destroy();
      this.modal_container_el.hide();
      if (this.modals.length > 0) {
        return setTimeout(function() {
          return _this.show_next();
        }, 500);
      } else {
        this.showing = false;
        return (_ref = this.get_parent_scene()) != null ? _ref.focus() : void 0;
      }
    };

    Modals.prototype.get_parent_scene = function() {
      return this.app.scene.subscenes.get_current();
    };

    return Modals;

  })();

}).call(this);
(function() {

  Rutv.View.Component.App.Subscenes = (function() {

    function Subscenes(app) {
      this.app = app;
      this.scene_container_el = this.app.scene.el.find('window__container');
      this.subscenes = [];
      this.current_scene = null;
    }

    Subscenes.prototype.get_current = function() {
      return this.subscenes[this.subscenes.length - 1];
    };

    Subscenes.prototype.push = function(scene) {
      var _ref;
      if ((_ref = this.current_scene) != null) {
        _ref.blur().hide();
      }
      this.subscenes.push(this.current_scene = scene);
      this.scene_container_el.append(this.current_scene.render().el);
      return this.current_scene.show().focus();
    };

    Subscenes.prototype.pop = function() {
      var _ref;
      if ((_ref = this.current_scene) != null) {
        _ref.destroy();
      }
      this.subscenes.pop();
      if (this.subscenes.length > 0) {
        return (this.current_scene = this.subscenes[this.subscenes.length - 1]).show().focus();
      } else {
        return this.current_scene = null;
      }
    };

    Subscenes.prototype.replace = function(scene) {
      this.pop();
      return this.push(scene);
    };

    return Subscenes;

  })();

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.App.ChannelByNumberSelector = (function(_super) {

    __extends(ChannelByNumberSelector, _super);

    ChannelByNumberSelector.prototype.template_name = 'channel_by_number_selector';

    function ChannelByNumberSelector(app) {
      this.app = app;
      ChannelByNumberSelector.__super__.constructor.call(this, this.app.platform);
      this.showing = false;
      this;

    }

    ChannelByNumberSelector.prototype.enable = function(channels) {
      this.channels = channels;
      return this;
    };

    ChannelByNumberSelector.prototype.enabled = function() {
      return this.channels != null;
    };

    ChannelByNumberSelector.prototype.render = function(main_page) {
      this.main_page = main_page;
      (this.el = this.el_factory()).set_classname('channel-by-number-selector').hide();
      this.key_trap.on('key_press', this.keypress_handler, this).on('after_key_press', this.after_keypress_handler, this);
      this.clear();
      return this;
    };

    ChannelByNumberSelector.prototype.clear = function() {
      return this.numbers = [];
    };

    ChannelByNumberSelector.prototype.show = function() {
      var _this = this;
      ChannelByNumberSelector.__super__.show.call(this);
      this.showing = true;
      clearTimeout(this.complete_timeout_id);
      this.complete_timeout_id = setTimeout(function() {
        return _this.complete();
      }, 5000);
      return this;
    };

    ChannelByNumberSelector.prototype.hide = function() {
      ChannelByNumberSelector.__super__.hide.call(this);
      this.showing = false;
      clearTimeout(this.complete_timeout_id);
      this.clear();
      this.update_content();
      return this;
    };

    ChannelByNumberSelector.prototype.process_keypress = function(key) {
      switch (key) {
        case Rutv.Key.Button1:
          return this.add(1);
        case Rutv.Key.Button2:
          return this.add(2);
        case Rutv.Key.Button3:
          return this.add(3);
        case Rutv.Key.Button4:
          return this.add(4);
        case Rutv.Key.Button5:
          return this.add(5);
        case Rutv.Key.Button6:
          return this.add(6);
        case Rutv.Key.Button7:
          return this.add(7);
        case Rutv.Key.Button8:
          return this.add(8);
        case Rutv.Key.Button9:
          return this.add(9);
        case Rutv.Key.Button0:
          return this.add_zero();
        case Rutv.Key.Back:
          return this.cancel();
        case Rutv.Key.Ok:
          return this.complete();
      }
    };

    ChannelByNumberSelector.prototype.cancel = function() {
      if (this.showing) {
        this.hide();
        return false;
      }
    };

    ChannelByNumberSelector.prototype.complete = function() {
      var channel;
      if (this.showing) {
        channel = this.channels.get_by_button(this.get_number());
        if (channel != null) {
          this.trigger('channel_selected', channel);
        }
        this.hide();
        return false;
      }
    };

    ChannelByNumberSelector.prototype.after_keypress_handler = function(key) {
      if (this.enabled()) {
        return this.process_keypress(key);
      }
    };

    ChannelByNumberSelector.prototype.keypress_handler = function(key) {
      if (this.showing) {
        return this.process_keypress(key);
      }
    };

    ChannelByNumberSelector.prototype.add_zero = function() {
      if (this.numbers.length > 0) {
        return this.add(0);
      }
    };

    ChannelByNumberSelector.prototype.add = function(number) {
      if (!this.full()) {
        this.add_number(number);
      }
      this.show();
      return false;
    };

    ChannelByNumberSelector.prototype.add_number = function(number) {
      this.numbers.push(number);
      return this.update_content();
    };

    ChannelByNumberSelector.prototype.update_content = function() {
      var button, channel, _ref;
      button = this.get_number();
      channel = (_ref = this.channels) != null ? _ref.get_by_button(button) : void 0;
      return this.el.set_content(this.template(), this.app.resources.channel_by_number_selector(button, channel));
    };

    ChannelByNumberSelector.prototype.get_number = function() {
      var letters;
      letters = this.numbers.join('');
      if (letters.length > 0) {
        return parseInt(letters);
      } else {
        return letters;
      }
    };

    ChannelByNumberSelector.prototype.full = function() {
      return this.numbers.length > 4;
    };

    return ChannelByNumberSelector;

  })(Rutv.View.Component.Base);

  Rutv.Utils.extend(Rutv.View.Component.App.ChannelByNumberSelector.prototype, Rutv.Utils.Events);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Scene.App = (function(_super) {

    __extends(App, _super);

    App.prototype.template_name = 'app';

    function App(app) {
      App.__super__.constructor.call(this, app);
    }

    App.prototype.render = function() {
      App.__super__.render.call(this);
      this.platform.get_el().append(this.el.set_classname('layout'));
      this.subscenes = new Rutv.View.Component.App.Subscenes(this.app);
      this.modals = new Rutv.View.Component.App.Modals(this.app);
      this.el.append((new Rutv.View.Component.App.Loading(this.app)).render().el);
      return this;
    };

    App.prototype.create_number_input = function() {
      var channel_by_number_selector,
        _this = this;
      channel_by_number_selector = new Rutv.View.Component.App.ChannelByNumberSelector(this.app);
      this.app.user.get_channels().done(function(channels) {
        return channel_by_number_selector.on('channel_selected', function(channel) {
          return _this.subscenes.push(new Rutv.View.Scene.Player(_this.app, channel, channels)).play();
        }).enable(channels);
      });
      return channel_by_number_selector;
    };

    App.prototype.show = function() {
      App.__super__.show.call(this);
      this.login();
      return this;
    };

    App.prototype.login = function() {
      return this.app.auth_manager.on('login', this.token_login_handler, this).on('login_fail', this.show_login_scene, this).login();
    };

    App.prototype.show_home_scene = function() {
      this.el.append(this.create_number_input().render().el);
      return this.subscenes.replace(new Rutv.View.Scene.Home(this.app));
    };

    App.prototype.token_login_handler = function() {
      this.app.auth_manager.off('login', this.token_login_handler, this).off('login_fail', this.show_login_scene, this);
      return this.show_home_scene();
    };

    App.prototype.show_login_scene = function() {
      this.subscenes.push(new Rutv.View.Scene.Login(this.app));
      return this.app.auth_manager.off('login', this.token_login_handler, this).off('login_fail', this.show_login_scene, this).on('login', this.code_login_handler, this);
    };

    App.prototype.code_login_handler = function() {
      this.app.auth_manager.off('login', this.code_login_handler, this);
      return this.show_home_scene();
    };

    return App;

  })(Rutv.View.Scene.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Scene.Login = (function(_super) {

    __extends(Login, _super);

    Login.prototype.template_name = 'login';

    function Login(app) {
      Login.__super__.constructor.call(this, app, app.resources.login);
    }

    Login.prototype.render = function() {
      Login.__super__.render.call(this);
      this.el.set_classname('login');
      this.code_el = this.el.find('code');
      this.app.auth_manager.on('code', this.set_code, this).login_by_code();
      this.key_trap.on('key_press', this.keypress_handler, this);
      return this;
    };

    Login.prototype.set_code = function(new_code) {
      this.code_el.set_content(new_code);
      return this;
    };

    Login.prototype.destroy = function() {
      this.app.auth_manager.off('code', this.set_code, this);
      this.key_trap.off('key_press', this.keypress_handler, this);
      return Login.__super__.destroy.call(this);
    };

    Login.prototype.blur = function() {
      this.key_trap.off('key_press', this.keypress_handler, this);
      return this;
    };

    Login.prototype.keypress_handler = function(key) {
      switch (key) {
        case Rutv.Key.Back:
          this.exit();
          return false;
        case Rutv.Key.Exit:
          this.exit();
          return false;
      }
    };

    Login.prototype.exit = function() {
      var _this = this;
      return this.app.scene.modals.show((new Rutv.View.Modal.ExitConfirmation(this.app)).on('confirm', function() {
        return _this.app.terminate();
      }));
    };

    return Login;

  })(Rutv.View.Scene.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Scene.Home = (function(_super) {

    __extends(Home, _super);

    function Home(app) {
      Home.__super__.constructor.call(this, app);
      this.current_pane = null;
    }

    Home.prototype.render = function() {
      Home.__super__.render.call(this);
      this.el.set_class('home').append(this.create_menu().render().el).append(this.create_tv_pane().render().el).append(this.create_account_pane().render().hide().el).append(this.create_hints().render().el);
      this.menu.tv_button.select();
      return this;
    };

    Home.prototype.create_hints = function() {
      return this.hints = new Rutv.View.Component.Hints(this.app);
    };

    Home.prototype.create_menu = function() {
      var _this = this;
      return (this.menu = new Rutv.View.Component.Home.Menu.Base(this.app)).on('tv', function() {
        return _this.tv_pane.show().focus();
      }).on('account', function() {
        return _this.account_pane.show().focus();
      }).on('queue', function() {
        return _this.app.scene.subscenes.push(new Rutv.View.Scene.Queue(_this.app));
      }).on('focus', function() {
        var _ref;
        return (_ref = _this.current_pane) != null ? _ref.blur() : void 0;
      }).on('down', function() {
        return _this.current_pane.focus();
      }).on('back exit', function() {
        return _this.exit();
      });
    };

    Home.prototype.create_account_pane = function() {
      var _this = this;
      return (this.account_pane = this.pane_factory(Rutv.View.Component.Home.Account.Base)).on('back', function() {
        return _this.menu.tv_button.focus().select();
      }).on('exit', function() {
        return _this.exit();
      });
    };

    Home.prototype.create_tv_pane = function() {
      var _this = this;
      return (this.tv_pane = this.pane_factory(Rutv.View.Component.Home.Tv.Base)).on('back exit', function() {
        return _this.exit();
      });
    };

    Home.prototype.pane_factory = function(constructor) {
      var _this = this;
      return (new constructor(this.app)).on('before_show', function() {
        var _ref;
        return (_ref = _this.current_pane) != null ? _ref.hide() : void 0;
      }).on('show', function(pane) {
        return _this.current_pane = pane;
      }).on('focus', function() {
        return _this.menu.blur();
      }).on('up', function() {
        return _this.menu.focus();
      });
    };

    Home.prototype.focus = function() {
      var _ref;
      if ((_ref = this.current_pane) != null) {
        _ref.focus();
      }
      return this;
    };

    Home.prototype.blur = function() {
      var _ref, _ref1;
      if ((_ref = this.current_pane) != null) {
        _ref.blur();
      }
      if ((_ref1 = this.menu) != null) {
        _ref1.blur();
      }
      return this;
    };

    return Home;

  })(Rutv.View.Scene.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Scene.Player = (function(_super) {

    __extends(Player, _super);

    function Player(app, playable, collection) {
      this.app = app;
      this.playable = playable;
      this.collection = collection;
      this.collection_type = null;
      this.detect_collection_type();
      Player.__super__.constructor.call(this, this.app);
    }

    Player.prototype.render = function() {
      var _this = this;
      Player.__super__.render.call(this);
      this.el.set_classname('player').append((this.info_display = new Rutv.View.Component.Player.InfoDisplay(this.app)).render().hide().el).append((this.video_player = new Rutv.View.Component.Player.Video(this.app, this.collection, this.collection_type, this.info_display)).on('mouse_move', this.show_panels, this).on('schedule_load', function() {
        return _this.panels.title.set_collection(_this.collection.get_current().schedule);
      }).on('current_changed', function(duration) {
        return _this.update_band(duration);
      }).on('rewind forward', function() {
        return _this.band.move_mark();
      }).on('resume', function() {
        return _this.band.start_mark_move();
      }).on('pause', function() {
        return _this.band.stop_move();
      }).on('current_is_live_streaming', function() {
        return _this.hide_band();
      }).on('current_programm_content_changed', function(content) {
        var _ref, _ref1;
        return (_ref = _this.panels) != null ? (_ref1 = _ref.title) != null ? _ref1.set_content(content) : void 0 : void 0;
      }).render().el).append((this.panels = new Rutv.View.Component.Player.Panels(this.app, this.collection, this.collection_type)).on('back', function() {
        return _this.app.scene.subscenes.pop();
      }).on('exit', function() {
        return _this.exit();
      }).on('play', function() {
        return _this.video_player.resume();
      }).on('pause', function() {
        return _this.video_player.pause();
      }).on('move_previous', function() {
        return _this.video_player.move_previous();
      }).on('move_next', function() {
        return _this.video_player.move_next();
      }).on('toggle_ratio', function() {
        return _this.video_player.toggle_ratio();
      }).on('rewind', function() {
        return _this.video_player.rewind();
      }).on('forward', function() {
        return _this.video_player.forward();
      }).on('show', function() {
        return _this.video_player.current_title_content_for_hls();
      }).render().el);
      this.panels.controls.el.append(this.create_band().render().el);
      this.key_trap.on('after_key_press', this.show_panels, this);
      return this;
    };

    Player.prototype.create_band = function() {
      var _this = this;
      return (this.band = new Rutv.View.Component.Player.Band(this.app, this.video_player)).on('finished', function() {
        return _this.panels.show();
      });
    };

    Player.prototype.update_band = function(duration) {
      this.panels.controls.use_with_rewind_buttons();
      this.band.set_duration(duration);
      return this.band.show();
    };

    Player.prototype.hide_band = function() {
      this.panels.controls.use_without_rewind_buttons();
      return this.band.hide();
    };

    Player.prototype.show_panels = function() {
      this.panels.show();
      return false;
    };

    Player.prototype.destroy = function() {
      this.key_trap.off('after_key_press', this.show_panels, this);
      this.video_player.off('mouse_move', this.show_panels, this);
      this.video_player.destroy();
      this.panels.destroy();
      this.band.destroy();
      return Player.__super__.destroy.call(this);
    };

    Player.prototype.detect_collection_type = function() {
      if (this.playable.get_channel != null) {
        this.collection_type = 'programms';
        this.collection.set_current(this.playable.get_channel());
        return this.collection = this.collection.get_current().schedule;
      } else if (this.playable.get_schedule != null) {
        return this.collection_type = 'channels';
      } else {
        return this.collection_type = 'queue';
      }
    };

    Player.prototype.play = function() {
      this.collection.set_current(this.playable);
      if (this.collection_type === 'channels') {
        this.video_player.load_schedule();
      }
      return this;
    };

    Player.prototype.focus = function() {
      Player.__super__.focus.call(this);
      this.panels.focus();
      return this;
    };

    Player.prototype.blur = function() {
      this.panels.blur();
      this.band.destroy();
      return this;
    };

    return Player;

  })(Rutv.View.Scene.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Scene.Channels.Base = (function(_super) {

    __extends(Base, _super);

    function Base(app) {
      Base.__super__.constructor.call(this, app);
      this.channels_deferred = this.channels_deffered_factory();
    }

    Base.prototype.render = function() {
      Base.__super__.render.call(this);
      this.el.append((this.header = this.header_factory()).render().el).append((this.list = this.list_factory()).render().el).append(this.create_hints().render().el);
      return this;
    };

    Base.prototype.create_hints = function() {
      return this.hints = new Rutv.View.Component.Hints(this.app, [[Rutv.Key.Back, this.app.resources.hints.back, this.list]]);
    };

    Base.prototype.focus = function() {
      this.header.focus();
      this.list.focus();
      return this;
    };

    Base.prototype.blur = function() {
      this.header.blur();
      this.list.blur();
      return this;
    };

    Base.prototype.channels_deffered_factory = function() {
      throw "should be implemented in subclass";
    };

    Base.prototype.header_factory = function() {
      throw "should be implemented in subclass";
    };

    Base.prototype.list_factory = function() {
      var _this = this;
      return (new Rutv.View.Component.Channels.List(this.app, this.channels_deferred, this.resources.list)).on('back', function() {
        return _this.app.scene.subscenes.pop();
      }).on('exit', function() {
        return _this.exit();
      }).on('schedule', function() {
        return _this.app.scene.subscenes.push(new Rutv.View.Scene.Schedule(_this.app, _this.list.channels));
      });
    };

    Base.prototype.destroy = function() {
      this.list.destroy();
      this.header.destroy();
      return Base.__super__.destroy.call(this);
    };

    return Base;

  })(Rutv.View.Scene.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Scene.Channels.All = (function(_super) {

    __extends(All, _super);

    function All(app) {
      All.__super__.constructor.call(this, app);
      this.resources = this.app.resources.channels.all;
    }

    All.prototype.channels_deffered_factory = function() {
      return this.app.user.get_channels();
    };

    All.prototype.header_factory = function() {
      return new Rutv.View.Component.Channels.Header(this.app, this.channels_deferred, this.resources.heading, 'header_all-channels');
    };

    return All;

  })(Rutv.View.Scene.Channels.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Scene.Channels.Favourited = (function(_super) {

    __extends(Favourited, _super);

    function Favourited(app) {
      Favourited.__super__.constructor.call(this, app);
      this.resources = this.app.resources.channels.favourited;
    }

    Favourited.prototype.channels_deffered_factory = function() {
      return this.app.user.get_favourited_channels();
    };

    Favourited.prototype.header_factory = function() {
      return new Rutv.View.Component.Channels.Header(this.app, this.channels_deferred, this.resources.heading, 'header_favourited-channels');
    };

    return Favourited;

  })(Rutv.View.Scene.Channels.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Scene.Schedule = (function(_super) {

    __extends(Schedule, _super);

    function Schedule(app, channels_data) {
      this.app = app;
      this.channels_data = channels_data;
      Schedule.__super__.constructor.call(this, this.app);
    }

    Schedule.prototype.render = function() {
      Schedule.__super__.render.call(this);
      this.el.append(this.create_about_programm().render().el).append(this.create_hints().render().el).append(this.create_channels().render().el);
      this.key_trap.on('key_press', this.keypress_handler, this);
      return this;
    };

    Schedule.prototype.create_about_programm = function() {
      return this.about_programm = new Rutv.View.Component.Schedule.AboutProgramm(this.app);
    };

    Schedule.prototype.create_channels = function() {
      return (this.channels = new Rutv.View.Component.Schedule.Channels(this.app, this.channels_data)).on('focus_programm', this.focus_programm_handler, this).on('press_programm', this.press_programm_handler, this);
    };

    Schedule.prototype.focus_programm_handler = function(programm_component) {
      return this.about_programm.set_programm(programm_component.get_programm());
    };

    Schedule.prototype.press_programm_handler = function(channel_component, programm_component) {
      if (programm_component.programm.get_available()) {
        return this.app.scene.subscenes.push(new Rutv.View.Scene.Player(this.app, programm_component.get_programm(), this.channels.channels)).play();
      }
    };

    Schedule.prototype.create_hints = function() {
      return this.hints = new Rutv.View.Component.Hints(this.app, [[Rutv.Key.Back, this.app.resources.hints.back, this]]);
    };

    Schedule.prototype.keypress_handler = function(key) {
      switch (key) {
        case Rutv.Key.Back:
          this.app.scene.subscenes.pop();
          return false;
        case Rutv.Key.Exit:
          this.exit();
          return false;
      }
    };

    Schedule.prototype.destroy = function() {
      this.key_trap.off('key_press', this.keypress_handler, this);
      this.channels.destroy();
      return Schedule.__super__.destroy.call(this);
    };

    Schedule.prototype.blur = function() {
      if (this.channels != null) {
        this.channels.blur();
      }
      return this;
    };

    Schedule.prototype.focus = function() {
      if (this.channels != null) {
        this.channels.focus();
      }
      return this;
    };

    return Schedule;

  })(Rutv.View.Scene.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Scene.Queue = (function(_super) {

    __extends(Queue, _super);

    function Queue(app) {
      Queue.__super__.constructor.call(this, app);
      this.resources = this.app.resources.queue;
      this.queue_deferred = this.queue_deferred_factory();
    }

    Queue.prototype.render = function() {
      Queue.__super__.render.call(this);
      this.el.append((this.header = this.header_factory()).render().el).append((this.list = this.list_factory()).render().el).append(this.create_hints().render().el);
      return this;
    };

    Queue.prototype.create_hints = function() {
      return this.hints = new Rutv.View.Component.Hints(this.app, [[Rutv.Key.Back, this.app.resources.hints.back, this.list]]);
    };

    Queue.prototype.focus = function() {
      this.header.focus();
      this.list.focus();
      return this;
    };

    Queue.prototype.blur = function() {
      this.header.blur();
      this.list.blur();
      return this;
    };

    Queue.prototype.queue_deferred_factory = function() {
      return this.app.user.get_queue();
    };

    Queue.prototype.header_factory = function() {
      return new Rutv.View.Component.Queue.Header(this.app, this.queue_deferred, this.resources.heading, 'header_all-channels');
    };

    Queue.prototype.list_factory = function() {
      var _this = this;
      return (new Rutv.View.Component.Queue.List(this.app, this.queue_deferred, this.resources.list)).on('back', function() {
        return _this.app.scene.subscenes.pop();
      }).on('exit', function() {
        return _this.exit();
      }).on('schedule', function() {
        return _this.app.scene.subscenes.push(new Rutv.View.Scene.Schedule(_this.app, _this.list.programms));
      });
    };

    Queue.prototype.destroy = function() {
      this.list.destroy();
      this.header.destroy();
      return Queue.__super__.destroy.call(this);
    };

    return Queue;

  })(Rutv.View.Scene.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Component.Modal.Button = (function(_super) {

    __extends(Button, _super);

    function Button(app, label) {
      this.app = app;
      this.label = label;
      Button.__super__.constructor.call(this, this.app.platform);
    }

    Button.prototype.render = function() {
      Button.__super__.render.call(this);
      this.el.set_classname('modal__button').set_content(this.label);
      return this;
    };

    return Button;

  })(Rutv.View.Component.Button);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Modal.Base = (function(_super) {

    __extends(Base, _super);

    Base.prototype.template_name = 'modal';

    function Base(app, resources, additional_class_name) {
      this.app = app;
      this.resources = resources != null ? resources : {};
      this.additional_class_name = additional_class_name != null ? additional_class_name : '';
      Base.__super__.constructor.call(this, this.app.platform);
    }

    Base.prototype.render = function() {
      (this.el = this.el_factory()).set_classname("modal " + this.additional_class_name).set_content(this.template(), this.template_data());
      this.create_buttons();
      return this;
    };

    Base.prototype.template_data = function() {
      return this.resources;
    };

    Base.prototype.create_buttons = function() {
      var button_container_el, index, label, _i, _len, _ref;
      button_container_el = this.el.find('buttons');
      this.buttons = [];
      _ref = this.resources.buttons;
      for (index = _i = 0, _len = _ref.length; _i < _len; index = ++_i) {
        label = _ref[index];
        this.buttons.push(this.button_factory(label, index, button_container_el));
      }
      button_container_el.add_class("modal__buttons_" + this.buttons.length);
      if (this.buttons.length !== 1) {
        return this.attach_buttons_navigation_handers();
      }
    };

    Base.prototype.attach_buttons_navigation_handers = function() {
      var button, index, _i, _len, _ref, _results;
      _ref = this.buttons;
      _results = [];
      for (index = _i = 0, _len = _ref.length; _i < _len; index = ++_i) {
        button = _ref[index];
        _results.push(this.attach_button_navigation_headers(button, index));
      }
      return _results;
    };

    Base.prototype.attach_button_navigation_headers = function(button, index) {
      var max_index,
        _this = this;
      max_index = this.buttons.length - 1;
      if (index === max_index) {
        button.on('left', function() {
          return _this.buttons[0].focus();
        });
      } else {
        button.on('left', function() {
          return _this.buttons[index + 1].focus();
        });
      }
      if (index === 0) {
        return button.on('right', function() {
          return _this.buttons[max_index].focus();
        });
      } else {
        return button.on('right', function() {
          return _this.buttons[index - 1].focus();
        });
      }
    };

    Base.prototype.button_factory = function(label, index, button_container_el) {
      var button,
        _this = this;
      button_container_el.append((button = new Rutv.View.Component.Modal.Button(this.app, label)).on('press', function() {
        return _this.trigger("button_" + index + "_press");
      }).on('before_focus', this.blur_focused, this).on('focus', this.button_focus_handler, this).on('back', function() {
        return _this.trigger('back', _this);
      }).render().el);
      return button;
    };

    Base.prototype.set_focused_button = function(button) {
      var _ref;
      if ((_ref = this.focused_button) != null) {
        _ref.blur();
      }
      return (this.focused_button = button).focus();
    };

    Base.prototype.blur_focused = function() {
      var _ref;
      return (_ref = this.focused_button) != null ? _ref.blur() : void 0;
    };

    Base.prototype.button_focus_handler = function(button) {
      this.focused_button = button;
      return this.trigger('focus');
    };

    Base.prototype.destroy = function() {
      var button, _i, _len, _ref;
      _ref = this.buttons;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        button = _ref[_i];
        button.destroy();
      }
      return Base.__super__.destroy.call(this);
    };

    Base.prototype.focus = function() {
      var _ref;
      if ((_ref = this.buttons[0]) != null) {
        _ref.focus();
      }
      return this;
    };

    return Base;

  })(Rutv.View.Base);

  Rutv.Utils.extend(Rutv.View.Modal.Base.prototype, Rutv.Utils.Events);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Modal.ExitConfirmation = (function(_super) {

    __extends(ExitConfirmation, _super);

    function ExitConfirmation(app) {
      var _this = this;
      ExitConfirmation.__super__.constructor.call(this, app, app.resources.modal.exit_confirmation);
      this.on('button_0_press', function() {
        return _this.trigger('confirm', _this);
      });
      this.on('button_0_press button_1_press', function() {
        return _this.trigger('back', _this);
      });
    }

    return ExitConfirmation;

  })(Rutv.View.Modal.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Modal.Error.NoStreamingUrl = (function(_super) {

    __extends(NoStreamingUrl, _super);

    function NoStreamingUrl(app) {
      var _this = this;
      NoStreamingUrl.__super__.constructor.call(this, app, app.resources.modal.error.no_streaming_url);
      this.on('button_0_press', function() {
        return _this.trigger('back', _this);
      });
    }

    return NoStreamingUrl;

  })(Rutv.View.Modal.Base);

}).call(this);
(function() {
  var __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rutv.View.Modal.Error.RequestFailed = (function(_super) {

    __extends(RequestFailed, _super);

    function RequestFailed(app) {
      var _this = this;
      RequestFailed.__super__.constructor.call(this, app, app.resources.modal.error.request_failed);
      this.on('button_0_press', function() {
        return _this.trigger('retry', _this);
      });
      this.on('button_1_press', function() {
        return _this.trigger('ok', _this);
      });
      this.on('button_0_press button_1_press', function() {
        return _this.trigger('back', _this);
      });
    }

    return RequestFailed;

  })(Rutv.View.Modal.Base);

}).call(this);

/**
* @author Alexandr Borisov (ab@cifronomika.ru)
*/


(function() {

  Rutv.Application = (function() {
    /**
    * Application facade
    * @name Rutv.Application
    * @constructor
    * @param config Object with config variables
    * @param platform Platform class, to isolate specific things
    */

    function Application(config, platform) {
      this.config = config;
      this.platform = platform;
      this.resources = Rutv.Locale[this.config.language];
      this.server = new Rutv.Server.Base(this);
      this.user = new Rutv.Model.User(this);
      this.auth_manager = new Rutv.Manager.Auth(this.config, this.server, this);
    }

    /**
    * starts application
    */


    Application.prototype.run = function() {
      (this.scene = new Rutv.View.Scene.App(this)).render().show();
      return this;
    };

    Application.prototype.terminate = function() {
      this.platform.exit();
      return this;
    };

    return Application;

  })();

}).call(this);
Rutv.VERSION = 'local';
(function() {
  var app, platform;

  platform = (new Rutv.Platform.Detector(config)).detect();

  (app = new Rutv.Application(config, platform)).run();

  window.app = app;

  if (Rutv.VERSION != null) {
    if (typeof console !== "undefined" && console !== null) {
      if (typeof console.info === "function") {
        console.info("Rutv app ver. " + Rutv.VERSION);
      }
    }
  }

}).call(this);
